# Controls AnimeGenre pages.
class AnimeGenreController < ApplicationController
  before_filter :login_required, :except => [:index, :list]
  after_filter :store_location, :only => [:index, :list, :edit, :manage]
  
  require_role 'anime_genre_moderator', :for_all_except => [:index, :list]
  
  in_place_edit_with_validation_for :anime_genre, :name
  in_place_edit_with_validation_for :anime_genre, :description
  
  # Renders a list of AnimeGenres.
  def index
    list
    render :action => :list
  end

  # Render a list of AnimeGenres.
  def list
    @anime_genres = AnimeGenre.paginate :all, :page => params[:page], :order => 'name ASC'
    @title = 'Genericon Trivia System - Anime Genres List'
  end

  # Render the screen to insert a new AnimeGenre.
  def new
    # We reference flash[] in case we've been redirected following an error.
    @anime_genre = AnimeGenre.new flash[:anime_genre]
    @title = 'Genericon Trivia System - New Anime Genre'
  end

  # Create a new AnimeGenre and redirect back to new.
  def create
	# Create the AnimeGenre.
    @anime_genre = AnimeGenre.new params[:anime_genre]
    
    # Save and redirect.
    if @anime_genre.save
      flash[:notice] = "Successfully submitted your anime genre."
    else
      # Render the errors, and retain the values input previously for next time.
      flash[:error] = render_to_string :inline => "<%= error_messages_for 'anime_genre' %>"
      flash[:error_is_html] = true
      flash[:anime_genre] = params[:anime_genre]
    end
    
    # Redirect back to the new path.
    redirect_to new_anime_genre_path
  end

  # Shows the edit page for an AnimeGenre.
  def edit
    @anime_genre = AnimeGenre.find params[:id]
    @title = "Genericon Trivia System - Edit Anime Genre - #{@anime_genre.name}"
  end
  
  # Destroys an AnimeGenre.
  def destroy
    # I sure hope you added a warning.
    
    if AnimeGenre.destroy params[:id]
      flash[:notice] = "Successfully deleted the anime genre."
    else
      flash[:error] = "There was a problem deleting that anime genre."
    end
    
    redirect_to manage_anime_genre_path
  end

  # Shows the list of AnimeGenres to be managed.
  def manage
    # TODO: Here, we order by whether or not the genre is approved.
    @anime_genres = AnimeGenre.paginate :all, :page => params[:page], :order => 'name ASC'
    @title = 'Genericon Trivia System - Anime Genre Management'
  end
  
  # Select the destination for a merge_into operation.
  def select_merge
    @anime_genre = AnimeGenre.find(params[:id])
    @title = 'Genericon Trivia System - Merge Anime Genres'
  end
  
  # Merge the given AnimeGenre into the destination AnimeGenre.
  def merge_into
    @src = AnimeGenre.find(params[:id])
    @dest = AnimeGenre.find(params[:dest])
    
    if @src.merge_into! @dest
      flash[:notice] = "Successfully merged the anime genres."
    else
      flash[:error] = "Could not merge the anime genres."
      flash[:error_is_html] = false
    end
	
    redirect_to manage_anime_genre_path
  end
  
  # Return the list of anime genres to merge into.
  def select_merge_search
    find_options = {
      :conditions => [ "LOWER(name) LIKE ?", '%' + params[:name].downcase + '%' ],
      :order => "name ASC",
      :limit => 10 }
    
    @main = AnimeGenre.find(params[:id])
    @items = AnimeGenre.find(:all, find_options)
    
    if params[:name].empty?
      merge_list = ''
    else
      merge_list = render_to_string :partial => 'select_merge_search'
    end
    
    render :update do |page|
      AnimeGenre.query_cache.clear_query_cache if AnimeGenre.method_defined? :query_cache
      page.replace_html("merge_options", merge_list)
    end
  end

end
