require 'set'

# Controls AnimeTriviaQuestion pages.
class AnimeTriviaQuestionController < ApplicationController
  before_filter :login_required, :except => [:index, :list, :show, :new,
                                             :create,
                                             :auto_complete_for_anime_series_name,
                                             :find_anime_genres_like,
                                             :find_similar_genres_to_anime,
                                             :play, :new_game,
                                             :randomize_play_window, :next_question,
					     :remove_anime_series_from_game,
					     :set_max_from_series]
  before_filter :clear_unused, :except => [:play, :randomize_play_window,
                                           :next_question,
					   :remove_anime_series_from_game,
					   :set_max_from_series]
  after_filter :store_location, :only => [:index, :list, :show, :new, :edit,
                                          :manage, :play, :new_game]
  
  require_role 'anime_trivia_approver', :for_all_except => [:index, :list,
                                                            :show, :new,
                                                            :create,
                                                            :auto_complete_for_anime_series_name,
                                                            :find_anime_genres_like,
                                                            :find_similar_genres_to_anime,
                                                            :play, :new_game,
                                                            :randomize_play_window,
                                                            :next_question,
					  		    :remove_anime_series_from_game,
							    :set_max_from_series]
  require_role 'anime_trivia_moderator', :for => [:destroy]
  
  in_place_edit_with_validation_for :anime_trivia_question, :question
  in_place_edit_with_validation_for :anime_trivia_question, :answer
  
  # Cleans the unused questions on exit.
  def clear_unused
    GameSession.clean_sessions(session[:unused]) if session[:unused]
    session[:unused] = nil
  end

  # Custom validation for setting anime series name.
  def set_anime_series_name
    @item = AnimeTriviaQuestion.find(params[:id])
    @anime_series = AnimeSeries.find_by_name params[:anime_series][:name]
    if @anime_series != nil
      @item.anime_series = @anime_series
      @item.save
      render :update do |page| 
        page.replace_html("anime_series_name_#{params[:id]}_in_place_editor", 
                          @item.anime_series.name)
      end
    else
      render :update do |page|
        page.alert('Must have a pre-existing anime set.')
        AnimeTriviaQuestion.query_cache.clear_query_cache if AnimeTriviaQuestion.method_defined?:query_cache
        AnimeSeries.query_cache.clear_query_cache if AnimeSeries.method_defined?:query_cache
        @item.reload
        page.replace_html("anime_series_name_#{params[:id]}_in_place_editor", 
                          @item.anime_series.name)
      end
    end
  end
  
  # Custom validation for setting question type.
  def set_anime_trivia_question_question_type
    @item = AnimeTriviaQuestion.find(params[:id])
    @question_type = params[:value]
    @item.question_type = @question_type
    if @item.save
      head :ok
    else
      head :bad_request
    end
  end
  
  # Renders a new AnimeTriviaQuestion acceptance page.
  def index
    new
    render :action => :new
  end

  # Render a list of AnimeTriviaQuestions.
  def list
    if !params[:letter].nil?
      params[:page] = (AnimeTriviaQuestion.count(:conditions => ["LOWER(`anime_series`.name) < LOWER(?)", params[:letter]], :joins => 'INNER JOIN `anime_series` ON `anime_trivia_questions`.anime_series_id=`anime_series`.id') / AnimeTriviaQuestion.per_page + 1).to_i
    end
    params.delete :letter
    @anime_trivia_questions = AnimeTriviaQuestion.paginate :all, :page => params[:page], :select => '`anime_trivia_questions`.*, `anime_series`.name', :joins => 'INNER JOIN `anime_series` ON `anime_trivia_questions`.anime_series_id=`anime_series`.id', :order => '`anime_series`.name ASC, `anime_trivia_questions`.created_at ASC, `anime_trivia_questions`.id ASC'
    @title = 'Genericon Trivia System - Anime Trivia Question List'
  end

  # Render the details of an AnimeTriviaQuestion for normal users.
  def show
    @anime_trivia_question = AnimeTriviaQuestion.find params[:id]
    @anime_series = @anime_trivia_question.anime_series
    @title = "Genericon Trivia System - Anime Trivia Question Details"
  end

  # Render the screen to insert a new AnimeTriviaQuestion.
  def new
    # We reference flash[] in case we've been redirected following an error.
    @anime_trivia_question = AnimeTriviaQuestion.new flash[:anime_trivia_question]
    @anime_series = AnimeSeries.new flash[:anime_series]
    @genre_checkbox = { }
    @genre_checkbox.default = '0'
    @genre_checkbox.merge! flash[:genre_checkbox] if !flash[:genre_checkbox].nil?
    @title = 'Genericon Trivia System - New Anime Trivia Question'
  end

  # Create a new AnimeTriviaQuestion and redirect back to new.
  def create
    # Validate existence of at least one checked genre.  We can bypass this if
    # we WANT, but we don't want to.
    has_one_genre = false
    params[:genre_checkbox].each do |genre, value|
      if value == '1'
        has_one_genre = true
        break
      end
    end
    
    # Load the series
    create_series = false
    @anime_series = AnimeSeries.find_by_name(params[:anime_series][:name])
    # If we can't load it, create it.
    if @anime_series == nil
      @anime_series = AnimeSeries.new(params[:anime_series])
      params[:genre_checkbox].each do |genre, value|
        if value == '1'
          @anime_series.anime_genres << AnimeGenre.find_by_underscored_name(genre)
        end
      end
      create_series = true
    end
    
    # Check for changes in the genres
    AnimeGenre.find(:all).each do |genre|
      # If we have a difference between genres, note the potential change.
      if (params[:genre_checkbox][genre.underscored_name] == '1') != @anime_series.anime_genres.include?(genre)
        if @anime_series.anime_genres.include? genre
          @anime_series.add_pending_change_support :anime_genres, :remove, genre.id
        else
          @anime_series.add_pending_change_support :anime_genres, :add, genre.id
        end
      end
    end
    
    @anime_trivia_question = AnimeTriviaQuestion.new params[:anime_trivia_question]
    @anime_trivia_question.anime_series = @anime_series

    # Run validation checks to get full list of errors...
    @anime_series.valid?
    @anime_trivia_question.valid?
    
    if has_one_genre
      success = true
      AnimeTriviaQuestion.transaction do
        AnimeSeries.transaction do
          success &&= @anime_series.save
          success &&= @anime_trivia_question.save
        end
      end
    else
      @anime_series.errors.add 'genres', '-- you must select at least one'
    end

    # Save and redirect.
    if success
      flash[:notice] = "Successfully submitted your anime trivia question."
      
      # Auto approve if we're logged in as an approver.
      if current_user.has_role? 'anime_trivia_approver'
        @anime_trivia_question.approve!
      end
      if current_user.has_role? 'anime_series_approver'
        @anime_series.approve!
      end
    else
      # Render the errors, and retain the values input previously for next time.
      flash[:error] = render_to_string :inline => "<%= error_messages_for 'anime_trivia_question' %><%= error_messages_for 'anime_series' %>"
      flash[:error_is_html] = true
      flash[:anime_trivia_question] = params[:anime_trivia_question]
      flash[:anime_series] = params[:anime_series]
      flash[:genre_checkbox] = params[:genre_checkbox]
    end
    
    # Redirect back to the new path.
    redirect_to new_anime_trivia_question_path
  end

  # Shows the edit page for an AnimeTriviaQuestion.
  def edit
    @anime_trivia_question = AnimeTriviaQuestion.find params[:id]
    @anime_series = @anime_trivia_question.anime_series || AnimeSeries.new()
    @title = "Genericon Trivia System - Edit Anime Trivia Question"
  end

  # Approves a pending change for an AnimeTriviaQuestion.
  def approve_pending_change
    @anime_trivia_question = AnimeTriviaQuestion.find params[:id]
    
    # Moderators commit the change.
    if current_user.has_role? 'anime_trivia_moderator'
      @anime_trivia_question.apply_pending_change params[:key].to_sym, params[:change_action].to_sym, params[:value]
    # Everyone else (read: approvers) just has their support added.
    else
      @anime_trivia_question.add_pending_change_support params[:key].to_sym, params[:change_action].to_sym, params[:value]
    end
    if @anime_trivia_question.save
      flash[:notice] = "Successfully approved the pending change."
    else
      # Render the errors, and retain the values input previously for next time.
      flash[:error] = render_to_string :inline => "<%= error_messages_for 'anime_trivia_question' %>"
      flash[:error_is_html] = true
    end
    
    redirect_to edit_anime_trivia_question_path, params[:id]
  end
  
  # Rejects a pending change for an AnimeTriviaQuestion.
  def reject_pending_change
    @anime_trivia_question = AnimeTriviaQuestion.find params[:id]
    
    # Moderators need to delete the pending change entirely
    if current_user.has_role? 'anime_trivia_moderator'
      @anime_trivia_question.delete_pending_change params[:key].to_sym, params[:change_action].to_sym, params[:value]
    # Everyone else (read: approvers) just has their support (if any) removed.
    else
      @anime_trivia_question.remove_pending_change_support params[:key].to_sym, params[:change_action].to_sym, params[:value]
    end
    if @anime_trivia_question.save
      flash[:notice] = "Successfully rejected the pending change."
    else
      # Render the errors, and retain the values input previously for next time.
      flash[:error] = render_to_string :inline => "<%= error_messages_for 'anime_trivia_question' %>"
      flash[:error_is_html] = true
    end
    
    redirect_to edit_anime_trivia_question_path, params[:id]
  end

  # Destroys an AnimeTriviaQuestion.
  def destroy
    # I sure hope you added a warning.
    # TODO: block when there are dependencies.
    
    if AnimeTriviaQuestion.destroy params[:id]
      flash[:notice] = "Successfully deleted the anime trivia question."
    else
      flash[:error] = "There was a problem deleting that anime trivia question."
    end
    
    redirect_to manage_anime_trivia_question_path
  end
  
  # Shows the list of AnimeTriviaQuestions to be managed.
  def manage
    order = params[:order_by] || 'state_id ASC, `anime_trivia_questions`.approved_at DESC, `anime_trivia_questions`.created_at DESC'
    @anime_trivia_questions = AnimeTriviaQuestion.paginate :all, :page => params[:page], :select => "`anime_trivia_questions`.*, CASE `anime_trivia_questions`.state WHEN 'pending_approval' THEN 1 WHEN 'pending_changes' THEN 2 WHEN 'approved' THEN 3 END AS state_id, CASE `anime_trivia_questions`.question_type WHEN 'normal' THEN 1 WHEN 'bonus' THEN 2 WHEN 'trick' THEN 3 END AS question_type_id, `anime_series`.name, `ratings`.rating", :joins => "INNER JOIN `anime_series` ON `anime_trivia_questions`.anime_series_id=`anime_series`.id LEFT JOIN (SELECT item_id, SUM(rating) AS rating FROM `game_history_entries` GROUP BY item_type, item_id HAVING item_type='AnimeTriviaQuestion') AS `ratings` ON `anime_trivia_questions`.id=`ratings`.item_id", :order => order
    @order = order
    @title = 'Genericon Trivia System - Anime Trivia Question Management'
  end
  
  # Approve an AnimeTriviaQuestion.
  def approve
    @anime_trivia_question = AnimeTriviaQuestion.find(params[:id])
    
    if @anime_trivia_question.pending_approval? and @anime_trivia_question.approvable_changes?
      if @anime_trivia_question.approve!
        flash[:notice] = "Successfully approved anime trivia question."
      else
        flash[:error] = render_to_string :inline => "<%= error_messages_for 'anime_trivia_question' %>"
        flash[:error_is_html] = true
      end
    elsif @anime_trivia_question.pending_changes? and @anime_trivia_question.approvable_changes?
      if @anime_trivia_question.approve_changes!
        flash[:notice] = "Successfully approved anime trivia question changes."
      else
        flash[:error] = render_to_string :inline => "<%= error_messages_for 'anime_trivia_question' %>"
        flash[:error_is_html] = true
      end
    else
      flash[:error] = render_to_string :inline => "<%= error_messages_for 'anime_trivia_question' %>"
      flash[:error_is_html] = true
    end
    
    redirect_to edit_anime_trivia_question_path, params[:id]
  end

  # Configure a new game.
  def new_game
    # End any active game.
    if !session[:unused].nil?
      begin
        @game_session = GameSession.find(session[:unused])
        if @game_session.game_type != 'anime_trivia_question'
          GameSession.clean_sessions(session[:unused])
          session[:unused] = nil
        end
      rescue ActiveRecord::RecordNotFound
        session[:unused] = nil
      end
    end
    
    # Setup the new game.
  end

  # Play Anime Trivia.
  def play
    # First, check the old session to make sure it's a valid one.  If we came
    # from another game, we NEED to reset.  We also need to gracefully restart
    # from a timed-out game.
    if !session[:unused].nil?
      begin
        @game_session = GameSession.find(session[:unused])
        if @game_session.game_type != 'anime_trivia_question'
          GameSession.clean_sessions(session[:unused])
          session[:unused] = nil
        end
      rescue ActiveRecord::RecordNotFound
        session[:unused] = nil
      end
    end
    
    if session[:unused].nil?
      @unused = AnimeTriviaQuestion.find(:all)
      @unused = @unused.select {|q| !q.pending_approval? and !q.anime_series.pending_approval? and (q.anime_series.anime_continuum.nil? or !q.anime_series.anime_continuum.pending_approval?)}
      @unused_normal = @unused.select {|q| q.question_type == :normal}
      @unused = @unused.collect { |q| q.id }
      @unused_normal = @unused_normal.collect { |q| q.id }
      @window = [ ]
      @counters = { }
      @counters.default = 0
      @max_from_series = params[:max_from_series].to_i
      @max_from_series = @unused.length if @max_from_series == 0
      
      8.times do
        id = @unused_normal.slice!(rand(@unused_normal.length))
        @unused.delete(id)
        @question = AnimeTriviaQuestion.find(id)
	
        # Make sure to filter by max_from_series (though this is sort of
        # irrelevant when starting.
        while @counters[@question.anime_series.id] >= @max_from_series and !@unused_normal.empty?
          id = @unused_normal.slice!(rand(@unused_normal.length))
          @unused.delete id
          @question = AnimeTriviaQuestion.find(id)
        end
        
        if @counters[@question.anime_series.id] < @max_from_series
          @window << id
        end
        
        if @unused_normal.empty?
          break
        end
      end
      
      @game_session = GameSession.new(:game_type => 'anime_trivia_question')
      @game_session.data = {:unused => @unused, :unused_normal => @unused_normal, :window => @window, :max_from_series => @max_from_series}
      @game_session.save
      session[:unused] = @game_session.id
    else
      @game_session = GameSession.find(session[:unused])
      @unused = @game_session.data[:unused]
      @unused_normal = @game_session.data[:unused_normal]
      @window = @game_session.data[:window]
      @counters = { }
      @counters.default = 0
      @counters.merge!(@game_session.data[:counters]) if @game_session.data[:counters]
      @max_from_series = @game_session.data[:max_from_series]
      
      # Can keep playing until the window is empty.
      if @window.empty?
        GameSession.clean_sessions(session[:unused])
        session[:unused] = nil
        flash[:notice] = 'All questions have been used up.  Game over.'
        redirect_to :action => 'index'
        return
      end
    end
    
    # Render the trivia questions.
    @window = @window.collect {|id| AnimeTriviaQuestion.find(id)}
    @title = "Genericon Trivia System - Anime Trivia"
  end

  # Remove the given question from the window and get a new question.
  def next_question
    id = params[:id]
    bonus_id = params[:bonus_id]
    
    # Save the ratings
    if params[:rating].nil?
      rating = 0
    else
      rating = params[:rating].to_i
      if rating < -1
        rating = -1
      elsif rating > 1
        rating = 1
      end
    end
    if params[:bonus_rating].nil?
      bonus_rating = 0
    else
      bonus_rating = params[:bonus_rating].to_i
      if bonus_rating < -1
        bonus_rating = -1
      elsif bonus_rating > 1
        bonus_rating = 1
      end
    end
    game_history_entry = GameHistoryEntry.new :item => AnimeTriviaQuestion.find(id), :rating => rating
    game_history_entry.save
    if !bonus_id.nil?
      game_history_entry = GameHistoryEntry.new :item => AnimeTriviaQuestion.find(bonus_id), :rating => bonus_rating
      game_history_entry.save
    end
    
    @game_session = GameSession.find(params[:session])
    @unused = @game_session.data[:unused]
    @unused_normal = @game_session.data[:unused_normal]
    @window = @game_session.data[:window]
    @counters = { }
    @counters.default = 0
    @counters.merge!(@game_session.data[:counters]) if @game_session.data[:counters]
    @max_from_series = @game_session.data[:max_from_series]
    
    # Remove the question.
    idx = @window.index id.to_i
    @window.delete_at idx
    if !bonus_id.nil?
      @unused.delete bonus_id.to_i
    end
    
    # Increment the appropriate counter and remove anything still from
    # now-disallowed series, saving the counter.
    @question = AnimeTriviaQuestion.find(id)
    @counters[@question.anime_series.id] = @counters[@question.anime_series.id] + 1
    
    if @counters[@question.anime_series.id] >= @max_from_series
      remove_questions = [ ]
      add_questions = [ ]
      @window.each do |old_id|
        if AnimeTriviaQuestion.find(old_id).anime_series.id == @question.anime_series.id
          # Remember to remove this now-disallowed question.
          remove_questions << old_id
          
          # And get a new question to replace it with
          if !@unused_normal.empty?
            new_id = @unused_normal.slice!(rand(@unused_normal.length))
            new_question = AnimeTriviaQuestion.find(new_id)
            @unused.delete new_id
            
            # Make sure to filter by max_from_series
            while @counters[new_question.anime_series.id] >= @max_from_series and !@unused_normal.empty?
              new_id = @unused_normal.slice!(rand(@unused_normal.length))
              @unused.delete new_id
              new_question = AnimeTriviaQuestion.find(new_id)
            end
            
            if @counters[new_question.anime_series.id] < @max_from_series
              add_questions << new_id
            end
          end
        end
      end
      remove_questions.each do |id|
        insert_at = @window.index(id)
        @window.delete_at(insert_at)
        @window.insert(insert_at, add_questions.shift) if !add_questions.empty?
      end
    end
    
    # Get the next question.
    if !@unused_normal.empty?
      id = @unused_normal.slice!(rand(@unused_normal.length))
      @question = AnimeTriviaQuestion.find(id)
      @unused.delete id
      
      # Make sure to filter by max_from_series
      while @counters[@question.anime_series.id] >= @max_from_series and !@unused_normal.empty?
        id = @unused_normal.slice!(rand(@unused_normal.length))
        @unused.delete new_id
        @question = AnimeTriviaQuestion.find(id)
      end
      
      if @counters[@question.anime_series.id] < @max_from_series
        @window.insert(idx, id)
      end
    end
    
    data = @game_session.data || { }
    data[:window] = @window
    data[:unused_normal] = @unused_normal
    data[:unused] = @unused
    data[:counters] = @counters
    @game_session.data = data
    @game_session.save
    
    if !@window.empty?
      @window = @window.collect {|id| AnimeTriviaQuestion.find(id)}
      render :partial => "question_window", :locals => {:question_window => @window, :i => params[:i], :session => {:unused => @game_session.id}}
    else
      GameSession.clean_sessions(params[:session])
      session[:unused] = nil
      flash[:notice] = 'All questions have been used up.  Game over.'
      redirect_to :action => 'index'
      return
    end
  end
  
  # Rerandomizes @window for play.
  def randomize_play_window
    @game_session = GameSession.find(session[:unused])
    @unused = @game_session.data[:unused]
    @unused_normal = @game_session.data[:unused_normal]
    @window = @game_session.data[:window]
    @counters = { }
    @counters.default = 0
    @counters.merge!(@game_session.data[:counters]) if @game_session.data[:counters]
    @max_from_series = @game_session.data[:max_from_series]
    
    @window.each {|id| @unused << id; @unused_normal << id}
    
    @window = [ ]
    8.times do
      id = @unused_normal.slice!(rand(@unused_normal.length))
      @unused.delete(id)
      @question = AnimeTriviaQuestion.find(id)
      
      # Make sure to filter by max_from_series
      while @counters[@question.anime_series.id] >= @max_from_series and !@unused_normal.empty?
        id = @unused_normal.slice!(rand(@unused_normal.length))
        @unused.delete id
        @question = AnimeTriviaQuestion.find(id)
      end
      
      if @counters[@question.anime_series.id] < @max_from_series
        @window << id
      end
      
      if @unused_normal.empty?
        break
      end
    end
    
    # Remember to save the current window and unused.
    @game_session = GameSession.find(session[:unused])
    data = @game_session.data || { }
    data[:window] = @window
    data[:unused_normal] = @unused_normal
    data[:unused] = @unused
    @game_session.data = data
    @game_session.save
    
    @window = @window.collect {|id| AnimeTriviaQuestion.find(id)}
    
    render :partial => 'question_window', :object => @window, :locals => {:session => session}
  end

  def remove_anime_series_from_game
    id = params[:id]
    
    @game_session = GameSession.find(params[:session])
    @unused = @game_session.data[:unused]
    @unused_normal = @game_session.data[:unused_normal]
    @window = @game_session.data[:window]
    @counters = { }
    @counters.default = 0
    @counters.merge!(@game_session.data[:counters]) if @game_session.data[:counters]
    @max_from_series = @game_session.data[:max_from_series]
    
    questions_to_remove = AnimeTriviaQuestion.find(:all, :conditions => ['anime_series_id = ?', id.to_i]).to_set
    questions_to_remove.collect! {|q| q.id}
    @window.reject! {|i| questions_to_remove.include? i}
    @unused.reject! {|i| questions_to_remove.include? i}
    @unused_normal.reject! {|i| questions_to_remove.include? i}
    
    if @window.length < 8
      (8 - @window.length).times do
        id = @unused_normal.slice!(rand(@unused_normal.length))
        @unused.delete(id)
        @question = AnimeTriviaQuestion.find(id)
        
        # Make sure to filter by max_from_series
        while @counters[@question.anime_series.id] >= @max_from_series and !@unused_normal.empty?
          id = @unused_normal.slice!(rand(@unused_normal.length))
          @unused.delete new_id
          @question = AnimeTriviaQuestion.find(id)
        end
        
        if @counters[@question.anime_series.id] < @max_from_series
          @window << id
        end
        
        if @unused_normal.empty?
          break
        end
      end
    end
    
    # Remember to save the current window and unused.
    data = @game_session.data || { }
    data[:window] = @window
    data[:unused_normal] = @unused_normal
    data[:unused] = @unused
    @game_session.data = data
    @game_session.save
    
    @window = @window.collect {|id| AnimeTriviaQuestion.find(id)}
    
    render :partial => 'question_window', :object => @window, :locals => { :session => { :unused => params[:session] } }
  end

  # Change the maximum questions from series.
  def set_max_from_series
    @game_session = GameSession.find(params[:session])
    data = @game_session.data || { }
    if params[:max_from_series].nil? or params[:max_from_series].empty? or params[:max_from_series].to_i == 0
      data[:max_from_series] = data[:unused].length
    else
      data[:max_from_series] = params[:max_from_series].to_i
    end
    @game_session.data = data
    @game_session.save
  end

end
