# Filters added to this controller apply to all controllers in the application.
# Likewise, all the methods added will be available for all controllers.

class ApplicationController < ActionController::Base
  # AuthenticatedSystem must be included for RoleRequirement, and is provided by installing acts_as_authenticates and running 'script/generate authenticated account user'.
  include AuthenticatedSystem
  # You can move this into a different controller, if you wish.  This module gives you the require_role helpers, and others.
  include RoleRequirementSystem
  # We need to be able to add userstamps to most objects and relations.
  include Userstamp

  # Always retrieve any errors that were cached in the session hash due to a redirect.
  before_filter :set_stampers

  helper :all # include all helpers, all the time

  # See ActionController::RequestForgeryProtection for details
  # Uncomment the :secret if you're not using the cookie session store
  protect_from_forgery # :secret => 'd773f2b56165e59fb30a2c3d5627e399'
  
  # See ActionController::Base for details 
  # Uncomment this to filter the contents of submitted sensitive data parameters
  # from your application log (in this case, all fields with names like "password"). 
  # filter_parameter_logging :password
  
  # All controllers use the main layout unless otherwise specified.
  layout 'main'
  
  # Some general auto_complete functions used in multiple locations.
  auto_complete_for :user, :login

  # We do have a few methods.
  before_filter :login_required, :only => [:unapproved_items, :recent_changes]
  after_filter :store_location, :only => [:unapproved_items, :recent_changes]
  require_role 'super_approver', :only => [:unapproved_items, :recent_changes]
  
  # Auto-complete AnimeContinuum names.
  def auto_complete_for_anime_continuum_name
    find_options = {
      :conditions => [ "LOWER(name) LIKE ?", '%' + params[:anime_continuum][:name].downcase + '%' ],
      :order => "name ASC",
      :limit => 10 }
    
    @items = AnimeContinuum.find(:all, find_options)
    
    # Merge in alternate names that match as well.
    find_options = {
      :conditions => [ "item_type = 'AnimeContinuum' AND LOWER(value) LIKE ?", '%' + params[:anime_continuum][:name].downcase + '%' ],
      :order => "value ASC",
      :limit => 10 }
    
    @alternate_names = AlternateName.find(:all, find_options)
    @alternate_names = @alternate_names.map {|name| name.item}
    @alternate_names = @alternate_names.select {|item| !@items.include? item}
    
    @items = @items.concat(@alternate_names).sort {|x,y| x.name <=> y.name}
    @items = @items[0, 10]
    
    render :inline => "<%= auto_complete_result @items, 'name' %>"
  end
  
  # Auto-complete AnimeSeries names.
  def auto_complete_for_anime_series_name
    find_options = {
      :conditions => [ "LOWER(name) LIKE ?", '%' + params[:anime_series][:name].downcase + '%' ],
      :order => "name ASC",
      :limit => 10 }
    
    @items = AnimeSeries.find(:all, find_options)
    
    # Merge in alternate names that match as well.
    find_options = {
      :conditions => [ "item_type = 'AnimeSeries' AND LOWER(value) LIKE ?", '%' + params[:anime_series][:name].downcase + '%' ],
      :order => "value ASC",
      :limit => 10 }
    
    @alternate_names = AlternateName.find(:all, find_options)
    @alternate_names = @alternate_names.map {|name| name.item}
    @alternate_names = @alternate_names.select {|item| !@items.include? item}
    
    @items = @items.concat(@alternate_names).sort {|x,y| x.name <=> y.name}
    @items = @items[0, 10]
    
    render :inline => "<%= auto_complete_result @items, 'name' %>"
  end
  
  def find_similar_genres_to_anime
    anime = params[:anime]
    anime = AnimeSeries.find_by_name anime
    if anime == nil
      genres = []
    else
      genres = AnimeGenre.find_genres_like anime.anime_genres.collect { |genre| genre.name }
    end
    render(:text => genres.to_json)
    response.headers['Content-Type'] = 'application/json'
  end

  def find_anime_genres_like
    genres = params[:genres]
    if genres != nil
      genres = genres.split(',')
    end
    genres = AnimeGenre.find_genres_like genres
    render(:text => genres.to_json)
    response.headers['Content-Type'] = 'application/json'
  end
  
  # Auto-complete VideoGameSystem names.
  def auto_complete_for_video_game_system_name
    find_options = {
      :conditions => [ "LOWER(name) LIKE ?", '%' + params[:video_game_system][:name].downcase + '%' ],
      :order => "name ASC",
      :limit => 10 }
    
    @items = VideoGameSystem.find(:all, find_options)
    
    # Merge in alternate names that match as well.
    find_options = {
      :conditions => [ "item_type = 'VideoGameSystem' AND LOWER(value) LIKE ?", '%' + params[:video_game_system][:name].downcase + '%' ],
      :order => "value ASC",
      :limit => 10 }
    
    @alternate_names = AlternateName.find(:all, find_options)
    @alternate_names = @alternate_names.map {|name| name.item}
    @alternate_names = @alternate_names.select {|item| !@items.include? item}
    
    @items = @items.concat(@alternate_names).sort {|x,y| x.name <=> y.name}
    @items = @items[0, 10]
    
    render :inline => "<%= auto_complete_result @items, 'name' %>"
  end
  
  # Auto-complete VideoGameContinuum names.
  def auto_complete_for_video_game_continuum_name
    find_options = {
      :conditions => [ "LOWER(name) LIKE ?", '%' + params[:video_game_continuum][:name].downcase + '%' ],
      :order => "name ASC",
      :limit => 10 }
    
    @items = VideoGameContinuum.find(:all, find_options)
    
    # Merge in alternate names that match as well.
    find_options = {
      :conditions => [ "item_type = 'VideoGameContinuum' AND LOWER(value) LIKE ?", '%' + params[:video_game_continuum][:name].downcase + '%' ],
      :order => "value ASC",
      :limit => 10 }
    
    @alternate_names = AlternateName.find(:all, find_options)
    @alternate_names = @alternate_names.map {|name| name.item}
    @alternate_names = @alternate_names.select {|item| !@items.include? item}
    
    @items = @items.concat(@alternate_names).sort {|x,y| x.name <=> y.name}
    @items = @items[0, 10]
    
    render :inline => "<%= auto_complete_result @items, 'name' %>"
  end
  
  # Auto-complete VideoGame names.
  def auto_complete_for_video_game_name
    find_options = {
      :conditions => [ "LOWER(name) LIKE ?", '%' + params[:video_game][:name].downcase + '%' ],
      :order => "name ASC",
      :limit => 10 }
    
    @items = VideoGame.find(:all, find_options)
    
    # Merge in alternate names that match as well.
    find_options = {
      :conditions => [ "item_type = 'VideoGame' AND LOWER(value) LIKE ?", '%' + params[:video_game][:name].downcase + '%' ],
      :order => "value ASC",
      :limit => 10 }
    
    @alternate_names = AlternateName.find(:all, find_options)
    @alternate_names = @alternate_names.map {|name| name.item}
    @alternate_names = @alternate_names.select {|item| !@items.include? item}
    
    @items = @items.concat(@alternate_names).sort {|x,y| x.name <=> y.name}
    @items = @items[0, 10]
    
    render :inline => "<%= auto_complete_result @items, 'name' %>"
  end
  
  def find_genres_for_game
    video_game = params[:video_game]
    video_game = VideoGame.find_by_name video_game
    if video_game == nil
      genres = []
    else
      genres = video_game.video_game_genres
    end
    genres.collect! { |genre| {:name => genre.name,
                               :selected => true,
                               :underscored_name => genre.underscored_name}}
    render(:text => genres.to_json)
    response.headers['Content-Type'] = 'application/json'
  end

  def find_system_for_game
    video_game = params[:video_game]
    video_game = VideoGame.find_by_name video_game
    if video_game == nil or video_game.video_game_system.nil?
      render(:text => '')
    else
      render(:text => video_game.video_game_system.name)
    end
    response.headers['Content-Type'] = 'text/plain'
  end

  # Retrieves the error from the session, if specified.
  #def retrieve_errors
  #  if session[:error]
  #    flash[:error] = session[:error]
  #    session[:error] = nil
  #  elsif session[:notice]
  #    flash[:notice] = session[:notice]
  #    session[:notice] = nil
  #  end
  #end
  
  # Set the stamper for all objects
  def set_stampers
    User.stamper = current_user
  end

  # List all unapproved items.
  def unapproved_items
    anime_trivia_questions = AnimeTriviaQuestion.find :all, :conditions => "state != 'approved'", :order => 'updated_at DESC'
    anime_series = AnimeSeries.find :all, :conditions => "state != 'approved'", :order => 'updated_at DESC'
    anime_scenes = AnimeScene.find :all, :conditions => "state != 'approved'", :order => 'updated_at DESC'
    anime_continuums = AnimeContinuum.find :all, :conditions => "state != 'approved'", :order => 'updated_at DESC'
    video_game_trivia_questions = VideoGameTriviaQuestion.find :all, :conditions => "state != 'approved'", :order => 'updated_at DESC'
    video_games = VideoGame.find :all, :conditions => "state != 'approved'", :order => 'updated_at DESC'
    video_game_systems = VideoGameSystem.find :all, :conditions => "state != 'approved'", :order => 'updated_at DESC'
    video_game_continuums = VideoGameContinuum.find :all, :conditions => "state != 'approved'", :order => 'updated_at DESC'
    video_game_scenes = VideoGameScene.find :all, :conditions => "state != 'approved'", :order => 'updated_at DESC'
    
    @items = []
    anime_trivia_question = nil
    anime = nil
    anime_continuum = nil
    anime_scene = nil
    video_game = nil
    video_game_trivia_question = nil
    video_game_continuum = nil
    video_game_system = nil
    video_game_scene = nil
    while !anime_trivia_questions.empty? or !anime_series.empty? or !anime_continuums.empty? or !anime_scenes.empty? or !video_games.empty? or !video_game_trivia_questions.empty? or !video_game_continuums.empty? or !video_game_systems.empty? or !video_game_scenes.empty?
      anime_trivia_question = anime_trivia_questions.shift if anime_trivia_question.nil? and !anime_trivia_questions.empty?
      anime = anime_series.shift if anime.nil? and !anime_series.empty?
      anime_continuum = anime_continuums.shift if anime_continuum.nil? and !anime_continuums.empty?
      anime_scene = anime_scenes.shift if anime_scene.nil? and !anime_scenes.empty?
      video_game = video_games.shift if video_game.nil? and !video_games.empty?
      video_game_trivia_question = video_game_trivia_questions.shift if video_game_trivia_question.nil? and !video_game_trivia_questions.empty?
      video_game_continuum = video_game_continuums.shift if video_game_continuum.nil? and !video_game_continuums.empty?
      video_game_system = video_game_systems.shift if video_game_system.nil? and !video_game_systems.empty?
      video_game_scene = video_game_scenes.shift if video_game_scene.nil? and !video_game_scenes.empty?
      
      newest = nil
      newest = anime_trivia_question if !anime_trivia_question.nil? and (newest.nil? or newest.updated_at < anime_trivia_question.updated_at)
      newest = anime if !anime.nil? and (newest.nil? or newest.updated_at < anime.updated_at)
      newest = anime_scene if !anime_scene.nil? and (newest.nil? or newest.updated_at < anime_scene.updated_at)
      newest = anime_continuum if !anime_continuum.nil? and (newest.nil? or newest.updated_at < anime_continuum.updated_at)
      newest = video_game if !video_game.nil? and (newest.nil? or newest.updated_at < video_game.updated_at)
      newest = video_game_system if !video_game_system.nil? and (newest.nil? or newest.updated_at < video_game_system.updated_at)
      newest = video_game_continuum if !video_game_continuum.nil? and (newest.nil? or newest.updated_at < video_game_continuum.updated_at)
      newest = video_game_scene if !video_game_scene.nil? and (newest.nil? or newest.updated_at < video_game_scene.updated_at)
      newest = video_game_trivia_question if !video_game_trivia_question.nil? and (newest.nil? or newest.updated_at < video_game_trivia_question.updated_at)
      
      if !newest.nil?
        anime_trivia_question = nil if newest == anime_trivia_question
        anime = nil if newest == anime
        anime_scene = nil if newest == anime_scene
        anime_continuum = nil if newest == anime_continuum
        video_game = nil if newest == video_game
        video_game_system = nil if newest == video_game_system
        video_game_continuum = nil if newest == video_game_continuum
        video_game_trivia_question = nil if newest == video_game_trivia_question
        video_game_scene = nil if newest == video_game_scene
        
        @items << newest
      end
    end
    
    @items = @items.paginate :page => params[:page]
    @title = 'Genericon Trivia System - Unapproved Items'
  end
  
  # List all recent changes.
  def recent_changes
    anime_trivia_questions = AnimeTriviaQuestion.find :all, :order => 'updated_at DESC'
    anime_series = AnimeSeries.find :all, :order => 'updated_at DESC'
    anime_scenes = AnimeScene.find :all, :order => 'updated_at DESC'
    anime_continuums = AnimeContinuum.find :all, :order => 'updated_at DESC'
    video_game_trivia_questions = VideoGameTriviaQuestion.find :all, :order => 'updated_at DESC'
    video_games = VideoGame.find :all, :order => 'updated_at DESC'
    video_game_systems = VideoGameSystem.find :all, :order => 'updated_at DESC'
    video_game_continuums = VideoGameContinuum.find :all, :order => 'updated_at DESC'
    video_game_scenes = VideoGameScene.find :all, :order => 'updated_at DESC'
    
    @items = []
    anime_trivia_question = nil
    anime = nil
    anime_continuum = nil
    anime_scene = nil
    video_game = nil
    video_game_trivia_question = nil
    video_game_continuum = nil
    video_game_system = nil
    video_game_scene = nil
    while !anime_trivia_questions.empty? or !anime_series.empty? or !anime_continuums.empty? or !anime_scenes.empty? or !video_games.empty? or !video_game_trivia_questions.empty? or !video_game_continuums.empty? or !video_game_systems.empty? or !video_game_scenes.empty?
      anime_trivia_question = anime_trivia_questions.shift if anime_trivia_question.nil? and !anime_trivia_questions.empty?
      anime = anime_series.shift if anime.nil? and !anime_series.empty?
      anime_continuum = anime_continuums.shift if anime_continuum.nil? and !anime_continuums.empty?
      anime_scene = anime_scenes.shift if anime_scene.nil? and !anime_scenes.empty?
      video_game = video_games.shift if video_game.nil? and !video_games.empty?
      video_game_trivia_question = video_game_trivia_questions.shift if video_game_trivia_question.nil? and !video_game_trivia_questions.empty?
      video_game_continuum = video_game_continuums.shift if video_game_continuum.nil? and !video_game_continuums.empty?
      video_game_system = video_game_systems.shift if video_game_system.nil? and !video_game_systems.empty?
      video_game_scene = video_game_scenes.shift if video_game_scene.nil? and !video_game_scenes.empty?
      
      newest = nil
      newest = anime_trivia_question if !anime_trivia_question.nil? and (newest.nil? or newest.updated_at < anime_trivia_question.updated_at)
      newest = anime if !anime.nil? and (newest.nil? or newest.updated_at < anime.updated_at)
      newest = anime_scene if !anime_scene.nil? and (newest.nil? or newest.updated_at < anime_scene.updated_at)
      newest = anime_continuum if !anime_continuum.nil? and (newest.nil? or newest.updated_at < anime_continuum.updated_at)
      newest = video_game if !video_game.nil? and (newest.nil? or newest.updated_at < video_game.updated_at)
      newest = video_game_system if !video_game_system.nil? and (newest.nil? or newest.updated_at < video_game_system.updated_at)
      newest = video_game_continuum if !video_game_continuum.nil? and (newest.nil? or newest.updated_at < video_game_continuum.updated_at)
      newest = video_game_scene if !video_game_scene.nil? and (newest.nil? or newest.updated_at < video_game_scene.updated_at)
      newest = video_game_trivia_question if !video_game_trivia_question.nil? and (newest.nil? or newest.updated_at < video_game_trivia_question.updated_at)
      
      if !newest.nil?
        anime_trivia_question = nil if newest == anime_trivia_question
        anime = nil if newest == anime
        anime_scene = nil if newest == anime_scene
        anime_continuum = nil if newest == anime_continuum
        video_game = nil if newest == video_game
        video_game_system = nil if newest == video_game_system
        video_game_continuum = nil if newest == video_game_continuum
        video_game_trivia_question = nil if newest == video_game_trivia_question
        video_game_scene = nil if newest == video_game_scene
        
        @items << newest
      end
    end
    
    @items = @items.paginate :page => params[:page]
    @title = 'Genericon Trivia System - Recent Changes'
  end
end
