# This controller handles comment functions, like changing the type and adding
# a reply 
class CommentsController < ApplicationController
  before_filter :login_required
  require_role 'admin', :for => [:delete, :edit_comment_type, :update_comment_type]
  
  # Generates a form to create a comment.
  def new
    @parent_id = params[:id]
    if @parent_id == '0'
      @item_type = params[:item_type]
      @item_id = params[:item_id]
    else
      parent = Comment.find @parent_id
      @item_type = parent.item_type
      @item_id = parent.item_id
    end
    render :action => 'comment_box.js.rjs'
  end
  
  # Gets the description of a comment type from the type itself.
  def get_type_description
    @type = params[:type].to_sym
    render :action => 'comment_type_description.js.rjs'
  end
  
  # Adds a reply to a comment
  def create
    id = params[:comment_parent_id]
    comment = Comment.new(:comment_type => params[:comment_type].to_sym,
                          :text => params[:comment_message])
    
    if id == '0'
      comment.parent = nil
      comment.item = params[:comment_item_type].to_s.camelize.constantize.find params[:comment_item_id]
    else
      Comment.find(id).children << comment
    end
    
    if comment.valid?
      comment.save
      flash[:notice] = 'The comment was successfully added.'
    end
    
    redirect_back_or_default '/'
  end
  
  # Deletes a comment
  def delete
    id = params[:id]
    Comment.destroy id
    flash[:notice] = 'The comment was successfully deleted.'
    redirect_back_or_default '/'
  end
  
  # Makes a comment into a note.
  def make_note
    comment = Comment.find params[:id]
    # We need to be dynamic in determining access denied.
    if !current_user.has_role?(comment.item_type.underscore + '_moderator')
      access_denied
    else
      comment.comment_type = :note
      comment.save
      flash[:notice] = 'The comment was turned into a Note.'
      redirect_back_or_default '/'
    end
  end
  
  # Enables the comment-type editor field.
  def edit_comment_type
    @comment = Comment.find params[:id]
    render :action => 'comment_type_editor.js.rjs'
  end
  
  # Updates the comment-type of a comment.
  def update_comment_type
    @comment = Comment.find params[:id]
    @comment.comment_type = params["comment#{params[:id]}_type"].to_sym
    @comment.save
    flash[:notice] = "The comment's type was changed."
    redirect_back_or_default '/'
  end
end
