# Allows viewing of details of roles.
class RolesController < ApplicationController
  before_filter :login_required
  before_filter :require_delegator, :only => [:add_user, :remove_user,
                                              :add_delegator, :remove_delegator]
  
  # Renders a list of roles.
  def index
    list
    render :action => 'list'
  end

  # Lists roles.  Should only allow delegation if delegator or admin.
  def list
    @roles = Role.paginate :all, :page => params[:page], :order => 'name ASC'
    @title = 'Genericon Trivia System - Role List'
  end
  
  # Shows role detail (including user lists).  Should only allow delegation if delegator or admin.
  def show
    @role = Role.find(params[:id])
    @title = 'Genericon Trivia System - Role Details'
  end
  
  # Adds a role to a user.
  def add_user
    @user = User.find_by_login(params[:user][:login])
    role = Role.find(params[:role])
    if @user.nil?
      flash[:error] = "Could not add user: User not found."
    else
      @user.roles << role
      if @user.valid?
        @user.save
        flash[:notice] = "Successfully added role to #{@user.login}."
      end
    end
    redirect_to show_role_path(role.id)
  end
  
  # Removes a role from a user.
  def remove_user
    @user = User.find(params[:user])
    role = Role.find(params[:role])
    if @user.nil?
      flash[:error] = "Could not remove user: User not found."
    elsif role.name == 'admin' and @user == current_user
      flash[:error] = "You cannot remove admin status from yourself!"
    else
      if @user.delegator_roles.include?(role)
        @user.delegator_roles.delete(role)
        flash[:notice] = "Successfully removed role from #{@user.login}. Also removed delegator status."
      end
      @user.roles.delete(role)
      if @user.valid?
        @user.save
        if flash[:notice].nil?
          flash[:notice] = "Successfully removed role from #{@user.login}."
        end
      else
        flash[:notice] = nil
      end
    end
    redirect_to show_role_path(role.id)
  end
  
  # Adds a delegator.
  def add_delegator
    @user = User.find_by_login(params[:user][:login])
    role = Role.find(params[:role])
    if @user.nil?
      flash[:error] = "Could not add delegator: User not found."
    else
      if !@user.roles.include?(role)
        @user.roles << role
        flash[:notice] = "Successfully added delegator status to #{@user.login}. Also added role."
      end
      @user.delegator_roles << role
      if @user.valid?
        @user.save
        if flash[:notice].nil?
          flash[:notice] = "Successfully added delegator status to #{@user.login}."
        end
      else
        flash[:notice] = nil
      end
    end
    redirect_to show_role_path(role.id)
  end
  
  # Removes a delegator.
  def remove_delegator
    @user = User.find(params[:user])
    role = Role.find(params[:role])
    if @user.nil?
      flash[:error] = "Could not remove delegator: User not found."
    else
      @user.delegator_roles.delete(role)
      if @user.valid?
        @user.save
        flash[:notice] = "Successfully removed delegator status from #{@user.login}."
      end
    end
    redirect_to show_role_path(role.id)
  end

protected
  # Requires the appropriate delegator before an action may take place.
  def require_delegator
    return access_denied unless current_user.can_delegate?(Role.find(params[:role]))
    
    true
  end
end
