require 'ftools'
require 'mime/types'

# Filters access to uploads to prevent execution.
class UploadsController < ApplicationController
  def index
    # Which file are we proxying?
    @item = params[:attachment].singularize.classify.constantize.find(params[:id])
    @file = @item.attachment_for(params[:attachment].singularize.to_sym)
    
    # First, check the etag.
    @etag = File.stat(@file.path(params[:style]))
    @etag = @etag.ino.to_s(16) + '-' + @etag.size.to_s(16) + '-' + @etag.mtime.to_i.to_s(16)
    if request.env['HTTP_IF_NONE_MATCH'] && request.env['HTTP_IF_NONE_MATCH'] == @etag
      # On a match, return a 304.
      return head(:not_modified)
    end
    
    # Predict the MIME type we're returning.  Trust the value in the DB most.
    @mime_type = @file.content_type
    # If it's image/pjpeg or image/x-png, correct the MIME type.
    if @mime_type == 'image/pjpeg'
      @mime_type = 'image/jpeg'
    elif @mime_type == 'image/x-png'
      @mime_type = 'image/png'
    end

    # If it's text/plain or application/octet-stream, try to outguess the
    # browser.
    if @mime_type == 'text/plain' or @mime_type == 'application/octet-stream'
      @mime_type = MIME::Types.type_for(@file.path(params[:style]))
    end
    
    headers['ETag'] = @etag
    send_file @file.path(params[:style]), :filename => File.basename(@file.url(params[:style])), :type => @mime_type, :disposition => 'inline'
  end
end