# Controls VideoGameContinuum pages.
class VideoGameContinuumController < ApplicationController
  before_filter :login_required, :except => [:index, :list, :show, :new,
                                             :create, :search_select_merge_video_game_continuums]
  after_filter :store_location, :only => [:index, :list, :show, :new, :edit,
                                          :manage, :select_merge]
  
  require_role 'video_game_continuum_approver', :for_all_except => [:index, :list,
                                                                    :show, :new,
                                                                    :create, :search_select_merge_video_game_continuums]
  require_role 'video_game_continuum_moderator', :for => [:destroy, :select_merge,
                                                          :merge_into]
  
  in_place_edit_with_validation_for :video_game_continuum, :name
  
  # Renders a list of VideoGameContinuums.
  def index
    list
    render :action => :list
  end

  # Render a list of VideoGameContinuums.
  def list
    @video_game_continuums = VideoGameContinuum.paginate :all, :page => params[:page], :order => 'name ASC'
    @title = 'Genericon Trivia System - Video Game Continuums List'
  end

  # Render the details of an VideoGameContinuum for normal users.
  def show
    @video_game_continuum = VideoGameContinuum.find params[:id]
    @title = "Genericon Trivia System - Video Game Continuum Details - #{@video_game_continuum.name}"
  end

  # Render the screen to insert a new VideoGameContinuum.
  def new
    # We reference flash[] in case we've been redirected following an error.
    @video_game_continuum = VideoGameContinuum.new flash[:video_game_continuum]
    @submission_note = flash[:submission_note]
    @title = 'Genericon Trivia System - New Video Game Continuum'
  end

  # Create a new VideoGameContinuum and redirect back to new.
  def create
    # Translate the alternate_names hash to an array...
    params[:video_game_continuum][:alternate_names] = params[:video_game_continuum][:alternate_names].keys.sort.collect { |key| params[:video_game_continuum][:alternate_names][key] }
    # Remove duplicate alternate_names.
    params[:video_game_continuum][:alternate_names] = params[:video_game_continuum][:alternate_names].uniq
    # Remove empty alternate_names
    params[:video_game_continuum][:alternate_names] = params[:video_game_continuum][:alternate_names].reject { |name| name.nil? or name.empty? }
    
    # Create the VideoGameContinuum.
    @video_game_continuum = VideoGameContinuum.new params[:video_game_continuum]
    
    # Create the submission note if there was one.
    if !params[:submission_note].nil? and !params[:submission_note].empty?
      comment = @video_game_continuum.comments.build :item => @video_game_continuum, :text => params[:submission_note], :comment_type => :submission_note
    end

    success = true
    VideoGameContinuum.transaction do
      Comment.transaction do
        success &&= @video_game_continuum.save
        success &&= comment.save if !params[:submission_note].nil? and !params[:submission_note].empty?
      end
    end
    
    # Save and redirect.
    if success
      flash[:notice] = "Successfully submitted your video game continuum."
      
      # Auto approve if we're logged in as an approver.
      if current_user.has_role? 'video_game_continuum_approver'
        @video_game_continuum.approve!
      end
    else
      # Render the errors, and retain the values input previously for next time.
      flash[:error] = render_to_string :inline => "<%= error_messages_for 'video_game_continuum' %>"
      flash[:error_is_html] = true
      flash[:video_game_continuum] = params[:video_game_continuum]
      flash[:submission_note] = params[:submission_note]
    end
    
    # Redirect back to the new path.
    redirect_to new_video_game_continuum_path
  end

  # Shows the edit page for an VideoGameContinuum.
  def edit
    @video_game_continuum = VideoGameContinuum.find params[:id]
    @title = "Genericon Trivia System - Edit Video Game Continuum - #{@video_game_continuum.name}"
  end
  
  # Adds an alternate name to an VideoGameContinuum.
  def add_alternate_name
    @video_game_continuum = VideoGameContinuum.find params[:id]
    
    # Moderators automatically add the alternate name, while approvers add it as a pending change.
    if current_user.has_role? 'video_game_continuum_moderator'
      @video_game_continuum.alternate_names << params[:new_alternate_name]
    elsif current_user.has_role? 'video_game_continuum_approver'
      @video_game_continuum.add_pending_change_support :alternate_names, :add, params[:new_alternate_name]
    end
    if @video_game_continuum.save
      flash[:notice] = "Successfully submitted the alternate name."
    else
      # Render the errors, and retain the values input previously for next time.
      flash[:error] = render_to_string :inline => "<%= error_messages_for 'video_game_continuum' %>"
      flash[:error_is_html] = true
    end
    
    redirect_to edit_video_game_continuum_path, params[:id]
  end
  
  # Removes an alternate_name from an VideoGameContinuum.
  def remove_alternate_name
    @video_game_continuum = VideoGameContinuum.find params[:id]
    
    # Moderators automatically remove the alternate name, while approvers add it as a pending change.
    if @video_game_continuum.alternate_names.include? params[:name]
      if current_user.has_role? 'video_game_continuum_moderator'
        @video_game_continuum.alternate_names.delete params[:name]
      elsif current_user.has_role? 'video_game_continuum_approver'
        @video_game_continuum.add_pending_change_support :alternate_names, :remove, params[:name]
      end
      if @video_game_continuum.save
        flash[:notice] = "Successfully removed the alternate name."
      else
        # Render the errors, and retain the values input previously for next time.
        flash[:error] = render_to_string :inline => "<%= error_messages_for 'video_game_continuum' %>"
        flash[:error_is_html] = true
      end
    end
    
    redirect_to edit_video_game_continuum_path, params[:id]
  end
  
  # Approves a pending change for an VideoGameContinuum.
  def approve_pending_change
    @video_game_continuum = VideoGameContinuum.find params[:id]
    
    # Moderators commit the change.
    if current_user.has_role? 'video_game_continuum_moderator'
      @video_game_continuum.apply_pending_change params[:key].to_sym, params[:change_action].to_sym, params[:value]
    # Everyone else (read: approvers) just has their support added.
    else
      @video_game_continuum.add_pending_change_support params[:key].to_sym, params[:change_action].to_sym, params[:value]
    end
    if @video_game_continuum.save
      flash[:notice] = "Successfully approved the pending change."
    else
      # Render the errors, and retain the values input previously for next time.
      flash[:error] = render_to_string :inline => "<%= error_messages_for 'video_game_continuum' %>"
      flash[:error_is_html] = true
    end
    
    redirect_to edit_video_game_continuum_path, params[:id]
  end
  
  # Rejects a pending change for an VideoGameContinuum.
  def reject_pending_change
    @video_game_continuum = VideoGameContinuum.find params[:id]
    
    # Moderators need to delete the pending change entirely
    if current_user.has_role? 'video_game_continuum_moderator'
      @video_game_continuum.delete_pending_change params[:key].to_sym, params[:change_action].to_sym, params[:value]
    # Everyone else (read: approvers) just has their support (if any) removed.
    else
      @video_game_continuum.remove_pending_change_support params[:key].to_sym, params[:change_action].to_sym, params[:value]
    end
    if @video_game_continuum.save
      flash[:notice] = "Successfully rejected the pending change."
    else
      # Render the errors, and retain the values input previously for next time.
      flash[:error] = render_to_string :inline => "<%= error_messages_for 'video_game_continuum' %>"
      flash[:error_is_html] = true
    end
    
    redirect_to edit_video_game_continuum_path, params[:id]
  end

  # Destroys an VideoGameContinuum.
  def destroy
    # I sure hope you added a warning.
    # TODO: block when there are dependencies.
    
    if VideoGameContinuum.destroy params[:id]
      flash[:notice] = "Successfully deleted the video game continuum."
    else
      flash[:error] = "There was a problem deleting that video game continuum."
    end
    
    redirect_to manage_video_game_continuum_path
  end

  # Shows the list of VideoGameContinuums to be managed.
  def manage
    # TODO: Here, we order by whether or not the continuum is approved.
    @video_game_continuums = VideoGameContinuum.paginate :all, :page => params[:page], :select => "*, CASE state WHEN 'pending_approval' THEN 1 WHEN 'pending_changes' THEN 2 WHEN 'approved' THEN 3 END AS state_id", :order => 'state_id ASC, approved_at DESC, created_at DESC, name ASC'
    @title = 'Genericon Trivia System - Video Game Continuum Management'
  end

  # Approve an VideoGameContinuum.
  def approve
    @video_game_continuum = VideoGameContinuum.find(params[:id])
    
    if @video_game_continuum.pending_approval? and @video_game_continuum.approvable_changes?
      if @video_game_continuum.approve!
        flash[:notice] = "Successfully approved the video game continuum."
      else
        flash[:error] = render_to_string :inline => "<%= error_messages_for 'video_game_continuum' %>"
        flash[:error_is_html] = true
      end
    elsif @video_game_continuum.pending_changes? and @video_game_continuum.approvable_changes?
      if @video_game_continuum.approve_changes!
        flash[:notice] = "Successfully approved the video game continuum changes."
      else
        flash[:error] = render_to_string :inline => "<%= error_messages_for 'video_game_continuum' %>"
        flash[:error_is_html] = true
      end
    else
      flash[:error] = render_to_string :inline => "<%= error_messages_for 'video_game_continuum' %>"
      flash[:error_is_html] = true
    end
    
    redirect_to edit_video_game_continuum_path, params[:id]
  end
  
  # Select the destination for a merge_into operation.
  def select_merge
    @video_game_continuum = VideoGameContinuum.find(params[:id])
    @title = 'Genericon Trivia System - Merge Video Game Continuums'
  end
  
  # Merge the given VideoGameContinuum into the destination VideoGameContinuum.
  def merge_into
    @src = VideoGameContinuum.find(params[:id])
    @dest = VideoGameContinuum.find(params[:dest])
    
    if @src.merge_into! @dest
      flash[:notice] = "Successfully merged the video game continuums."
    else
      flash[:error] = "Could not merge the video game continuums."
      flash[:error_is_html] = false
    end
    
    redirect_to manage_video_game_continuum_path
  end
  
  # Return the list of video game continuums to merge into.
  def select_merge_search
    find_options = {
      :conditions => [ "LOWER(name) LIKE ?", '%' + params[:name].downcase + '%' ],
      :order => "name ASC",
      :limit => 10 }
    
    @main = VideoGameContinuum.find(params[:id])
    @items = VideoGameContinuum.find(:all, find_options)
    
    if params[:name].empty?
      merge_list = ''
    else
      merge_list = render_to_string :partial => 'select_merge_search'
    end
    
    render :update do |page|
      VideoGameContinuum.query_cache.clear_query_cache if VideoGameContinuum.method_defined? :query_cache
      page.replace_html("merge_options", merge_list)
    end
  end

end
