# Controls VideoGame pages.
class VideoGameController < ApplicationController
  before_filter :login_required, :except => [:index, :list, :show, :new,
                                             :create, :auto_complete_for_video_game_continuum_name,
                                             :auto_complete_for_video_game_system_name,
                                             :find_genres_for_game,
#                                             :find_similar_genres_to_video_game,
                                             :search_select_merge_video_game]
  after_filter :store_location, :only => [:index, :list, :show, :new, :edit,
                                          :manage, :select_merge]
  
  require_role 'video_game_approver', :for_all_except => [:index, :list,
                                                          :show, :new,
                                                          :create, :auto_complete_for_video_game_continuum_name,
                                                          :auto_complete_for_video_game_system_name,
                                                          :find_genres_for_game,
#                                                          :find_similar_genres_to_video_game,
                                                          :search_select_merge_video_game]
  require_role 'video_game_moderator', :for => [:destroy, :select_merge,
                                                :merge_into]
  
  in_place_edit_with_validation_for :video_game, :name
  in_place_edit_with_validation_for :video_game, :year
  
  # Custom validation for setting video game continuum name.
  def set_video_game_continuum_name
    @item = VideoGame.find(params[:id])
    @video_game_continuum = VideoGameContinuum.find_by_name params[:video_game_continuum][:name]
    if @video_game_continuum != nil
      @item.video_game_continuum = @video_game_continuum
      @item.save
      render :update do |page| 
        page.replace_html("video_game_continuum_name_#{params[:id]}_in_place_editor", 
                          @item.video_game_continuum.name)
      end
    else
      render :update do |page|
        page.alert('Must have a pre-existing video game continuum set.')
        VideoGame.query_cache.clear_query_cache if VideoGame.method_defined? :query_cache
        VideoGameContinuum.query_cache.clear_query_cache if VideoGameContinuum.method_defined? :query_cache
        @item.reload
        page.replace_html("video_game_continuum_name_#{params[:id]}_in_place_editor", 
                          @item.video_game_continuum.name)
      end
    end
  end
  
  # Custom validation for setting video game system name.
  def set_video_game_system_name
    @item = VideoGame.find(params[:id])
    @video_game_system = VideoGameSystem.find_by_name params[:video_game_system][:name]
    if @video_game_system != nil
      @item.video_game_system = @video_game_system
      @item.save
      render :update do |page| 
        page.replace_html("video_game_system_name_#{params[:id]}_in_place_editor", 
                          @item.video_game_system.name)
      end
    else
      render :update do |page|
        page.alert('Must have a pre-existing video game system set.')
        VideoGame.query_cache.clear_query_cache if VideoGame.method_defined? :query_cache
        VideoGameSystem.query_cache.clear_query_cache if VideoGameSystem.method_defined? :query_cache
        @item.reload
        page.replace_html("video_game_system_name_#{params[:id]}_in_place_editor", 
                          @item.video_game_system.name)
      end
    end
  end
  
  # Renders a list of VideoGames.
  def index
    list
    render :action => :list
  end

  # Render a list of VideoGames.
  def list
    @video_games = VideoGame.paginate :all, :page => params[:page], :order => 'name ASC'
    @title = 'Genericon Trivia System - Video Game List'
  end

  # Render the details of an VideoGames for normal users.
  def show
    @video_game = VideoGame.find params[:id]
    @title = "Genericon Trivia System - Video Game Details - #{@video_game.name}"
  end

  # Render the screen to insert a new VideoGame.
  def new
    # We reference flash[] in case we've been redirected following an error.
    @video_game = VideoGame.new flash[:video_game]
    @genre_checkbox = { }
    @genre_checkbox.default = '0'
    @genre_checkbox.merge! flash[:genre_checkbox] if !flash[:genre_checkbox].nil?
    @video_game_continuum = VideoGameContinuum.new flash[:video_game_continuum]
    @video_game_system = VideoGameSystem.new flash[:video_game_system]
    @submission_note = flash[:submission_note]
    @title = 'Genericon Trivia System - New Video Game'
  end

  # Create a new VideoGame and redirect back to new.
  def create
    # Clean up the parameters, like year and alternate_names
    if params[:video_game][:year].empty?
      params[:video_game][:year] = nil
    end
    
    # Translate the alternate_names hash to an array...
    params[:video_game][:alternate_names] = params[:video_game][:alternate_names].keys.sort.collect { |key| params[:video_game][:alternate_names][key] }
    # Remove duplicate alternate_names.
    params[:video_game][:alternate_names] = params[:video_game][:alternate_names].uniq
    # Remove empty alternate_names
    params[:video_game][:alternate_names] = params[:video_game][:alternate_names].reject { |name| name.nil? or name.empty? }
	
    # Validate existence of at least one checked genre.  We can bypass this if
    # we WANT, but we don't want to.
    has_one_genre = false
    params[:genre_checkbox].each do |genre, value|
      if value == '1'
        has_one_genre = true
        break
      end
    end
    
    # Load the continuum
    create_continuum = false
    if !params[:video_game_continuum][:name].empty?
      @video_game_continuum = VideoGameContinuum.find_by_name(params[:video_game_continuum][:name])
      # If we can't load it, create it.
      if @video_game_continuum == nil
        @video_game_continuum = VideoGameContinuum.new(params[:video_game_continuum])
        create_continuum = true
      end
    else
      @video_game_continuum = nil
    end
    
    # Load the system
    create_system = false
    if !params[:video_game_system][:name].empty?
      @video_game_system = VideoGameSystem.find_by_name(params[:video_game_system][:name])
      # If we can't load it, create it.
      if @video_game_system == nil
        @video_game_system = VideoGameSystem.new(params[:video_game_system])
        create_system = true
      end
    else
      @video_game_system = nil
    end
    
    # Create the VideoGame.
    @video_game = VideoGame.new params[:video_game]
    @video_game.video_game_continuum = @video_game_continuum
    @video_game.video_game_system = @video_game_system

    params[:genre_checkbox].each do |genre, value|
      if value == '1'
        @video_game.video_game_genres << VideoGameGenre.find_by_underscored_name(genre)
      end
    end
    
    # Create the submission note if there was one.
    if !params[:submission_note].nil? and !params[:submission_note].empty?
      comment = @video_game.comments.build :item => @video_game, :text => params[:submission_note], :comment_type => :submission_note
    end
    
    @video_game_system.valid? if !@video_game_system.nil?
    @video_game_continuum.valid? if !@video_game_continuum.nil?
    @video_game.valid?
    
    if has_one_genre
      success = true
      VideoGame.transaction do
        VideoGameContinuum.transaction do
          VideoGameSystem.transaction do
            Comment.transaction do
              success &&= @video_game_continuum.save if !@video_game_continuum.nil?
              success &&= @video_game_system.save if !@video_game_system.nil?
              success &&= @video_game.save
              success &&= comment.save if !params[:submission_note].nil? and !params[:submission_note].empty?
            end
          end
        end
      end
    else
      @video_game.errors.add 'genres', '-- you must select at least one'
    end
    
    # Save and redirect.
    if success
      flash[:notice] = "Successfully submitted your video game."
      
      # Auto approve if we're logged in as an approver.
      if current_user.has_role? 'video_game_approver'
        @video_game.approve!
      end
      if !@video_game_continuum.nil? and current_user.has_role? 'video_game_continuum_approver'
        @video_game_continuum.approve!
      end
      if !@video_game_system.nil? and current_user.has_role? 'video_game_system_approver'
        @video_game_system.approve!
      end
    else
      # Render the errors, and retain the values input previously for next time.
      flash[:error] = render_to_string :inline => "<%= error_messages_for 'video_game' %>"
      flash[:error_is_html] = true
      flash[:video_game] = params[:video_game]
      flash[:genre_checkbox] = params[:genre_checkbox]
      flash[:video_game_continuum] = params[:video_game_continuum]
      flash[:video_game_system] = params[:video_game_system]
      flash[:submission_note] = params[:submission_note]
    end
    
    # Redirect back to the new path.
    redirect_to new_video_game_path
  end

  # Shows the edit page for a VideoGame.
  def edit
    @video_game = VideoGame.find params[:id]
    @video_game_continuum = @video_game.video_game_continuum || VideoGame.new()
    @video_game_system = @video_game.video_game_system || VideoGame.new()
    @title = "Genericon Trivia System - Edit Video Game - #{@video_game.name}"
  end
  
  # Adds an alternate name to a VideoGame.
  def add_alternate_name
    @video_game = VideoGame.find params[:id]
    
    # Moderators automatically add the alternate name, while approvers add it as a pending change.
    if current_user.has_role? 'video_game_moderator'
      @video_game.alternate_names << params[:new_alternate_name]
    elsif current_user.has_role? 'video_game_approver'
      @video_game.add_pending_change_support :alternate_names, :add, params[:new_alternate_name]
    end
    if @video_game.save
      flash[:notice] = "Successfully submitted the alternate name."
    else
      # Render the errors, and retain the values input previously for next time.
      flash[:error] = render_to_string :inline => "<%= error_messages_for 'video_game' %>"
      flash[:error_is_html] = true
    end
    
    redirect_to edit_video_game_path, params[:id]
  end
  
  # Removes an alternate name from a VideoGame.
  def remove_alternate_name
    @video_game = VideoGame.find params[:id]
    
    # Moderators automatically remove the alternate name, while approvers add it as a pending change.
    if @video_game.alternate_names.include? params[:name]
      if current_user.has_role? 'video_game_moderator'
        @video_game.alternate_names.delete params[:name]
      elsif current_user.has_role? 'video_game_approver'
        @video_game.add_pending_change_support :alternate_names, :remove, params[:name]
      end
      if @video_game.save
        flash[:notice] = "Successfully removed the alternate name."
      else
        # Render the errors, and retain the values input previously for next time.
        flash[:error] = render_to_string :inline => "<%= error_messages_for 'video_game' %>"
        flash[:error_is_html] = true
      end
    end
    
    redirect_to edit_video_game_path, params[:id]
  end
  
  # Adds an video game genre to a VideoGame.
  def add_video_game_genre
    if params[:new_video_game_genre] == ''
      redirect_to edit_video_game_path, params[:id]
      return
    end
    
    @video_game = VideoGame.find params[:id]
    
    # Moderators automatically add the video game genre, while approvers add it as a pending change.
    if current_user.has_role? 'video_game_moderator'
      @video_game.video_game_genres << VideoGameGenre.find(params[:new_video_game_genre])
    elsif current_user.has_role? 'video_game_approver'
      @video_game.add_pending_change_support :video_game_genres, :add, params[:new_video_game_genre]
    end
    if @video_game.save
      flash[:notice] = "Successfully submitted the genre."
    else
      # Render the errors, and retain the values input previously for next time.
      flash[:error] = render_to_string :inline => "<%= error_messages_for 'video_game' %>"
      flash[:error_is_html] = true
    end
    
    redirect_to edit_video_game_path, params[:id]
  end
  
  # Removes an VideoGameGenre from a VideoGame.
  def remove_video_game_genre
    @video_game = VideoGame.find params[:id]
    
    # Moderators automatically remove the genre, while approvers add it as a pending change.
    if @video_game.video_game_genres.include? VideoGameGenre.find(params[:genre_id])
      if current_user.has_role? 'video_game_moderator'
        @video_game.video_game_genres.delete VideoGameGenre.find(params[:genre_id])
      elsif current_user.has_role? 'video_game_approver'
        @video_game.add_pending_change_support :video_game_genres, :remove, params[:genre_id]
      end
      if @video_game.save
        flash[:notice] = "Successfully removed the genre."
      else
        # Render the errors, and retain the values input previously for next time.
        flash[:error] = render_to_string :inline => "<%= error_messages_for 'video_game' %>"
        flash[:error_is_html] = true
      end
    end
    
    redirect_to edit_video_game_path, params[:id]
  end
  
  # Approves a pending change for a VideoGame.
  def approve_pending_change
    @video_game = VideoGame.find params[:id]
    
    # Moderators commit the change.
    if current_user.has_role? 'video_game_moderator'
      @video_game.apply_pending_change params[:key].to_sym, params[:change_action].to_sym, params[:value]
    # Everyone else (read: approvers) just has their support added.
    else
      @video_game.add_pending_change_support params[:key].to_sym, params[:change_action].to_sym, params[:value]
    end
    if @video_game.save
      flash[:notice] = "Successfully approved the pending change."
    else
      # Render the errors, and retain the values input previously for next time.
      flash[:error] = render_to_string :inline => "<%= error_messages_for 'video_game' %>"
      flash[:error_is_html] = true
    end
    
    redirect_to edit_video_game_path, params[:id]
  end
  
  # Rejects a pending change for a VideoGame.
  def reject_pending_change
    @video_game = VideoGame.find params[:id]
    
    # Moderators need to delete the pending change entirely
    if current_user.has_role? 'video_game_moderator'
      @video_game.delete_pending_change params[:key].to_sym, params[:change_action].to_sym, params[:value]
    # Everyone else (read: approvers) just has their support (if any) removed.
    else
      @video_game.remove_pending_change_support params[:key].to_sym, params[:change_action].to_sym, params[:value]
    end
    if @video_game.save
      flash[:notice] = "Successfully rejected the pending change."
    else
      # Render the errors, and retain the values input previously for next time.
      flash[:error] = render_to_string :inline => "<%= error_messages_for 'video_game' %>"
      flash[:error_is_html] = true
    end
    
    redirect_to edit_video_game_path, params[:id]
  end

  # Destroys a VideoGame.
  def destroy
    # I sure hope you added a warning.
    # TODO: block when there are dependencies.
    
    if VideoGame.destroy params[:id]
      flash[:notice] = "Successfully deleted the video game."
    else
      flash[:error] = "There was a problem deleting that video game."
    end
    
    redirect_to manage_video_game_path
  end

  # Shows the list of VideoGames to be managed.
  def manage
    # TODO: Here, we order by whether or not the series is approved.
    @video_games = VideoGame.paginate :all, :page => params[:page], :select => "*, CASE state WHEN 'pending_approval' THEN 1 WHEN 'pending_changes' THEN 2 WHEN 'approved' THEN 3 END AS state_id", :order => 'state_id ASC, approved_at DESC, created_at DESC, name ASC'
    @title = 'Genericon Trivia System - Video Game Management'
  end

  # Approve a VideoGame.
  def approve
    @video_game = VideoGame.find(params[:id])
    
    if @video_game.pending_approval? and @video_game.approvable_changes?
      if @video_game.approve!
        flash[:notice] = "Successfully approved video game."
      else
        flash[:error] = render_to_string :inline => "<%= error_messages_for 'video_game' %>"
        flash[:error_is_html] = true
      end
    elsif @video_game.pending_changes? and @video_game.approvable_changes?
      if @video_game.approve_changes!
        flash[:notice] = "Successfully approved video game changes."
      else
        flash[:error] = render_to_string :inline => "<%= error_messages_for 'video_game' %>"
        flash[:error_is_html] = true
      end
    else
      flash[:error] = render_to_string :inline => "<%= error_messages_for 'video_game' %>"
      flash[:error_is_html] = true
    end
    
    redirect_to edit_video_game_path, params[:id]
  end

  # Select the destination for a merge_into operation.
  def select_merge
    @video_game = VideoGame.find(params[:id])
    @title = 'Genericon Trivia System - Merge Video Game'
  end
  
  # Merge the given VideoGame into the destination VideoGame.
  def merge_into
    @src = VideoGame.find(params[:id])
    @dest = VideoGame.find(params[:dest])
    
    if @src.merge_into! @dest
      flash[:notice] = "Successfully merged the video game."
    else
      flash[:error] = "Could not merge the video game."
      flash[:error_is_html] = false
    end
    
    redirect_to manage_video_game_path
  end
  
  # Return the list of video games to merge into.
  def select_merge_search
    find_options = {
      :conditions => [ "LOWER(name) LIKE ?", '%' + params[:name].downcase + '%' ],
      :order => "name ASC",
      :limit => 10 }
    
    @main = VideoGame.find(params[:id])
    @items = VideoGame.find(:all, find_options)
    
    if params[:name].empty?
      merge_list = ''
    else
      merge_list = render_to_string :partial => 'select_merge_search'
    end
    
    render :update do |page|
      VideoGame.query_cache.clear_query_cache if VideoGame.method_defined? :query_cache
      page.replace_html("merge_options", merge_list)
    end
  end

end
