# Controls VideoGameGenre pages.
class VideoGameGenreController < ApplicationController
  before_filter :login_required, :except => [:index, :list]
  after_filter :store_location, :only => [:index, :list, :edit, :manage]
  
  require_role 'video_game_genre_moderator', :for_all_except => [:index, :list]
  
  in_place_edit_with_validation_for :video_game_genre, :name
  in_place_edit_with_validation_for :video_game_genre, :description

  # Handles setting the parent of a VideoGameGenre.
  def set_video_game_genre_parent_id
    @video_game_genre = VideoGameGenre.find(params[:id])
    if params[:parent_id].empty?
      @video_game_genre.parent = nil
    else
      @video_game_genre.parent = VideoGameGenre.find(params[:parent_id])
    end
    
    if @video_game_genre.save
      return head(:ok)
    else
      return head(:internal_server_error)
    end
  end
  
  # Renders a list of VideoGameGenres.
  def index
    list
    render :action => :list
  end

  # Render a list of VideoGameGenres.
  def list
    @video_game_genres = VideoGameGenre.paginate :all, :page => params[:page], :order => 'name ASC'
    @title = 'Genericon Trivia System - Video Game Genres List'
  end

  # Render the screen to insert a new VideoGameGenre.
  def new
    # We reference flash[] in case we've been redirected following an error.
    @video_game_genre = VideoGameGenre.new flash[:video_game_genre]
    @title = 'Genericon Trivia System - New Video Game Genre'
  end

  # Create a new VideoGameGenre and redirect back to new.
  def create
    # Create the VideoGameGenre.
    @video_game_genre = VideoGameGenre.new params[:video_game_genre]
    
    # Save and redirect.
    if @video_game_genre.save
      flash[:notice] = "Successfully submitted your video game genre."
    else
      # Render the errors, and retain the values input previously for next time.
      flash[:error] = render_to_string :inline => "<%= error_messages_for 'video_game_genre' %>"
      flash[:error_is_html] = true
      flash[:video_game_genre] = params[:video_game_genre]
    end
    
    # Redirect back to the new path.
    redirect_to new_video_game_genre_path
  end

  # Shows the edit page for an VideoGameGenre.
  def edit
    @video_game_genre = VideoGameGenre.find params[:id]
    @title = "Genericon Trivia System - Edit Video Game Genre - #{@video_game_genre.name}"
  end
  
  # Destroys an VideoGameGenre.
  def destroy
    # I sure hope you added a warning.
    
    if VideoGameGenre.destroy params[:id]
      flash[:notice] = "Successfully deleted the video game genre."
    else
      flash[:error] = "There was a problem deleting that video game genre."
    end
    
    redirect_to manage_video_game_genre_path
  end

  # Shows the list of VideoGameGenres to be managed.
  def manage
    # TODO: Here, we order by whether or not the genre is approved.
    @video_game_genres = VideoGameGenre.paginate :all, :page => params[:page], :order => 'name ASC'
    @title = 'Genericon Trivia System - Video Game Genre Management'
  end
  
  # Select the destination for a merge_into operation.
  def select_merge
    @video_game_genre = VideoGameGenre.find(params[:id])
    @title = 'Genericon Trivia System - Merge Video Game Genres'
  end
  
  # Merge the given VideoGameGenre into the destination VideoGameGenre.
  def merge_into
    @src = VideoGameGenre.find(params[:id])
    @dest = VideoGameGenre.find(params[:dest])
    
    if @src.merge_into! @dest
      flash[:notice] = "Successfully merged the video game genres."
    else
      flash[:error] = "Could not merge the video game genres."
      flash[:error_is_html] = false
    end
	
    redirect_to manage_video_game_genre_path
  end
  
  # Return the list of video game genres to merge into.
  def select_merge_search
    find_options = {
      :conditions => [ "LOWER(name) LIKE ?", '%' + params[:name].downcase + '%' ],
      :order => "name ASC",
      :limit => 10 }
    
    @main = VideoGameGenre.find(params[:id])
    @items = VideoGameGenre.find(:all, find_options)
    
    if params[:name].empty?
      merge_list = ''
    else
      merge_list = render_to_string :partial => 'select_merge_search'
    end
    
    render :update do |page|
      VideoGameGenre.query_cache.clear_query_cache if VideoGameGenre.method_defined? :query_cache
      page.replace_html("merge_options", merge_list)
    end
  end

end
