require 'zip/zip'
require 'ftools'
require 'mime/types'

class ZipFileWrapper
  def initialize entry
    @entry = entry
  end
  
  def original_filename
    File.basename(@entry.name)
  end
  
  def content_type
    MIME::Types.type_for(self.original_filename)
  end
  
  def size
    @entry.size
  end
  
  def to_tempfile
    tempfile = Tempfile.new("stream")
    tempfile.binmode
    tempfile.write(@entry.get_input_stream.read)
    tempfile.rewind
    tempfile
  end
  
  def inspect
    "#<ZipFileWrapper: name=#{@entry.name}>"
  end
  
  def not_image?
    tempfile = Tempfile.new("stream")
    tempfile.binmode
    tempfile.write(@entry.get_input_stream.read)
    tempfile.rewind
    type = begin
             Paperclip.run("identify", %Q[-format "%wx%h" "#{tempfile.path}"])
           rescue Paperclip::PaperclipCommandLineError
             "identify: no decode delegate for this image format"
           end
    type.match(/^identify: no decode delegate for this image format/)
  end
end

# Controls VideoGameScene pages.
class VideoGameSceneController < ApplicationController
  before_filter :login_required, :except => [:index, :list, :show, :new, :new_zip,
                                             :create, :create_zip,
                                             :auto_complete_for_video_game_name,
                                             :auto_complete_for_video_game_system_name,
					     :find_system_for_game,
                                             :find_genres_for_game,
#                                             :find_similar_genres_to_anime,
                                             :play, :image, :randomize_play_window,
                                             :new_game,
					     :remove_video_game_from_game,
       					     :set_game_history_entry,
					     :set_max_from_game]
  before_filter :clear_unused, :except => [:play, :image, :randomize_play_window,
                                           :new_game, :remove_video_game_from_game,
					   :set_game_history_entry,
					   :set_max_from_game]
  after_filter :store_location, :only => [:index, :list, :show, :new, :new_zip, :edit,
                                          :manage, :play, :new_game]
  
  require_role 'video_game_scene_approver', :for_all_except => [:index, :list,
                                                           :show, :new, :new_zip,
                                                           :create, :create_zip,
                                                           :auto_complete_for_video_game_name,
                                                           :auto_complete_for_video_game_system_name,
					     		   :find_system_for_game,
                                                           :find_genres_for_game,
#                                                           :find_similar_genres_to_anime,
                                                           :play, :image, :randomize_play_window,
                                                           :new_game,
							   :remove_video_game_from_game,
							   :set_game_history_entry,
					   		   :set_max_from_game]
  require_role 'video_game_scene_moderator', :for => [:destroy]
  
  in_place_edit_for :video_game_scene, :location
  
  # Cleans the unused questions on exit.
  def clear_unused
    GameSession.clean_sessions(session[:unused]) if session[:unused]
    session[:unused] = nil
  end

  # Custom validation for setting video game name.
  def set_video_game_name
    @item = VideoGameScene.find(params[:id])
    @video_game = VideoGame.find_by_name params[:video_game][:name]
    if @video_game != nil
      @item.video_game = @video_game
      @item.save
      render :update do |page| 
        page.replace_html("video_game_name_#{params[:id]}_in_place_editor", 
                          @item.video_game.name)
      end
    else
      render :update do |page|
        page.alert('Must have a pre-existing video game set.')
        VideoGameScene.query_cache.clear_query_cache if VideoGameScene.method_defined?:query_cache
        VideoGame.query_cache.clear_query_cache if VideoGame.method_defined?:query_cache
        @item.reload
        page.replace_html("video_game_name_#{params[:id]}_in_place_editor", 
                          @item.video_game.name)
      end
    end
  end
  
  # Renders a new VideoGameScene acceptance page.
  def index
    new
    render :action => :new
  end

  # Render a list of VideoGameScenes.
  def list
    if !params[:letter].nil?
      params[:page] = (VideoGameScene.count(:conditions => ["LOWER(`video_games`.name) < LOWER(?)", params[:letter]], :joins => 'INNER JOIN `video_games` ON `video_game_scenes`.video_game_id=`video_games`.id') / VideoGameScene.per_page + 1).to_i
    end
    params.delete :letter
    @video_game_scenes = VideoGameScene.paginate :all, :page => params[:page], :select => '`video_game_scenes`.*, `video_games`.name', :joins => 'INNER JOIN `video_games` ON `video_game_scenes`.video_game_id=`video_games`.id', :order => '`video_games`.name ASC, `video_game_scenes`.created_at ASC, `video_game_scenes`.id ASC'
    @title = 'Genericon Trivia System - Name-That-Video Game Scene List'
  end

  # Render the details of an VideoGameScene for normal users.
  def show
    @video_game_scene = VideoGameScene.find params[:id]
    @video_game = @video_game_scene.video_game
    @title = "Genericon Trivia System - Name-That-Video Game Scene Details"
  end

  # Render the screen to insert a new VideoGameScene.
  def new
    # We reference flash[] in case we've been redirected following an error.
    @video_game_scene = VideoGameScene.new flash[:video_game_scene]
    @video_game = VideoGame.new flash[:video_game]
    @video_game_system = VideoGameSystem.new flash[:video_game_system]
    @genre_checkbox = { }
    @genre_checkbox.default = '0'
    @genre_checkbox.merge! flash[:genre_checkbox] if !flash[:genre_checkbox].nil?
    @submission_note = flash[:submission_note]
    @title = 'Genericon Trivia System - New Name-That-Video Game Scene'
  end

  # Render the screen to insert a group of new VideoGameScenes.
  def new_zip
    # We reference flash[] in case we've been redirected following an error.
    @video_game_scene = VideoGameScene.new flash[:video_game_scene]
    @video_game = VideoGame.new flash[:video_game]
    @video_game_system = VideoGameSystem.new flash[:video_game_system]
    @genre_checkbox = { }
    @genre_checkbox.default = '0'
    @genre_checkbox.merge! flash[:genre_checkbox] if !flash[:genre_checkbox].nil?
    @submission_note = flash[:submission_note]
    @title = 'Genericon Trivia System - New Name-That-Video Game Scene Batch Upload'
  end

  # Create a new VideoGameScene and redirect back to new.
  def create
    # Validate existence of at least one checked genre.  We can bypass this if
    # we WANT, but we don't want to.
    has_one_genre = false
    params[:genre_checkbox].each do |genre, value|
      if value == '1'
        has_one_genre = true
        break
      end
    end
    
    # Load the series
    create_series = false
    @video_game = VideoGame.find_by_name(params[:video_game][:name])
    # If we can't load it, create it.
    if @video_game == nil
      @video_game = VideoGame.new(params[:video_game])
      params[:genre_checkbox].each do |genre, value|
        if value == '1'
          @video_game.video_game_genres << VideoGameGenre.find_by_underscored_name(genre)
        end
      end
      create_series = true
    end
    
    # Check for changes in the genres
    VideoGameGenre.find(:all).each do |genre|
      # If we have a difference between genres, note the potential change.
      if (params[:genre_checkbox][genre.underscored_name] == '1') != @video_game.video_game_genres.include?(genre)
        if @video_game.video_game_genres.include? genre
          @video_game.add_pending_change_support :video_game_genres, :remove, genre.id
        else
          @video_game.add_pending_change_support :video_game_genres, :add, genre.id
        end
      end
    end
    
    # Load the system
    create_system = false
    @video_game_system = VideoGameSystem.find_by_name(params[:video_game_system][:name])
    # If we can't load it, create it.
    if @video_game_system == nil
      @video_game_system = VideoGameSystem.new(params[:video_game_system])
      create_system = true
    end
    
    @video_game_scene = VideoGameScene.new params[:video_game_scene]
    @video_game_scene.video_game = @video_game

    # Create the submission note if there was one.
    if !params[:submission_note].nil? and !params[:submission_note].empty?
      comment = @video_game_scene.comments.build :item => @video_game_scene, :text => params[:submission_note], :comment_type => :submission_note
    end
    
    # Run validation checks to get full list of errors...
    @video_game.valid?
    @video_game_system.valid?
    @video_game_scene.valid?
    
    if has_one_genre
      success = true
      VideoGameScene.transaction do
        VideoGameSystem.transaction do
          success &&= @video_game_system.save
	  if success && @video_game.video_game_system != @video_game_system
            if @video_game.video_game_system.nil?
              @video_game.video_game_system = @video_game_system
            else
              @video_game.add_pending_change_support :video_game_system, :change, @video_game_system.id
            end
          end
          VideoGame.transaction do
            success &&= @video_game.save
            success &&= @video_game_scene.save
          end
        end
      end
    else
      @video_game.errors.add 'genres', '-- you must select at least one'
    end

    # Save and redirect.
    if success
      flash[:notice] = "Successfully submitted your Name-That-Video Game scene."
      
      # Auto approve if we're logged in as an approver.
      if current_user.has_role? 'video_game_scene_approver'
        @video_game_scene.approve!
      end
      if current_user.has_role? 'video_game_approver'
        @video_game.approve!
      end
    else
      # Render the errors, and retain the values input previously for next time.
      flash[:error] = render_to_string :inline => "<%= error_messages_for 'video_game_scene' %><%= error_messages_for 'video_game' %>"
      flash[:error_is_html] = true
      flash[:video_game_scene] = params[:video_game_scene].reject {|key, value| key == :video_game_scene}
      flash[:video_game] = params[:video_game]
      flash[:video_game_system] = params[:video_game_system]
      flash[:genre_checkbox] = params[:genre_checkbox]
      flash[:submission_note] = params[:submission_note]
    end
    
    # Redirect back to the new path.
    redirect_to new_video_game_scene_path
  end

  # Create a batch of new VideoGameScenes and redirect back to new.
  def create_zip
    # Validate existence of at least one checked genre.  We can bypass this if
    # we WANT, but we don't want to.
    has_one_genre = false
    params[:genre_checkbox].each do |genre, value|
      if value == '1'
        has_one_genre = true
        break
      end
    end
    
    # Load the series
    create_series = false
    @video_game = VideoGame.find_by_name(params[:video_game][:name])
    # If we can't load it, create it.
    if @video_game == nil
      @video_game = VideoGame.new(params[:video_game])
      params[:genre_checkbox].each do |genre, value|
        if value == '1'
          @video_game.video_game_genres << VideoGameGenre.find_by_underscored_name(genre)
        end
      end
      create_series = true
    end
    
    # Check for changes in the genres
    VideoGameGenre.find(:all).each do |genre|
      # If we have a difference between genres, note the potential change.
      if (params[:genre_checkbox][genre.underscored_name] == '1') != @video_game.video_game_genres.include?(genre)
        if @video_game.video_game_genres.include? genre
          @video_game.add_pending_change_support :video_game_genres, :remove, genre.id
        else
          @video_game.add_pending_change_support :video_game_genres, :add, genre.id
        end
      end
    end
    
    # Load the system
    create_system = false
    @video_game_system = VideoGameSystem.find_by_name(params[:video_game_system][:name])
    # If we can't load it, create it.
    if @video_game_system == nil
      @video_game_system = VideoGameSystem.new(params[:video_game_system])
      create_system = true
    end
    
    zip = params[:video_game_scene][:video_game_scene]
    params[:video_game_scene].delete :video_game_scene
    @video_game_scene = VideoGameScene.new params[:video_game_scene]

    scenes = []
    comments = []
    
    is_zip = false
    
    tempfile = Tempfile.new(zip.original_filename)
    tempfile.binmode
    tempfile.write(zip.read)
    tempfile.rewind
    tempfile.close
    
    if zip.content_type == 'application/zip'
      is_zip = true
    else
      # Some browsers don't set this type :-(
      fm = FileMagic.new(FileMagic::MAGIC_MIME)
      res = fm.file(tempfile.path)
      if res == 'application/zip'
        is_zip = true
      else
        @video_game_scene.errors.add 'video_game_scene', "^Image ZIP wasn't a zip file"
      end
      fm.close
    end
    
    if is_zip
      Zip::ZipFile::foreach(tempfile.path) {
        |entry|
	
        if entry.file?
          img = ZipFileWrapper.new(entry)
	  
	  if img.not_image?
	    next
    	  end
	  
	  # Yeah, this is because of some pass-by-ref issues in Production.
          video_game_scene = @video_game_scene.clone
          video_game_scene.video_game = @video_game
	  video_game_scene.video_game_scene = img
          
          # Create the submission note if there was one.
          if !params[:submission_note].nil? and !params[:submission_note].empty?
            comment = @video_game_scene.comments.build :item => @video_game_scene, :text => params[:submission_note], :comment_type => :submission_note
            comments << comment
          end
	  
	  scenes << video_game_scene
        end
      }
    end
    tempfile.unlink
    
    # Run validation checks to get full list of errors...
    @video_game.valid?
    @video_game_system.valid?
    scenes.each { |scene| scene.valid? }
    
    if has_one_genre and is_zip
      success = true
      scenes.each do |scene|
        VideoGameScene.transaction do
          VideoGameSystem.transaction do
            success &&= @video_game_system.save
	    if success && @video_game.video_game_system != @video_game_system
              if @video_game.video_game_system.nil?
                @video_game.video_game_system = @video_game_system
              else
                @video_game.add_pending_change_support :video_game_system, :change, @video_game_system.id
              end
            end
            VideoGame.transaction do
              success &&= @video_game.save
              Comment.transaction do
                success &&= scene.save
                if comments.size > 0
                  comment = comments.pop
                  success &&= comment.save if !params[:submission_note].nil? and !params[:submission_note].empty?
                end
              end
            end
          end
        end
      end
    elsif !has_one_genre
      @video_game.errors.add 'genres', '-- you must select at least one'
    end

    # Save and redirect.
    if success
      flash[:notice] = "Successfully submitted your Name-That-Video Game scenes."
      
      # Auto approve if we're logged in as an approver.
      if current_user.has_role? 'video_game_scene_approver'
        scenes.each do |scene|
          scene.approve!
        end
      end
      if current_user.has_role? 'video_game_approver'
        @video_game.approve!
      end
    else
      # Render the errors, and retain the values input previously for next time.
      flash[:error] = render_to_string :inline => "<%= error_messages_for 'video_game_scene' %><%= error_messages_for 'video_game' %>"
      flash[:error_is_html] = true
      flash[:video_game_scene] = params[:video_game_scene].reject {|key, value| key == :video_game_scene}
      flash[:video_game] = params[:video_game]
      flash[:video_game_system] = params[:video_game_system]
      flash[:genre_checkbox] = params[:genre_checkbox]
      flash[:submission_note] = params[:submission_note]
    end
    
    # Redirect back to the new path.
    redirect_to new_zip_video_game_scene_path
  end

  # Shows the edit page for an VideoGameScene.
  def edit
    @video_game_scene = VideoGameScene.find params[:id]
    @video_game = @video_game_scene.video_game || VideoGame.new()
    @video_game_system = @video_game.video_game_system || VideoGameSystem.new()
    @title = "Genericon Trivia System - Edit Name-That-Video Game Scene"
  end

  # Approves a pending change for an VideoGameScene.
  def approve_pending_change
    @video_game_scene = VideoGameScene.find params[:id]
    
    # Moderators commit the change.
    if current_user.has_role? 'video_game_scene_moderator'
      @video_game_scene.apply_pending_change params[:key].to_sym, params[:change_action].to_sym, params[:value]
    # Everyone else (read: approvers) just has their support added.
    else
      @video_game_scene.add_pending_change_support params[:key].to_sym, params[:change_action].to_sym, params[:value]
    end
    if @video_game_scene.save
      flash[:notice] = "Successfully approved the pending change."
    else
      # Render the errors, and retain the values input previously for next time.
      flash[:error] = render_to_string :inline => "<%= error_messages_for 'video_game_scene' %>"
      flash[:error_is_html] = true
    end
    
    redirect_to edit_video_game_scene_path, params[:id]
  end
  
  # Rejects a pending change for an VideoGameScene.
  def reject_pending_change
    @video_game_scene = VideoGameScene.find params[:id]
    
    # Moderators need to delete the pending change entirely
    if current_user.has_role? 'video_game_scene_moderator'
      @video_game_scene.delete_pending_change params[:key].to_sym, params[:change_action].to_sym, params[:value]
    # Everyone else (read: approvers) just has their support (if any) removed.
    else
      @video_game_scene.remove_pending_change_support params[:key].to_sym, params[:change_action].to_sym, params[:value]
    end
    if @video_game_scene.save
      flash[:notice] = "Successfully rejected the pending change."
    else
      # Render the errors, and retain the values input previously for next time.
      flash[:error] = render_to_string :inline => "<%= error_messages_for 'video_game_scene' %>"
      flash[:error_is_html] = true
    end
    
    redirect_to edit_video_game_scene_path, params[:id]
  end

  # Destroys an VideoGameScene.
  def destroy
    # I sure hope you added a warning.
    # TODO: block when there are dependencies.
    
    if VideoGameScene.destroy params[:id]
      flash[:notice] = "Successfully deleted the Name-That-Video Game scene."
    else
      flash[:error] = "There was a problem deleting that Name-That-Video Game scene."
    end
    
    redirect_to manage_video_game_scene_path
  end
  
  # Shows the list of VideoGameScenes to be managed.
  def manage
    order = params[:order_by] || 'state_id ASC, `video_game_scenes`.approved_at DESC, `video_game_scenes`.created_at DESC'
    @video_game_scenes = VideoGameScene.paginate :all, :page => params[:page], :select => "`video_game_scenes`.*, CASE `video_game_scenes`.state WHEN 'pending_approval' THEN 1 WHEN 'pending_changes' THEN 2 WHEN 'approved' THEN 3 END AS state_id, `video_games`.name, `ratings`.rating", :joins => "INNER JOIN `video_games` ON `video_game_scenes`.video_game_id=`video_games`.id LEFT JOIN (SELECT item_id, SUM(rating) AS rating FROM `game_history_entries` GROUP BY item_type, item_id HAVING item_type='VideoGameScene') AS `ratings` ON `video_game_scenes`.id=`ratings`.item_id", :order => order
    @order = order
    @title = 'Genericon Trivia System - Name-That-Video Game Scene Management'
  end
  
  # Approve an VideoGameScene.
  def approve
    @video_game_scene = VideoGameScene.find(params[:id])
    
    if @video_game_scene.pending_approval? and @video_game_scene.approvable_changes?
      if @video_game_scene.approve!
        flash[:notice] = "Successfully approved Name-That-Video Game scene."
      else
        flash[:error] = render_to_string :inline => "<%= error_messages_for 'video_game_scene' %>"
        flash[:error_is_html] = true
      end
    elsif @video_game_scene.pending_changes? and @video_game_scene.approvable_changes?
      if @video_game_scene.approve_changes!
        flash[:notice] = "Successfully approved Name-That-Video Game scene changes."
      else
        flash[:error] = render_to_string :inline => "<%= error_messages_for 'video_game_scene' %>"
        flash[:error_is_html] = true
      end
    else
      flash[:error] = render_to_string :inline => "<%= error_messages_for 'video_game_scene' %>"
      flash[:error_is_html] = true
    end
    
    redirect_to edit_video_game_scene_path, params[:id]
  end

  # Configure a new game.
  def new_game
    # End any active game.
    if !session[:unused].nil?
      begin
        @game_session = GameSession.find(session[:unused])
        if @game_session.game_type != 'video_game_scene'
          GameSession.clean_sessions(session[:unused])
          session[:unused] = nil
        end
      rescue ActiveRecord::RecordNotFound
        session[:unused] = nil
      end
    end
    
    # Setup the new game.
  end

  # Play Name-That-Video Game.
  def play
    # First, check the old session to make sure it's a valid one.  If we came
    # from another game, we NEED to reset.  We also need to gracefully restart
    # from a timed-out game.
    if !session[:unused].nil?
      begin
        @game_session = GameSession.find(session[:unused])
        if @game_session.game_type != 'video_game_scene'
          GameSession.clean_sessions(session[:unused])
          session[:unused] = nil
        end
      rescue ActiveRecord::RecordNotFound
        session[:unused] = nil
      end
    end
    
    # Initialize the 'unused' array of scenes.
    if session[:unused].nil?
      @unused = VideoGameScene.find(:all)
      @unused = @unused.select {|q| !q.pending_approval? and !q.video_game.pending_approval? and (q.video_game.video_game_continuum.nil? or !q.video_game.video_game_continuum.pending_approval?) and !q.video_game.video_game_system.pending_approval?}
      @unused = @unused.collect { |q| q.id }
      @window = [ ]
      @counters = { }
      @counters.default = 0
      @max_from_game = params[:max_from_game].to_i
      @max_from_game = @unused.length if @max_from_game == 0
      
      9.times do
        id = @unused.slice!(rand(@unused.length))
        @scene = VideoGameScene.find(id)
        
        # Make sure to filter by max_from_game (though this is sort of
        # irrelevant when starting.
        while @counters[@scene.video_game.id] >= @max_from_game and !@unused.empty?
          id = @unused.slice!(rand(@unused.length))
          @scene = VideoGameScene.find(id)
        end
        
        if @counters[@scene.video_game.id] < @max_from_game
          @window << id
        end
        
        if @unused.empty?
          break
        end
      end
      
      # Remember to save the current window and unused.
      @game_session = GameSession.new(:game_type => 'video_game_scene')
      @game_session.data = {:unused => @unused, :window => @window, :max_from_game => @max_from_game}
      @game_session.save
      session[:unused] = @game_session.id
    else
      @game_session = GameSession.find(session[:unused])
      @unused = @game_session.data[:unused]
      @window = @game_session.data[:window]
      @counters = { }
      @counters.default = 0
      @counters.merge!(@game_session.data[:counters]) if @game_session.data[:counters]
      @max_from_game = @game_session.data[:max_from_game]
      
      # Can keep selecting until the window is empty.
      if @window.empty?
        GameSession.clean_sessions(session[:unused])
        session[:unused] = nil
        flash[:notice] = 'All scenes have been used up.  Game over.'
        redirect_to :action => 'index'
        return
      end
    end
    
    # Remove the current id from @window and @unused and replace it in @window.
    if !params[:id].nil? and !params[:id].empty?
      id = params[:id].to_i
      @unused.delete(id)
      
      # Get the index of it in @window so we know where to insert the new one.
      index = @window.index(id)
      indices = [ ]
      while !index.nil?
        indices << index
        @window.delete_at(index)
        index = @window.index(id)
      end
      
      while !indices.empty? and !@unused.empty?
        insert_at = indices.shift
        new_id = @unused.slice!(rand(@unused.length))
        new_scene = VideoGameScene.find(new_id)
        
        # Make sure to filter by max_from_game
        while @counters[new_scene.video_game.id] >= @max_from_game and !@unused.empty?
          new_id = @unused.slice!(rand(@unused.length))
          new_scene = VideoGameScene.find(new_id)
        end
        
        if @counters[new_scene.video_game.id] < @max_from_game
          @window.insert(insert_at, new_id)
        end
        
        if @unused.empty?
          break
        end
      end
      
      # Remember to save @window and @unused.
      @game_session = GameSession.find(session[:unused])
      data = @game_session.data || { }
      data[:window] = @window
      data[:unused] = @unused
      @game_session.data = data
      @game_session.save
      
      @scene = VideoGameScene.find(id)
      
      # Create the (initially neutral) GameHistoryEntry.
      @game_history_entry = GameHistoryEntry.new :item => @scene, :rating => 0
      @game_history_entry.save
      
      # Increment the appropriate counter and remove anything still from
      # now-disallowed series, saving the counter.
      @counters[@scene.video_game.id] = @counters[@scene.video_game.id] + 1
      
      if @counters[@scene.video_game.id] >= @max_from_game
        remove_scenes = [ ]
        add_scenes = [ ]
        @window.each do |old_id|
          if VideoGameScene.find(old_id).video_game.id == @scene.video_game.id
            # Remember to remove this now-disallowed scene.
            remove_scenes << old_id
            
            # And get a new scene to replace it with
            if !@unused.empty?
              new_id = @unused.slice!(rand(@unused.length))
              new_scene = VideoGameScene.find(new_id)
              
              # Make sure to filter by max_from_game
              while @counters[new_scene.video_game.id] >= @max_from_game and !@unused.empty?
                new_id = @unused.slice!(rand(@unused.length))
                new_scene = VideoGameScene.find(new_id)
              end
              
              if @counters[new_scene.video_game.id] < @max_from_game
                add_scenes << new_id
              end
            end
          end
        end
        remove_scenes.each do |id|
          insert_at = @window.index(id)
          @window.delete_at(insert_at)
          @window.insert(insert_at, add_scenes.shift) if !add_scenes.empty?
        end
        
        # Remember to save @window and @unused
        @game_session = GameSession.find(session[:unused])
        data = @game_session.data || { }
        data[:window] = @window
        data[:unused] = @unused
        @game_session.data = data
        @game_session.save
      end
      
      @game_session = GameSession.find(session[:unused])
      data = @game_session.data || { }
      data[:counters] = @counters
      @game_session.data = data
      @game_session.save
    else
      @scene = nil
    end
    
    @window = @window.collect {|id| VideoGameScene.find(id)}
    
    @title = "Genericon Trivia System - Name-That-Video Game"
  end

  # Shows an image corresponding to a given Name-That-Video Game Scene
  def image
    @scene = VideoGameScene.find(params[:id])
    @title = "Genericon Trivia System - Name-That-Video Game"
    @simple_render = true
  end

  # Rerandomizes @window for play.
  def randomize_play_window
    @game_session = GameSession.find(session[:unused])
    @unused = @game_session.data[:unused]
    @window = @game_session.data[:window]
    @max_from_game = @game_session.data[:max_from_game]
    @counters = { }
    @counters.default = 0
    @counters.merge!(@game_session.data[:counters]) if @game_session.data[:counters]
    
    @window.each {|id| @unused << id}
    
    @window = [ ]
    9.times do
      id = @unused.slice!(rand(@unused.length))
      @scene = VideoGameScene.find(id)
      
      # Make sure to filter by max_from_game (though this is sort of
      # irrelevant when starting.
      while @counters[@scene.video_game.id] >= @max_from_game and !@unused.empty?
        id = @unused.slice!(rand(@unused.length))
        @scene = VideoGameScene.find(id)
      end
      
      if @counters[@scene.video_game.id] < @max_from_game
        @window << id
      end
      
      if @unused.empty?
        break
      end
    end
    
    # Remember to save the current window and unused.
    @game_session = GameSession.find(session[:unused])
    data = @game_session.data || { }
    data[:window] = @window
    data[:unused] = @unused
    @game_session.data = data
    @game_session.save
    
    @window = @window.collect {|id| VideoGameScene.find(id)}
    
    render :partial => 'scene_window', :object => @window
  end

  def remove_video_game_from_game
    id = params[:id]
    
    @game_session = GameSession.find(params[:session])
    @unused = @game_session.data[:unused]
    @window = @game_session.data[:window]
    
    scenes_to_remove = VideoGameScene.find(:all, :conditions => ['video_game_id = ?', id.to_i]).to_set
    scenes_to_remove.collect! {|q| q.id}
    @window.reject! {|i| scenes_to_remove.include? i}
    @unused.reject! {|i| scenes_to_remove.include? i}
    
    if @window.length < 9
      (9 - @window.length).times do
        id = @unused.slice!(rand(@unused.length))
        
        @window << id
        
        if @unused.empty?
          break
        end
      end
    end
    
    # Remember to save the current window and unused.
    data = @game_session.data || { }
    data[:window] = @window
    data[:unused] = @unused
    @game_session.data = data
    @game_session.save
    
    @window = @window.collect {|id| VideoGameScene.find(id)}
    
    render :partial => 'scene_window', :object => @window
  end


  # Set the game history entry for this scene.
  def set_game_history_entry
    id = params[:id]
    history_id = params[:history_id]
    session = params[:session]
    rating = params[:rating].to_i
    if rating < -1
      rating = -1
    elsif rating > 1
      rating = 1
    end
    
    if history_id.nil?
      game_history_entry = GameHistoryEntry.new :item => VideoGameScene.find(id), :rating => rating
    else
      game_history_entry = GameHistoryEntry.find(history_id)
      game_history_entry.rating = rating
    end
    game_history_entry.save
    
    render :text => game_history_entry.id
  end

  # Change the maximum questions from game.
  def set_max_from_game
    @game_session = GameSession.find(params[:session])
    data = @game_session.data || { }
    if params[:max_from_game].nil? or params[:max_from_game].empty? or params[:max_from_game].to_i == 0
      data[:max_from_game] = data[:unused].length
    else
      data[:max_from_game] = params[:max_from_game].to_i
    end
    @game_session.data = data
    @game_session.save
  end

end
