# Controls VideoGameTriviaQuestion pages.
class VideoGameTriviaQuestionController < ApplicationController
  before_filter :login_required, :except => [:index, :list, :show, :new,
                                             :create,
                                             :auto_complete_for_video_game_name,
                                             :auto_complete_for_video_game_system_name,
					     :find_system_for_game,
                                             :find_genres_for_game,
#                                             :find_similar_genres_to_video_game,
                                             :play, :new_game,
                                             :randomize_play_window, :next_question,
					     :remove_video_game_from_game,
					     :set_max_from_game]
  before_filter :clear_unused, :except => [:play, :randomize_play_window,
                                           :next_question,
                			   :remove_video_game_from_game,
					   :set_max_from_game]
  after_filter :store_location, :only => [:index, :list, :show, :new, :edit,
                                          :manage, :play, :new_game]
  
  require_role 'video_game_trivia_approver', :for_all_except => [:index, :list,
                                                            :show, :new,
                                                            :create,
                                                            :auto_complete_for_video_game_name,
                                                            :auto_complete_for_video_game_system_name,
                                                            :find_genres_for_game,
							    :find_system_for_game,
#                                                            :find_similar_genres_to_video_game,
                                                            :play, :new_game,
                                                            :randomize_play_window,
                                                            :next_question,
							    :remove_video_game_from_game,
							    :set_max_from_game]
  require_role 'video_game_trivia_moderator', :for => [:destroy]
  
  in_place_edit_with_validation_for :video_game_trivia_question, :question
  in_place_edit_with_validation_for :video_game_trivia_question, :answer
  
  # Cleans the unused questions on exit.
  def clear_unused
    GameSession.clean_sessions(session[:unused]) if session[:unused]
    session[:unused] = nil
  end

  # Custom validation for setting video game name.
  def set_video_game_name
    @item = VideoGameTriviaQuestion.find(params[:id])
    @video_game = VideoGame.find_by_name params[:video_game][:name]
    if @video_game != nil
      @item.video_game = @video_game
      @item.save
      render :update do |page| 
        page.replace_html("video_game_name_#{params[:id]}_in_place_editor", 
                          @item.video_game.name)
      end
    else
      render :update do |page|
        page.alert('Must have a pre-existing video game set.')
        VideoGameTriviaQuestion.query_cache.clear_query_cache if VideoGameTriviaQuestion.method_defined?:query_cache
        VideoGame.query_cache.clear_query_cache if VideoGame.method_defined?:query_cache
        @item.reload
        page.replace_html("video_game_name_#{params[:id]}_in_place_editor", 
                          @item.video_game.name)
      end
    end
  end
  
  # Custom validation for setting question type.
  def set_video_game_trivia_question_question_type
    @item = VideoGameTriviaQuestion.find(params[:id])
    @question_type = params[:value]
    @item.question_type = @question_type
    if @item.save
      head :ok
    else
      head :bad_request
    end
  end
  
  # Renders a new VideoGameTriviaQuestion acceptance page.
  def index
    new
    render :action => :new
  end

  # Render a list of VideoGameTriviaQuestions.
  def list
    if !params[:letter].nil?
      params[:page] = (VideoGameTriviaQuestion.count(:conditions => ["LOWER(`video_games`.name) < LOWER(?)", params[:letter]], :joins => 'INNER JOIN `video_games` ON `video_game_trivia_questions`.video_game_id=`video_games`.id') / VideoGameTriviaQuestion.per_page + 1).to_i
    end
    params.delete :letter
    @video_game_trivia_questions = VideoGameTriviaQuestion.paginate :all, :page => params[:page], :select => '`video_game_trivia_questions`.*, `video_games`.name', :joins => 'INNER JOIN `video_games` ON `video_game_trivia_questions`.video_game_id=`video_games`.id', :order => '`video_games`.name ASC, `video_game_trivia_questions`.created_at ASC, `video_game_trivia_questions`.id ASC'
    @title = 'Genericon Trivia System - Video Game Trivia Question List'
  end

  # Render the details of an VideoGameTriviaQuestion for normal users.
  def show
    @video_game_trivia_question = VideoGameTriviaQuestion.find params[:id]
    @video_game = @video_game_trivia_question.video_game
    @title = "Genericon Trivia System - Video Game Trivia Question Details"
  end

  # Render the screen to insert a new VideoGameTriviaQuestion.
  def new
    # We reference flash[] in case we've been redirected following an error.
    @video_game_trivia_question = VideoGameTriviaQuestion.new flash[:video_game_trivia_question]
    @video_game = VideoGame.new flash[:video_game]
    @video_game_system = VideoGameSystem.new flash[:video_game_system]
    @genre_checkbox = { }
    @genre_checkbox.default = '0'
    @genre_checkbox.merge! flash[:genre_checkbox] if !flash[:genre_checkbox].nil?
    @title = 'Genericon Trivia System - New Video Game Trivia Question'
  end

  # Create a new VideoGameTriviaQuestion and redirect back to new.
  def create
    # Validate existence of at least one checked genre.  We can bypass this if
    # we WANT, but we don't want to.
    has_one_genre = false
    params[:genre_checkbox].each do |genre, value|
      if value == '1'
        has_one_genre = true
        break
      end
    end
    
    # Load the series
    create_series = false
    @video_game = VideoGame.find_by_name(params[:video_game][:name])
    # If we can't load it, create it.
    if @video_game == nil
      @video_game = VideoGame.new(params[:video_game])
      params[:genre_checkbox].each do |genre, value|
        if value == '1'
          @video_game.video_game_genres << VideoGameGenre.find_by_underscored_name(genre)
        end
      end
      create_series = true
    end
    
    # Check for changes in the genres
    VideoGameGenre.find(:all).each do |genre|
      # If we have a difference between genres, note the potential change.
      if (params[:genre_checkbox][genre.underscored_name] == '1') != @video_game.video_game_genres.include?(genre)
        if @video_game.video_game_genres.include? genre
          @video_game.add_pending_change_support :video_game_genres, :remove, genre.id
        else
          @video_game.add_pending_change_support :video_game_genres, :add, genre.id
        end
      end
    end
    
    # Load the system
    create_system = false
    @video_game_system = VideoGameSystem.find_by_name(params[:video_game_system][:name])
    # If we can't load it, create it.
    if @video_game_system == nil
      @video_game_system = VideoGameSystem.new(params[:video_game_system])
      create_system = true
    end
    
    @video_game_trivia_question = VideoGameTriviaQuestion.new params[:video_game_trivia_question]
    @video_game_trivia_question.video_game = @video_game

    # Run validation checks to get full list of errors...
    @video_game.valid?
    @video_game_system.valid?
    @video_game_trivia_question.valid?
    
    if has_one_genre
      success = true
      VideoGameTriviaQuestion.transaction do
        VideoGameSystem.transaction do
          success &&= @video_game_system.save
	  if success && @video_game.video_game_system != @video_game_system
            if @video_game.video_game_system.nil?
              @video_game.video_game_system = @video_game_system
            else
              @video_game.add_pending_change_support :video_game_system, :change, @video_game_system.id
            end
          end
          VideoGame.transaction do
            success &&= @video_game.save
            success &&= @video_game_trivia_question.save
          end
        end
      end
    else
      @video_game.errors.add 'genres', '-- you must select at least one'
    end

    # Save and redirect.
    if success
      flash[:notice] = "Successfully submitted your video game trivia question."
      
      # Auto approve if we're logged in as an approver.
      if current_user.has_role? 'video_game_trivia_approver'
        @video_game_trivia_question.approve!
      end
      if current_user.has_role? 'video_game_approver'
        @video_game.approve!
      end
      if current_user.has_role? 'video_game_system_approver'
        @video_game_system.approve!
      end
    else
      # Render the errors, and retain the values input previously for next time.
      flash[:error] = render_to_string :inline => "<%= error_messages_for 'video_game_trivia_question' %><%= error_messages_for 'video_game' %><%= error_messages_for 'video_game_system' %>"
      flash[:error_is_html] = true
      flash[:video_game_trivia_question] = params[:video_game_trivia_question]
      flash[:video_game] = params[:video_game]
      flash[:video_game_system] = params[:video_game_system]
      flash[:genre_checkbox] = params[:genre_checkbox]
    end
    
    # Redirect back to the new path.
    redirect_to new_video_game_trivia_question_path
  end

  # Shows the edit page for an VideoGameTriviaQuestion.
  def edit
    @video_game_trivia_question = VideoGameTriviaQuestion.find params[:id]
    @video_game = @video_game_trivia_question.video_game || VideoGame.new()
    @title = "Genericon Trivia System - Edit Video Game Trivia Question"
  end

  # Approves a pending change for an VideoGameTriviaQuestion.
  def approve_pending_change
    @video_game_trivia_question = VideoGameTriviaQuestion.find params[:id]
    
    # Moderators commit the change.
    if current_user.has_role? 'video_game_trivia_moderator'
      @video_game_trivia_question.apply_pending_change params[:key].to_sym, params[:change_action].to_sym, params[:value]
    # Everyone else (read: approvers) just has their support added.
    else
      @video_game_trivia_question.add_pending_change_support params[:key].to_sym, params[:change_action].to_sym, params[:value]
    end
    if @video_game_trivia_question.save
      flash[:notice] = "Successfully approved the pending change."
    else
      # Render the errors, and retain the values input previously for next time.
      flash[:error] = render_to_string :inline => "<%= error_messages_for 'video_game_trivia_question' %>"
      flash[:error_is_html] = true
    end
    
    redirect_to edit_video_game_trivia_question_path, params[:id]
  end
  
  # Rejects a pending change for an VideoGameTriviaQuestion.
  def reject_pending_change
    @video_game_trivia_question = VideoGameTriviaQuestion.find params[:id]
    
    # Moderators need to delete the pending change entirely
    if current_user.has_role? 'video_game_trivia_moderator'
      @video_game_trivia_question.delete_pending_change params[:key].to_sym, params[:change_action].to_sym, params[:value]
    # Everyone else (read: approvers) just has their support (if any) removed.
    else
      @video_game_trivia_question.remove_pending_change_support params[:key].to_sym, params[:change_action].to_sym, params[:value]
    end
    if @video_game_trivia_question.save
      flash[:notice] = "Successfully rejected the pending change."
    else
      # Render the errors, and retain the values input previously for next time.
      flash[:error] = render_to_string :inline => "<%= error_messages_for 'video_game_trivia_question' %>"
      flash[:error_is_html] = true
    end
    
    redirect_to edit_video_game_trivia_question_path, params[:id]
  end

  # Destroys an VideoGameTriviaQuestion.
  def destroy
    # I sure hope you added a warning.
    # TODO: block when there are dependencies.
    
    if VideoGameTriviaQuestion.destroy params[:id]
      flash[:notice] = "Successfully deleted the video game trivia question."
    else
      flash[:error] = "There was a problem deleting that video game trivia question."
    end
    
    redirect_to manage_video_game_trivia_question_path
  end
  
  # Shows the list of VideoGameTriviaQuestions to be managed.
  def manage
    order = params[:order_by] || 'state_id ASC, `video_game_trivia_questions`.approved_at DESC, `video_game_trivia_questions`.created_at DESC'
    @video_game_trivia_questions = VideoGameTriviaQuestion.paginate :all, :page => params[:page], :select => "`video_game_trivia_questions`.*, CASE `video_game_trivia_questions`.state WHEN 'pending_approval' THEN 1 WHEN 'pending_changes' THEN 2 WHEN 'approved' THEN 3 END AS state_id, CASE `video_game_trivia_questions`.question_type WHEN 'normal' THEN 1 WHEN 'bonus' THEN 2 WHEN 'trick' THEN 3 END AS question_type_id, `video_games`.name, `ratings`.rating", :joins => "INNER JOIN `video_games` ON `video_game_trivia_questions`.video_game_id=`video_games`.id LEFT JOIN (SELECT item_id, SUM(rating) AS rating FROM `game_history_entries` GROUP BY item_type, item_id HAVING item_type='VideoGameTriviaQuestion') AS `ratings` ON `video_game_trivia_questions`.id=`ratings`.item_id", :order => order
    @order = order
    @title = 'Genericon Trivia System - Video Game Trivia Question Management'
  end
  
  # Approve an VideoGameTriviaQuestion.
  def approve
    @video_game_trivia_question = VideoGameTriviaQuestion.find(params[:id])
    
    if @video_game_trivia_question.pending_approval? and @video_game_trivia_question.approvable_changes?
      if @video_game_trivia_question.approve!
        flash[:notice] = "Successfully approved video game trivia question."
      else
        flash[:error] = render_to_string :inline => "<%= error_messages_for 'video_game_trivia_question' %>"
        flash[:error_is_html] = true
      end
    elsif @video_game_trivia_question.pending_changes? and @video_game_trivia_question.approvable_changes?
      if @video_game_trivia_question.approve_changes!
        flash[:notice] = "Successfully approved video game trivia question changes."
      else
        flash[:error] = render_to_string :inline => "<%= error_messages_for 'video_game_trivia_question' %>"
        flash[:error_is_html] = true
      end
    else
      flash[:error] = render_to_string :inline => "<%= error_messages_for 'video_game_trivia_question' %>"
      flash[:error_is_html] = true
    end
    
    redirect_to edit_video_game_trivia_question_path, params[:id]
  end

  # Configure a new game.
  def new_game
    # End any active game.
    if !session[:unused].nil?
      begin
        @game_session = GameSession.find(session[:unused])
        if @game_session.game_type != 'video_game_trivia_question'
          GameSession.clean_sessions(session[:unused])
          session[:unused] = nil
        end
      rescue ActiveRecord::RecordNotFound
        session[:unused] = nil
      end
    end
    
    # Setup the new game.
  end

  # Play Video Game Trivia.
  def play
    # First, check the old session to make sure it's a valid one.  If we came
    # from another game, we NEED to reset.  We also need to gracefully restart
    # from a timed-out game.
    if !session[:unused].nil?
      begin
        @game_session = GameSession.find(session[:unused])
        if @game_session.game_type != 'video_game_trivia_question'
          GameSession.clean_sessions(session[:unused])
          session[:unused] = nil
        end
      rescue ActiveRecord::RecordNotFound
        session[:unused] = nil
      end
    end
    
    # Initialize the 'unused' array of questions.
    if session[:unused].nil?
      @unused = VideoGameTriviaQuestion.find(:all)
      @unused = @unused.select {|q| !q.pending_approval? and !q.video_game.pending_approval? and (q.video_game.video_game_continuum.nil? or !q.video_game.video_game_continuum.pending_approval?) and (q.video_game.video_game_system.nil? or !q.video_game.video_game_system.pending_approval?)}
      @unused_normal = @unused.select {|q| q.question_type == :normal}
      @unused = @unused.collect { |q| q.id }
      @unused_normal = @unused_normal.collect { |q| q.id }
      @window = []
      @counters = { }
      @counters.default = 0
      @max_from_game = params[:max_from_game].to_i
      @max_from_game = @unused.length if @max_from_game == 0
      
      8.times do
        id = @unused_normal.slice!(rand(@unused_normal.length))
        @unused.delete(id)
        @question = VideoGameTriviaQuestion.find(id)
	
        # Make sure to filter by max_from_game (though this is sort of
        # irrelevant when starting.
        while @counters[@question.video_game.id] >= @max_from_game and !@unused_normal.empty?
          id = @unused_normal.slice!(rand(@unused_normal.length))
          @unused.delete id
          @question = VideoGameTriviaQuestion.find(id)
        end
        
        if @counters[@question.video_game.id] < @max_from_game
          @window << id
        end
        
        if @unused_normal.empty?
          break
        end
      end
      
      @game_session = GameSession.new(:game_type => 'video_game_trivia_question')
      @game_session.data = {:unused => @unused, :unused_normal => @unused_normal, :window => @window, :max_from_game => @max_from_game}
      @game_session.save
      session[:unused] = @game_session.id
    else
      @game_session = GameSession.find(session[:unused])
      @unused = @game_session.data[:unused]
      @unused_normal = @game_session.data[:unused_normal]
      @window = @game_session.data[:window]
      @counters = { }
      @counters.default = 0
      @counters.merge!(@game_session.data[:counters]) if @game_session.data[:counters]
      @max_from_game = @game_session.data[:max_from_game]
      
      # Can keep playing until the window is empty.
      if @window.empty?
        GameSession.clean_sessions(session[:unused])
        session[:unused] = nil
        flash[:notice] = 'All questions have been used up.  Game over.'
        redirect_to :action => 'index'
        return
      end
    end
    
    # Render the trivia questions.
    @window = @window.collect {|id| VideoGameTriviaQuestion.find(id)}
    @title = "Genericon Trivia System - Video Game Trivia"
  end

  # Remove the given question from the window and get a new question.
  def next_question
    id = params[:id]
    bonus_id = params[:bonus_id]
    
    # Save the ratings
    if params[:rating].nil?
      rating = 0
    else
      rating = params[:rating].to_i
      if rating < -1
        rating = -1
      elsif rating > 1
        rating = 1
      end
    end
    if params[:bonus_rating].nil?
      bonus_rating = 0
    else
      bonus_rating = params[:bonus_rating].to_i
      if bonus_rating < -1
        bonus_rating = -1
      elsif bonus_rating > 1
        bonus_rating = 1
      end
    end
    game_history_entry = GameHistoryEntry.new :item => VideoGameTriviaQuestion.find(id), :rating => rating
    game_history_entry.save
    if !bonus_id.nil?
      game_history_entry = GameHistoryEntry.new :item => VideoGameTriviaQuestion.find(bonus_id), :rating => bonus_rating
      game_history_entry.save
    end
    
    @game_session = GameSession.find(params[:session])
    @unused = @game_session.data[:unused]
    @unused_normal = @game_session.data[:unused_normal]
    @window = @game_session.data[:window]
    @counters = { }
    @counters.default = 0
    @counters.merge!(@game_session.data[:counters]) if @game_session.data[:counters]
    @max_from_game = @game_session.data[:max_from_game]
    
    # Remove the question.
    idx = @window.index id.to_i
    @window.delete_at idx
    if !bonus_id.nil?
      @unused.delete bonus_id.to_i
    end
    
    # Increment the appropriate counter and remove anything still from
    # now-disallowed games, saving the counter.
    @question = VideoGameTriviaQuestion.find(id)
    @counters[@question.video_game.id] = @counters[@question.video_game.id] + 1
    
    if @counters[@question.video_game.id] >= @max_from_game
      remove_questions = [ ]
      add_questions = [ ]
      @window.each do |old_id|
        if VideoGameTriviaQuestion.find(old_id).video_game.id == @question.video_game.id
          # Remember to remove this now-disallowed question.
          remove_questions << old_id
          
          # And get a new question to replace it with
          if !@unused_normal.empty?
            new_id = @unused_normal.slice!(rand(@unused_normal.length))
            new_question = VideoGameTriviaQuestion.find(new_id)
            @unused.delete new_id
            
            # Make sure to filter by max_from_game
            while @counters[new_question.video_game.id] >= @max_from_game and !@unused_normal.empty?
              new_id = @unused_normal.slice!(rand(@unused_normal.length))
              @unused.delete new_id
              new_question = VideoGameTriviaQuestion.find(new_id)
            end
            
            if @counters[new_question.video_game.id] < @max_from_game
              add_questions << new_id
            end
          end
        end
      end
      remove_questions.each do |id|
        insert_at = @window.index(id)
        @window.delete_at(insert_at)
        @window.insert(insert_at, add_questions.shift) if !add_questions.empty?
      end
    end
    
    # Get the next question.
    if !@unused_normal.empty?
      id = @unused_normal.slice!(rand(@unused_normal.length))
      @question = VideoGameTriviaQuestion.find(id)
      @unused.delete id
      
      # Make sure to filter by max_from_game
      while @counters[@question.video_game.id] >= @max_from_game and !@unused_normal.empty?
        id = @unused_normal.slice!(rand(@unused_normal.length))
        @unused.delete new_id
        @question = VideoGameTriviaQuestion.find(id)
      end
      
      if @counters[@question.video_game.id] < @max_from_game
        @window.insert(idx, id)
      end
    end
    
    data = @game_session.data || { }
    data[:window] = @window
    data[:unused_normal] = @unused_normal
    data[:unused] = @unused
    data[:counters] = @counters
    @game_session.data = data
    @game_session.save
    
    if !@window.empty?
      @window = @window.collect {|id| VideoGameTriviaQuestion.find(id)}
      render :partial => "question_window", :locals => {:question_window => @window, :i => params[:i], :session => {:unused => @game_session.id}}
    else
      GameSession.clean_sessions(params[:session])
      session[:unused] = nil
      flash[:notice] = 'All questions have been used up.  Game over.'
      redirect_to :action => 'index'
      return
    end
  end
  
  # Rerandomizes @window for play.
  def randomize_play_window
    @game_session = GameSession.find(session[:unused])
    @unused = @game_session.data[:unused]
    @unused_normal = @game_session.data[:unused_normal]
    @window = @game_session.data[:window]
    @counters = { }
    @counters.default = 0
    @counters.merge!(@game_session.data[:counters]) if @game_session.data[:counters]
    @max_from_game = @game_session.data[:max_from_game]
    
    @window.each {|id| @unused << id; @unused_normal << id}
    
    @window = [ ]
    8.times do
      id = @unused_normal.slice!(rand(@unused_normal.length))
      @unused.delete(id)
      @question = VideoGameTriviaQuestion.find(id)
      
      # Make sure to filter by max_from_game
      while @counters[@question.video_game.id] >= @max_from_game and !@unused_normal.empty?
        id = @unused_normal.slice!(rand(@unused_normal.length))
        @unused.delete id
        @question = VideoGameTriviaQuestion.find(id)
      end
      
      if @counters[@question.video_game.id] < @max_from_game
        @window << id
      end
      
      if @unused_normal.empty?
        break
      end
    end
    
    # Remember to save the current window and unused.
    @game_session = GameSession.find(session[:unused])
    data = @game_session.data || { }
    data[:window] = @window
    data[:unused_normal] = @unused_normal
    data[:unused] = @unused
    @game_session.data = data
    @game_session.save
    
    @window = @window.collect {|id| VideoGameTriviaQuestion.find(id)}
    
    render :partial => 'question_window', :object => @window, :locals => {:session => session}
  end

  def remove_video_game_from_game
    id = params[:id]
    
    @game_session = GameSession.find(params[:session])
    @unused = @game_session.data[:unused]
    @unused_normal = @game_session.data[:unused_normal]
    @window = @game_session.data[:window]
    @counters = { }
    @counters.default = 0
    @counters.merge!(@game_session.data[:counters]) if @game_session.data[:counters]
    @max_from_game = @game_session.data[:max_from_game]
    
    questions_to_remove = VideoGameTriviaQuestion.find(:all, :conditions => ['video_game_id = ?', id.to_i]).to_set
    questions_to_remove.collect! {|q| q.id}
    @window.reject! {|i| questions_to_remove.include? i}
    @unused.reject! {|i| questions_to_remove.include? i}
    @unused_normal.reject! {|i| questions_to_remove.include? i}
    
    if @window.length < 8
      (8 - @window.length).times do
        id = @unused_normal.slice!(rand(@unused_normal.length))
        @unused.delete(id)
        @question = VideoGameTriviaQuestion.find(id)
        
        # Make sure to filter by max_from_game
        while @counters[@question.video_game.id] >= @max_from_game and !@unused_normal.empty?
          id = @unused_normal.slice!(rand(@unused_normal.length))
          @unused.delete new_id
          @question = VideoGameTriviaQuestion.find(id)
        end
        
        if @counters[@question.video_game.id] < @max_from_game
          @window << id
        end
        
        if @unused_normal.empty?
          break
        end
      end
    end
    
    # Remember to save the current window and unused.
    data = @game_session.data || { }
    data[:window] = @window
    data[:unused_normal] = @unused_normal
    data[:unused] = @unused
    @game_session.data = data
    @game_session.save
    
    @window = @window.collect {|id| VideoGameTriviaQuestion.find(id)}
    
    render :partial => 'question_window', :object => @window, :locals => { :session => { :unused => params[:session] } }
  end

  # Change the maximum questions from game.
  def set_max_from_game
    @game_session = GameSession.find(params[:session])
    data = @game_session.data || { }
    if params[:max_from_game].nil? or params[:max_from_game].empty? or params[:max_from_game].to_i == 0
      data[:max_from_game] = data[:unused].length
    else
      data[:max_from_game] = params[:max_from_game].to_i
    end
    @game_session.data = data
    @game_session.save
  end

end
