# Methods added to this helper will be available to all templates in the application.
module ApplicationHelper
  # Generates a link in a table-header to order a given list by the
  # inputs based on the current order.
  def order_link(text, ascending_order, descending_order, order, opts={})
    parms = Hash.new(params)
    parms.delete :page
    if order == ascending_order
      parms[:order_by] = descending_order
      link_to text + " " + image_tag('active_ascending.png', :alt => '(Ascending)'), parms, :class => 'active'
    elsif order == descending_order
      parms[:order_by] = ascending_order
      link_to text + " " + image_tag('active_descending.png', :alt => '(Descending)'), parms, :class => 'active'
    elsif opts[:default] == :descending
      parms[:order_by] = descending_order
      link_to text + " " + image_tag('descending.png', :alt => ''), parms
    else
      parms[:order_by] = ascending_order
      link_to text + " " + image_tag('ascending.png', :alt => ''), parms
    end
  end

  # Generates a 'disclosure' with the given text, open and closed images, and
  # ID of the element to contain inside the disclosure.
  def make_disclosure text, open_image, closed_image, element
    open_image_path = image_path open_image
    closed_image_path = image_path closed_image
    
    "<a href=\"#\" onclick=\"toggleDisclosure('#{element}', '#{open_image_path}', '#{closed_image_path}'); return false;\"><img src=\"#{closed_image_path}\" alt=\"\" id=\"#{element}_disclosure_image\" class=\"disclosure_image\" /></a> #{text}" 
  end
  
  # Generates a link to AniDB based on the anidb_id of the AnimeSeries.
  def anidb_path(anime_series)
    unless anime_series.anidb_id.nil?
      "http://anidb.net/perl-bin/animedb.pl?show=anime&aid=#{anime_series.anidb_id}"
    else
      ""
    end
  end
  
  # Returns an array of hashes containing the names and controllers of the main classes.
  def main_classes
    [{:name => 'Anime Trivia', :controller => 'anime_trivia_question'},
     {:name => 'Name-That-Anime', :controller => 'anime_scene'},
     {:name => 'Video Game Trivia', :controller => 'video_game_trivia_question'},
     {:name => 'Name-That-Video Game', :controller => 'video_game_scene'}]
  end
  
  # Returns the login of the user, or Anonymous if nil.
  def login_of(user)
	if user.nil?
	  "Anonymous"
	else
	  user.login
	end
  end
  
  # Generates the navbar seen on all of the 'new' pages.
  def navbar
    navbar = <<EOF
<div class="nav">
  <table>
    <tbody>
      <tr>
EOF
    
    main_classes.each do |klass|
      if klass[:controller] == controller.controller_name
        navbar += "        <td class=\"currentpage\">#{klass[:name]}</td>\n"
      else
        navbar += "        <td>" + link_to(klass[:name], { :controller => klass[:controller], :action => 'new' }) + "</td>\n"
      end
    end
    navbar += "        <td><a href=\"/other_submitters.html\">Other...</a></td>\n"
    
    navbar += <<EOF
      </tr>
    </tbody>
  </table>
</div>
EOF
    
    navbar
  end
  
  # This builds an in-place editor with the given field_id and options.
  def in_place_editor_func(field_id, options = { })
    function =  "new Ajax.InPlaceEditor("
    function << "'#{field_id}', "
    function << "'#{url_for(options[:url])}'"
    
    js_options = {}
    
    if protect_against_forgery?
      options[:with] ||= "Form.serialize(form)"
      options[:with] += " + '&authenticity_token=' + encodeURIComponent('#{form_authenticity_token}')"
    end
    
    js_options['okControl'] = %('#{options[:ok_control]}') if options[:ok_control]
    js_options['cancelControl'] = %('#{options[:cancel_control]}') if options[:cancel_control]
    js_options['savingText'] = %('#{options[:saving_text]}') if options[:saving_text]
    js_options['formClassName'] = %('#{options[:form_class_name]}') if options[:form_class_name]
    js_options['clickToEditText'] = %('#{options[:click_to_edit_text]}') if options[:click_to_edit_text]
    js_options['highlightcolor'] = %('#{options[:highlight_color]}') if options[:highlight_color]
    js_options['highlightendcolor'] = %('#{options[:highlight_end_color]}') if options[:highlight_end_color]
    js_options['cancelText'] = %('#{options[:cancel_text]}') if options[:cancel_text]
    js_options['okText'] = %('#{options[:save_text]}') if options[:save_text]
    js_options['loadingText'] = %('#{options[:loading_text]}') if options[:loading_text]
    js_options['rows'] = options[:rows] if options[:rows]
    js_options['cols'] = options[:cols] if options[:cols]
    js_options['size'] = options[:size] if options[:size]
    js_options['externalControl'] = "'#{options[:external_control]}'" if options[:external_control]
    js_options['loadTextURL'] = "'#{url_for(options[:load_text_url])}'" if options[:load_text_url]
    js_options['ajaxOptions'] = options[:options] if options[:options]
    js_options['htmlResponse'] = !options[:script] if options[:script]
    js_options['callback']   = "function(form) { return #{options[:with]}}" if options[:with]
    js_options['submitOnBlur'] = options[:submit_on_blur] if options[:submit_on_blur]
    js_options['textBetweenControls'] = %('#{options[:text_between_controls]}') if options[:text_between_controls]
    function << (', ' + options_for_javascript(js_options)) unless js_options.empty?
    function << ')'
  end
  
  # This outputs the javascript for the given in-place editor.
  def in_place_editor_script(field_id, options = { })
    javascript_tag(in_place_editor_func(field_id, options))
  end
  
  # This builds the span that is used to hold the in-place editor.
  def in_place_editor_span(object, method, tag_options = {})
    tag = ::ActionView::Helpers::InstanceTag.new object, method, self
    tag_options = {:tag => "span", :id => "#{object}_#{method}_#{tag.object.id}_in_place_editor", :class => "in_place_editor_field"}.merge!(tag_options)
    tag.to_content_tag tag_options.delete(:tag), tag_options
  end
  
  # This builds the in-place editor field including the span and script.
  def in_place_editor_field_ext(object, method, tag_options = {},
                                in_place_editor_options = {})
    tag = ::ActionView::Helpers::InstanceTag.new object, method, self
    in_place_editor_options[:url] = in_place_editor_options[:url] || url_for({ :action => "set_#{object}_#{method}", :id => tag.object.id })
    in_place_editor_span(object, method, tag_options) +
      in_place_editor_script("#{object}_#{method}_#{tag.object.id}_in_place_editor", in_place_editor_options)
  end
  
  
  # This builds a validated in-place-editor.
  def validated_in_place_editor_field(object, method, tag_options = {},
                                      in_place_editor_options = {})
    in_place_editor_field object, method, tag_options, in_place_editor_options
  end
  
  # This builds an in-place editor.
  def in_place_editor_field(object, method, tag_options = {},
                            in_place_editor_options = {})
    s = ''
    in_place_editor_options[:script] = true unless in_place_editor_options[:script]
    in_place_editor_options[:options] = "{ method: 'post' }" unless in_place_editor_options[:options]
    in_place_editor_options[:highlight_color] = '#FFFF88' unless in_place_editor_options[:highlight_color]
    in_place_editor_options[:highlight_end_color] = '#ADD8E6' unless in_place_editor_options[:highlight_end_color]
    in_place_editor_options[:ok_control] = 'imagelink' unless in_place_editor_options[:ok_control]
    in_place_editor_options[:save_text] = image_tag('approve.png', :alt => 'OK') unless in_place_editor_options[:save_text]
    in_place_editor_options[:cancel_control] = 'imagelink' unless in_place_editor_options[:cancel_control]
    in_place_editor_options[:cancel_text] = image_tag('delete.png', :alt => 'Cancel') unless in_place_editor_options[:cancel_text]
    unless in_place_editor_options[:external_control]
      in_place_editor_options[:external_control] = "edit_#{object}_#{method}" unless in_place_editor_options[:external_control]
      s = "<a href=\"#\" id=\"edit_#{object}_#{method}\">" + image_tag('edit.png', :alt => '(Edit)') + "</a> "
    end
    s + in_place_editor_field_ext(object, method, tag_options, in_place_editor_options)
  end

  def check_boxes_for_anime_genres
    # Use this to build the genre checkboxes.
    render :partial => 'layouts/anime_genre_checkboxes'
  end

  def anime_genre_div(genre)
    initial_genres = AnimeGenre.find_genres_like(nil).collect { |initial_genre| initial_genre[:name] }
    name = genre.underscored_name
    if initial_genres.include?(genre[:name])
      "<div class=\"genre\" id=\"genre_#{name}\">"
    else
      "<div class=\"genre\" id=\"genre_#{name}\" style=\"display: none;\">"
    end
  end
  
  def check_boxes_for_video_game_genres
    # Use this to build the genre checkboxes.
    render :partial => 'layouts/video_game_genre_checkboxes'
  end

  def video_game_genre_div(genre)
    name = genre.underscored_name
    "<div class=\"genre\" id=\"genre_#{name}\">"
  end
  
  def help_for(part)
    link_to(image_tag('info.gif', :size => '12x12', :alt => '(Help)'),
            { :controller => "help.html##{part}" },
            :popup => ['help', 'height=300,width=400'],
            :class => 'info')
  end
  
end
