# Represents a role for ACLs.
class Role < ActiveRecord::Base
  has_many :roles_users
  has_many :users, :through => :roles_users, :uniq => true
  has_many :delegators_roles
  has_many :delegators, :through => :delegators_roles, :uniq => true
  has_many :roles_encompassed_roles
  has_many :encompassed_roles, :through => :roles_encompassed_roles, :uniq => true
  has_many :roles_encompassing_roles, :class_name => 'RolesEncompassedRole', :foreign_key => 'encompassed_role_id'
  has_many :encompassing_roles, :through => :roles_encompassing_roles, :source => :role, :uniq => true
  
  # So that delegators_roles and roles_users are stamped
  stampable
  
  # Need to override the << action on encompassed_roles
  alias :_encompassed_roles :encompassed_roles
  private :_encompassed_roles
  def encompassed_roles
    roles = _encompassed_roles
    class << roles
      if !instance_methods.include? '_push'
        alias_method :_push, :<<
        define_method :<< do |*args|
          args.collect! { |role| [role.encompassed_roles, role] }
          args.flatten!.uniq!
          
          _push *args
        end
        alias_method :push, :<<
        alias_method :concat, :<<
      end
    end
    roles
  end
  
  # Returns the list of all users that can act with this role and the role they
  # obtained it through.
  def all_users
    total_users = {}
    total_users.default = []
    users.each { |user| total_users[user] += [self] }
    encompassing_roles.each do |role|
      role.users.each do |user|
        total_users[user] += [role]
      end
    end
    total_users
  end
  
  # Returns the list of all users that can delegate this role and the role they
  # obtained it through.
  def all_delegators
    total_delegators = {}
    total_delegators.default = []
    delegators.each { |user| total_delegators[user] += [self] }
    encompassing_roles.each do |role|
      role.delegators.each do |user|
        total_delegators[user] += [role]
      end
    end
    total_delegators
  end
  
  # Checks to see if a role (either as Role or String) is encompassed by this role.
  def encompasses_role?(role_in_question)
    @_list ||= self.encompassed_roles.collect(&:name)
    return true if self.name == "admin" or @_list.include?("admin") or self.name == role_in_question.to_s
    return true if (@_list.include?(role_in_question.to_s))
    # Otherwise, fall back on checking child roles.
    @_list.each do |role|
      return true if Role.find_by_name(role).encompasses_role?(role_in_question)
    end
    false
  end
  
  def to_s
    name
  end
end