# Manages sending e-mails notifying a User of important status changes.
class UserMailer < ActionMailer::Base
  # Sends a notification to activate a User.
  def signup_notification(user)
    setup_email(user)
    @subject    += 'Please activate your new account'
    @body[:url]  = "http://#{SITE_URL}/activate/#{user.activation_code}"
  end
  
  # Sends a notification that a User has been activated.
  def activation(user)
    setup_email(user)
    @subject    += 'Your account has been activated!'
    @body[:url]  = "http://#{SITE_URL}/"
  end
  
  # Sends a notification to reset a password.
  def reset_password(user)
    setup_email(user)
    @subject    += 'Resetting your password'
    @body[:url]  = "http://#{SITE_URL}/reset_password/#{user.activation_code}"
  end
  
protected
  # Sets up common e-mail information like recipients and subject.
  def setup_email(user)
    @recipients  = "#{user.email}"
    @from        = "pipian@pipian.com"
    @subject     = "[Genericon Trivia System] "
    @sent_on     = Time.now
    @body[:user] = user
  end
end
