# Represents a single anime genre.
class VideoGameGenre < ActiveRecord::Base
  has_and_belongs_to_many :video_games, :uniq => true
  has_many :children, :dependent => :destroy, :class_name => 'VideoGameGenre', :foreign_key => 'parent_id'
  belongs_to :parent, :class_name => 'VideoGameGenre'
  
  validates_uniqueness_of :name, :message => 'has already been taken'
  validates_presence_of :name, :message => 'may not be empty'
  
  # Modifies the addition of a series such that it doesn't cause an error.
  if !instance_methods.include? '_video_games'
    alias_method :_video_games, :video_games
    define_method :video_games do
      games = _video_games
      games.metaclass.instance_eval do
        if !instance_methods.include? '_push'
          alias_method :_push, :<<
          define_method :<< do |*values|
            values = values - to_ary
            values.collect! { |value| _push(value) }
          end
          alias_method :push, :<<
          alias_method :concat, :<<
        end

        if !instance_methods.include? '_delete'
          alias_method :_delete, :delete
          define_method :delete do |*values|
            values = to_ary.select { |item| values.include? item }
            values.collect! { |value| _delete(value) }
          end
        end
      end
      games
    end
  end
  
  # Merges this VideoGameGenre and its alternate names into another VideoGameGenre.  This is an irreversible operation (like destroy!)
  def merge_into!(other)
    for game in self.video_games
      game.video_game_genres.delete(self)
      game.video_game_genres << other
      game.save!
    end
    
    VideoGameGenre.delete(id) unless new_record?
    freeze

    true
  end
  
#  def VideoGameGenre.default_genres
#    g = [ 'Shounen', 'Shoujo', 'Action', 'Mecha', 'Adventure', 'Drama', 'Science Fiction', 'Romance', 'Comedy', 'Fantasy' ]
#    genres = []
#    g.each do |genre|
#      genres << VideoGameGenre.find_by_name(genre)
#    end
#    genres
#  end
  
  def underscored_name
    name.gsub(/[^a-zA-Z]/, '_').downcase
  end
  
  def VideoGameGenre.find_by_underscored_name(name)
    VideoGameGenre.find :first, :conditions => [ "lower(name) LIKE ?", name.gsub(/_/, '%') ]
  end
  
#  def VideoGameGenre.find_genres_like(genres)
#    if genres == nil || genres.size == 0
#      genres = VideoGameGenre.default_genres
#      genres.collect do |genre|
#        { :name => genre.name,
#          :selected => false,
#          :underscored_name => genre.underscored_name }
#      end
#    else
#      genres.uniq!
#      anime = []
#      genres.each do |genre|
#        anime += VideoGameGenre.find_by_name(genre).anime_series.collect {|series| series.id}
#        anime.uniq!
#      end
#      newgenres = {}
#      newgenres.default = 0
#      anime.each do |series|
#        anime_genres = VideoGameSeries.find(series).anime_genres
#        anime_genres.each do |genre|
#          newgenres[genre.name] += 1
#        end
#      end
#      newgenres = newgenres.sort {|a,b| a[1]<=>b[1]}
#      newgenres = newgenres.reverse.collect {|genre| genre[0]}
#      while newgenres.length > 10
#        newgenres.pop
#      end
#      genres.each do |genre|
#        if !newgenres.include? genre
#          newgenres.push genre
#        end
#      end
#      if newgenres.length < 10
#        # Pad out with the most popular default genres altogether.
#        topgenres = VideoGameGenre.default_genres
#        topgenres = topgenres.sort {|a,b| b.anime_series.length<=>a.anime_series.length || a.name<=>b.name}
#        topgenres = topgenres.reverse.collect {|genre| genre.name}
#        while newgenres.length < 10
#          genre = topgenres.pop
#          if !newgenres.include? genre
#            newgenres.push genre
#          end
#        end
#      end
#      newgenres.collect do |genre|
#        if genres.include? genre
#          { :name => genre,
#            :selected => true,
#            :underscored_name => genre.gsub(/[^a-zA-Z]/, '_').downcase }
#        else
#          { :name => genre,
#            :selected => false,
#            :underscored_name => genre.gsub(/[^a-zA-Z]/, '_').downcase }
#        end
#      end
#    end
#  end
end
