ActionController::Routing::Routes.draw do |map|
  # User actions...
  map.logout '/logout', :controller => 'sessions', :action => 'destroy'
  map.login '/login', :controller => 'sessions', :action => 'new'
  map.register '/register', :controller => 'users', :action => 'create'
  map.signup '/signup', :controller => 'users', :action => 'new'
  map.activate '/activate/:activation_code', :controller => 'users', :action => 'activate', :activation_code => nil
  map.reset_password '/reset_password/:activation_code', :controller => 'users', :action => 'reset_password', :activation_code => nil
  map.send_reset_password_email '/send_reset_password_email', :controller => 'users', :action => 'send_reset_password_email'
  map.unapproved_items '/unapproved_items', :controller => 'application', :action => 'unapproved_items'
  map.recent_changes '/recent_changes', :controller => 'application', :action => 'recent_changes'
  
  # User management...
  map.resources :users, :member => { :suspend     => :put,
                                     :unsuspend   => :put,
                                     :purge       => :delete,
                                     :show        => :get,
                                     :edit        => :get,
                                     :update      => :post },
                        :collection => { :list  => :get,
                                         :index => :get }
  map.add_role_user '/users/:user_id/add_role', :controller => 'users', :action => 'add_role', :conditions => { :method => :post }
  map.remove_role_user '/users/:user_id/remove_role', :controller => 'users', :action => 'remove_role', :conditions => { :method => :post }
  map.add_delegator_role_user '/users/:user_id/add_delegator_role', :controller => 'users', :action => 'add_delegator_role', :conditions => { :method => :post }
  map.remove_delegator_role_user '/users/:user_id/remove_delegator_role', :controller => 'users', :action => 'remove_delegator_role', :conditions => { :method => :post }
  
  map.resource :session
  
  # Role management...
  map.resources :roles, :member => { :show => :get },
                        :collection => { :list => :get }
  map.add_user_role '/roles/:role/add_user', :controller => 'roles', :action => 'add_user', :conditions => { :method => :post }
  map.remove_user_role '/roles/:role/remove_user/:user', :controller => 'roles', :action => 'remove_user', :conditions => { :method => :put }
  map.add_delegator_role '/roles/:role/add_delegator', :controller => 'roles', :action => 'add_delegator', :conditions => { :method => :post }
  map.remove_delegator_role '/roles/:role/remove_delegator/:user', :controller => 'roles', :action => 'remove_delegator', :conditions => { :method => :put }

  # AnimeSeries management...
  map.resources :anime_series, :member => { :show                      => :get,
                                            :edit                      => :get,
                                            :select_merge              => :get,
                                            :approve                   => :put,
                                            :destroy                   => :put,
                                            :set_anime_series_name     => :post,
                                            :set_anime_series_anidb_id => :post,
                                            :set_anime_series_years    => :post,
                                            :select_merge_search      => :post },
                               :collection => { :list   => :get,
                                                :new    => :get,
                                                :create => :post,
                                                :manage => :get }
  map.add_anime_series_alternate_name '/anime_series/:id/add_alternate_name', :controller => 'anime_series', :action => 'add_alternate_name', :conditions => { :method => :post }
  map.remove_anime_series_alternate_name '/anime_series/:id/remove_alternate_name/:name', :controller => 'anime_series', :action => 'remove_alternate_name', :conditions => { :method => :put }, :requirements => { :name => /.*/ }
  map.add_anime_series_anime_genre '/anime_series/:id/add_anime_genre', :controller => 'anime_series', :action => 'add_anime_genre', :conditions => { :method => :post }
  map.remove_anime_series_anime_genre '/anime_series/:id/remove_anime_genre/:genre_id', :controller => 'anime_series', :action => 'remove_anime_genre', :conditions => { :method => :put }
  map.approve_anime_series_pending_change '/anime_series/:id/approve_pending_change/:change_action/:key/:value', :controller => 'anime_series', :action => 'approve_pending_change', :conditions => { :method => :put }
  map.reject_anime_series_pending_change '/anime_series/:id/reject_pending_change/:change_action/:key/:value', :controller => 'anime_series', :action => 'reject_pending_change', :conditions => { :method => :put }
  map.anime_series_merge_into '/anime_series/:id/merge_into/:dest', :controller => 'anime_series', :action => 'merge_into', :conditions => { :method => :put }

  # AnimeContinuum management...
  map.resources :anime_continuum, :member => { :show                     => :get,
                                               :edit                     => :get,
                                               :select_merge             => :get,
                                               :approve                  => :put,
                                               :destroy                  => :put,
                                               :set_anime_continuum_name => :post,
                                               :select_merge_search      => :post },
                                  :collection => { :list   => :get,
                                                   :new    => :get,
                                                   :create => :post,
                                                   :manage => :get }
  map.add_anime_continuum_alternate_name '/anime_continuum/:id/add_alternate_name', :controller => 'anime_continuum', :action => 'add_alternate_name', :conditions => { :method => :post }
  map.remove_anime_continuum_alternate_name '/anime_continuum/:id/remove_alternate_name/:name', :controller => 'anime_continuum', :action => 'remove_alternate_name', :conditions => { :method => :put }, :requirements => { :name => /.*/ }
  map.approve_anime_continuum_pending_change '/anime_continuum/:id/approve_pending_change/:change_action/:key/:value', :controller => 'anime_continuum', :action => 'approve_pending_change', :conditions => { :method => :put }
  map.reject_anime_continuum_pending_change '/anime_continuum/:id/reject_pending_change/:change_action/:key/:value', :controller => 'anime_continuum', :action => 'reject_pending_change', :conditions => { :method => :put }
  map.anime_continuum_merge_into '/anime_continuum/:id/merge_into/:dest', :controller => 'anime_continuum', :action => 'merge_into', :conditions => { :method => :put }

  # AnimeGenre management...
  map.resources :anime_genre, :member => { :edit                        => :get,
                                           :select_merge                => :get,
                                           :destroy                     => :put,
                                           :set_anime_genre_name        => :post,
                                           :set_anime_genre_description => :post,
                                           :select_merge_search         => :post },
                              :collection => { :list   => :get,
                                               :new    => :get,
                                               :create => :post,
                                               :manage => :get }
  map.anime_genre_merge_into '/anime_genre/:id/merge_into/:dest', :controller => 'anime_genre', :action => 'merge_into', :conditions => { :method => :put }
  
  # AnimeTriviaQuestion management...
  map.resources :anime_trivia_question, :member => { :show                      => :get,
                                                     :edit                      => :get,
                                                     :approve                   => :put,
                                                     :destroy                   => :put },
                               :collection => { :list   => :get,
                                                :new    => :get,
                                                :create => :post,
                                                :manage => :get, 
                                                :play   => :get,
                                                :new_game => :get }
  map.approve_anime_trivia_question_pending_change '/anime_trivia_question/:id/approve_pending_change/:change_action/:key/:value', :controller => 'anime_trivia_question', :action => 'approve_pending_change', :conditions => { :method => :put }
  map.reject_anime_trivia_question_pending_change '/anime_trivia_question/:id/reject_pending_change/:change_action/:key/:value', :controller => 'anime_trivia_question', :action => 'reject_pending_change', :conditions => { :method => :put }

  # AnimeScene management...
  map.resources :anime_scene, :member => { :show                      => :get,
                                           :edit                      => :get,
                                           :image                     => :get,
                                           :approve                   => :put,
                                           :destroy                   => :put },
                               :collection => { :list   => :get,
                                                :new    => :get,
						:new_zip => :get,
                                                :create => :post,
						:create_zip => :post,
                                                :manage => :get, 
                                                :play   => :get,
                                                :new_game => :get }
  map.approve_anime_scene_pending_change '/anime_scene/:id/approve_pending_change/:change_action/:key/:value', :controller => 'anime_scene', :action => 'approve_pending_change', :conditions => { :method => :put }
  map.reject_anime_scene_pending_change '/anime_scene/:id/reject_pending_change/:change_action/:key/:value', :controller => 'anime_scene', :action => 'reject_pending_change', :conditions => { :method => :put }

  # VideoGame management...
  map.resources :video_game, :member => { :show                      => :get,
                                            :edit                      => :get,
                                            :select_merge              => :get,
                                            :approve                   => :put,
                                            :destroy                   => :put,
                                            :set_video_game_name       => :post,
                                            :set_video_game_year      => :post,
                                            :select_merge_search       => :post },
                               :collection => { :list   => :get,
                                                :new    => :get,
                                                :create => :post,
                                                :manage => :get }
  map.add_video_game_alternate_name '/video_game/:id/add_alternate_name', :controller => 'video_game', :action => 'add_alternate_name', :conditions => { :method => :post }
  map.remove_video_game_alternate_name '/video_game/:id/remove_alternate_name/:name', :controller => 'video_game', :action => 'remove_alternate_name', :conditions => { :method => :put }, :requirements => { :name => /.*/ }
  map.add_video_game_video_game_genre '/video_game/:id/add_video_game_genre', :controller => 'video_game', :action => 'add_video_game_genre', :conditions => { :method => :post }
  map.remove_video_game_video_game_genre '/video_game/:id/remove_video_game_genre/:genre_id', :controller => 'video_game', :action => 'remove_video_game_genre', :conditions => { :method => :put }
  map.approve_video_game_pending_change '/video_game/:id/approve_pending_change/:change_action/:key/:value', :controller => 'video_game', :action => 'approve_pending_change', :conditions => { :method => :put }
  map.reject_video_game_pending_change '/video_game/:id/reject_pending_change/:change_action/:key/:value', :controller => 'video_game', :action => 'reject_pending_change', :conditions => { :method => :put }
  map.video_game_merge_into '/video_game/:id/merge_into/:dest', :controller => 'video_game', :action => 'merge_into', :conditions => { :method => :put }

  # VideoGameSystem management...
  map.resources :video_game_system, :member => { :show                     => :get,
                                                 :edit                     => :get,
                                                 :select_merge             => :get,
                                                 :approve                  => :put,
                                                 :destroy                  => :put,
                                                 :set_video_game_system_name => :post,
                                                 :set_video_game_system_manufacturer => :post,
                                                 :select_merge_search      => :post },
                                    :collection => { :list   => :get,
                                                     :new    => :get,
                                                     :create => :post,
                                                     :manage => :get }
  map.add_video_game_system_alternate_name '/video_game_system/:id/add_alternate_name', :controller => 'video_game_system', :action => 'add_alternate_name', :conditions => { :method => :post }
  map.remove_video_game_system_alternate_name '/video_game_system/:id/remove_alternate_name/:name', :controller => 'video_game_system', :action => 'remove_alternate_name', :conditions => { :method => :put }, :requirements => { :name => /.*/ }
  map.approve_video_game_system_pending_change '/video_game_system/:id/approve_pending_change/:change_action/:key/:value', :controller => 'video_game_system', :action => 'approve_pending_change', :conditions => { :method => :put }
  map.reject_video_game_system_pending_change '/video_game_system/:id/reject_pending_change/:change_action/:key/:value', :controller => 'video_game_system', :action => 'reject_pending_change', :conditions => { :method => :put }
  map.video_game_system_merge_into '/video_game_system/:id/merge_into/:dest', :controller => 'video_game_system', :action => 'merge_into', :conditions => { :method => :put }

  # VideoGameContinuum management...
  map.resources :video_game_continuum, :member => { :show                     => :get,
                                               :edit                     => :get,
                                               :select_merge             => :get,
                                               :approve                  => :put,
                                               :destroy                  => :put,
                                               :set_video_game_continuum_name => :post,
                                               :select_merge_search      => :post },
                                  :collection => { :list   => :get,
                                                   :new    => :get,
                                                   :create => :post,
                                                   :manage => :get }
  map.add_video_game_continuum_alternate_name '/video_game_continuum/:id/add_alternate_name', :controller => 'video_game_continuum', :action => 'add_alternate_name', :conditions => { :method => :post }
  map.remove_video_game_continuum_alternate_name '/video_game_continuum/:id/remove_alternate_name/:name', :controller => 'video_game_continuum', :action => 'remove_alternate_name', :conditions => { :method => :put }, :requirements => { :name => /.*/ }
  map.approve_video_game_continuum_pending_change '/video_game_continuum/:id/approve_pending_change/:change_action/:key/:value', :controller => 'video_game_continuum', :action => 'approve_pending_change', :conditions => { :method => :put }
  map.reject_video_game_continuum_pending_change '/video_game_continuum/:id/reject_pending_change/:change_action/:key/:value', :controller => 'video_game_continuum', :action => 'reject_pending_change', :conditions => { :method => :put }
  map.video_game_continuum_merge_into '/video_game_continuum/:id/merge_into/:dest', :controller => 'video_game_continuum', :action => 'merge_into', :conditions => { :method => :put }

  # VideoGameGenre management...
  map.resources :video_game_genre, :member => { :edit                        => :get,
                                                :select_merge                => :get,
                                                :destroy                     => :put,
                                                :set_video_game_genre_name        => :post,
                                                :set_video_game_genre_parent_id => :post,
                                                :set_video_game_genre_description => :post,
                                                :select_merge_search         => :post },
                                   :collection => { :list   => :get,
                                                    :new    => :get,
                                                    :create => :post,
                                                    :manage => :get }
  map.video_game_genre_merge_into '/video_game_genre/:id/merge_into/:dest', :controller => 'video_game_genre', :action => 'merge_into', :conditions => { :method => :put }
  
  # VideoGameTriviaQuestion management...
  map.resources :video_game_trivia_question, :member => { :show                      => :get,
                                                          :edit                      => :get,
                                                          :approve                   => :put,
                                                          :destroy                   => :put },
                                    :collection => { :list   => :get,
                                                     :new    => :get,
                                                     :create => :post,
                                                     :manage => :get, 
                                                     :play   => :get,
                                                     :new_game => :get }
  map.approve_video_game_trivia_question_pending_change '/video_game_trivia_question/:id/approve_pending_change/:change_action/:key/:value', :controller => 'video_game_trivia_question', :action => 'approve_pending_change', :conditions => { :method => :put }
  map.reject_video_game_trivia_question_pending_change '/video_game_trivia_question/:id/reject_pending_change/:change_action/:key/:value', :controller => 'video_game_trivia_question', :action => 'reject_pending_change', :conditions => { :method => :put }

  # VideoGameScene management...
  map.resources :video_game_scene, :member => { :show                      => :get,
                                           :edit                      => :get,
                                           :image                     => :get,
                                           :approve                   => :put,
                                           :destroy                   => :put },
                               :collection => { :list   => :get,
                                                :new    => :get,
                                                :new_zip => :get,
                                                :create => :post,
						:create_zip => :post,
                                                :manage => :get, 
                                                :play   => :get,
                                                :new_game => :get }
  map.approve_video_game_scene_pending_change '/video_game_scene/:id/approve_pending_change/:change_action/:key/:value', :controller => 'video_game_scene', :action => 'approve_pending_change', :conditions => { :method => :put }
  map.reject_video_game_scene_pending_change '/video_game_scene/:id/reject_pending_change/:change_action/:key/:value', :controller => 'video_game_scene', :action => 'reject_pending_change', :conditions => { :method => :put }

  # Comment management...
  map.resources :comments, :member => { :delete              => :delete,
                                        :make_note           => :put,
                                        :edit_comment_type   => :post,
                                        :update_comment_type => :post },
                           :collection => { :new    => :post, 
                                            :create => :post }
  map.get_comment_type_description '/comments/get_type_description', :controller => 'comments', :action => 'get_type_description', :conditions => { :method => :post }
  
  map.connect '/uploads/:attachment/:id/:style/:basename', :controller => 'uploads', :action => 'index', :conditions => { :method => :get }, :requirements => { :basename => /.*/ }
  
  # The priority is based upon order of creation: first created -> highest priority.

  # Sample of regular route:
  #   map.connect 'products/:id', :controller => 'catalog', :action => 'view'
  # Keep in mind you can assign values other than :controller and :action

  # Sample of named route:
  #   map.purchase 'products/:id/purchase', :controller => 'catalog', :action => 'purchase'
  # This route can be invoked with purchase_url(:id => product.id)

  # Sample resource route (maps HTTP verbs to controller actions automatically):
  #   map.resources :products

  # Sample resource route with options:
  #   map.resources :products, :member => { :short => :get, :toggle => :post }, :collection => { :sold => :get }

  # Sample resource route with sub-resources:
  #   map.resources :products, :has_many => [ :comments, :sales ], :has_one => :seller
  
  # Sample resource route with more complex sub-resources
  #   map.resources :products do |products|
  #     products.resources :comments
  #     products.resources :sales, :collection => { :recent => :get }
  #   end

  # Sample resource route within a namespace:
  #   map.namespace :admin do |admin|
  #     # Directs /admin/products/* to Admin::ProductsController (app/controllers/admin/products_controller.rb)
  #     admin.resources :products
  #   end

  # You can have the root of your site routed with map.root -- just remember to delete public/index.html.
  map.root :controller => "anime_trivia_question"

  # See how all your routes lay out with "rake routes"

  # Install the default routes as the lowest priority.
  map.connect ':controller/:action'
  map.connect ':controller/:action/:id'
  map.connect ':controller/:action/:id.:format'
end
