/*
 * Original: http://adomas.org/javascript-mouse-wheel/
 * prototype extension by "Frank Monnerjahn" themonnie @gmail.com
 */
Object.extend(Event, {
    wheel:function (event){
	    var delta = 0;
	    if (!event) event = window.event;
	    if (event.wheelDelta) {
		delta = event.wheelDelta/120;
		if (window.opera) delta = -delta;
	    } else if (event.detail) { delta = -event.detail/2;     }
	    // Mozilla needs to be less that 1/3.
//	    } else if (event.detail) { delta = -event.detail;     }
	    if (delta < 0)
	      return -Math.round(-delta); //Safari Round
	    else
	      return Math.round(delta); //Safari Round
        }
    });
/*
 * end of extension
 */

var ZoomableWithCanvas = { };
ZoomableWithCanvas = Class.create({
  initialize: function(element, options) {
    // Need to wait until the image is fully loaded to get the right dimensions.
    Event.observe(window, 'load', this.doInit.bindAsEventListener(this, element, options));
  },
  
  doInit: function(event, element, options) {
    element              = $(element);
    this.element         = element;
    this.zoomLevel       = 0;
    this.initialPosition = element.cumulativeOffset();
    this.initialSize     = element.getDimensions();
    this.effect          = false;
    
    if(this.setOptions)
      this.setOptions(options);
    else
      this.options = options || { };

    this.options.zoomFactor = this.options.zoomFactor || 1.2;
    this.mode = this.options.initialMode || 'draggable';
    
    this.container = new Element('div');
    this.container.setStyle({
      width: this.initialSize.width + 'px',
      height: this.initialSize.height + 'px',
      overflow: 'hidden',
      position: 'absolute',
      left: this.initialPosition.left + 'px',
      top: this.initialPosition.top + 'px'});
    document.body.insert(this.container);
    this.container.insert(this.element.remove());
    
    this.elementConstraint = new Element('div');
    this.elementConstraint.setStyle({
      width: this.initialSize.width + 'px',
      height: this.initialSize.height + 'px'});
    this.element.insert({before: this.elementConstraint});
    this.elementConstraint.insert(this.element.remove());
    
    this.elementBundle = new Element('div');
    this.elementBundle.setStyle({
      width: this.initialSize.width + 'px',
      height: this.initialSize.height + 'px',
      position: 'relative',
      left: 0,
      top: 0,
      margin: 0});
    this.element.insert({before: this.elementBundle});
    this.elementBundle.insert(this.element.remove());
    
    this.canvas = new Element('canvas');
    this.canvas.setStyle({
      width: this.initialSize.width + 'px',
      height: this.initialSize.height + 'px',
      position: 'absolute'});
    this.canvas.width = this.initialSize.width;
    this.canvas.height = this.initialSize.height;
    this.element.insert({before: this.canvas});
    
    if (this.mode == 'draggable')
      this.setDraggable(true);
    this.mouseIsDown = false;

    Event.observe(this.elementBundle, 'mousewheel', this.onMouseWheel.bindAsEventListener(this), false);
    Event.observe(this.elementBundle, 'DOMMouseScroll', this.onMouseWheel.bindAsEventListener(this), false);
    
    Event.observe(this.elementBundle, 'mousedown', this.onMouseDown.bindAsEventListener(this), false);
    Event.observe(this.elementBundle, 'mousemove', this.onMouseMove.bindAsEventListener(this), false);
    Event.observe(this.elementBundle, 'mouseup', this.onMouseUp.bindAsEventListener(this), false);

    this.lines = [];
    
    if (this.options.onInitialize)
      this.options.onInitialize(this);
  },
  
  setDraggable: function(newDraggable) {
    if (newDraggable && !this.draggable) {
      this.mode = 'draggable';
      this.draggable = new Draggable(this.elementBundle, {
        starteffect: false,
        endeffect: false,
        snap: function(x,y,draggable) {
	  function constrain(n, lower, upper) {
	    if (n > upper) return upper;
	    else if (n < lower) return lower;
	    else return n;
	  }
	  
	  element_dimensions = draggable.element.getDimensions();
	  parent_dimensions = draggable.element.parentNode.getDimensions();
	  
	  return [
	    constrain(x, 0, parent_dimensions.width - element_dimensions.width),
	    constrain(y, 0, parent_dimensions.height - element_dimensions.height)
	  ]}});
    } else if (!newDraggable) {
      this.draggable.destroy();
      this.draggable = false;
    }
  },
  
  setDrawable: function(newDrawable) {
    if (newDrawable) {
      this.mode = 'drawable';
      this.mouseIsDown = false;
      this.setDraggable(false);
    }
  },
  
  onMouseWheel: function(event) {
    if (event.stop)
      event.stop();
    
    // Wait until the end of the last zoom to process the next.
    if (this.effect) { setTimeout(this.onMouseWheel.bind(this, event), this.effect.finishOn - this.effect.startOn); return; }

    // The very first thing we need is the current dimensions of the
    // zoomed element.
    var dim = this.elementBundle.getDimensions();
    
    // Next, we get the offset of the zoomed element from bottom-right.
    // This can be found from the style of the zoomed element.
    var offset = {x: parseInt(this.elementBundle.getStyle('left')),
		  y: parseInt(this.elementBundle.getStyle('top'))};
    
    // Now, as we have the original element size, we can get the
    // position of the center of the viewport/container as a fraction
    // of the width and height of the zoomed element (measured from
    // the bottom-right).
    var viewCenter = {x: (this.initialSize.width / 2 + offset.x) / dim.width,
		      y: (this.initialSize.height / 2 + offset.y) / dim.height};
    
    // Get the appropriate zoom factor from the direction and rate the
    // wheel was scrolled.
    var zoomFactor = Math.pow(this.options.zoomFactor, Event.wheel(event));
    
    // Now, generate the new dimensions and offset from the direction
    // and rate the wheel was scrolled.
    var newDim = {width: dim.width * zoomFactor,
		  height: dim.height * zoomFactor};
    
    // The new dimensions can't be smaller than the viewport
    // dimensions.
    if (newDim.width < this.initialSize.width)
      newDim.width = this.initialSize.width;
    if (newDim.height < this.initialSize.height)
      newDim.height = this.initialSize.height;
    
    var newOffset = {x: viewCenter.x * newDim.width - this.initialSize.width / 2,
		     y: viewCenter.y * newDim.height - this.initialSize.height / 2};
    
    // Zooming out has a few special cases to consider:
    if (Event.wheel(event) < 0) {
      // The offset can't have negative dimensions (right/bottom overflow)
      if (newOffset.x < 0) newOffset.x = 0;
      if (newOffset.y < 0) newOffset.y = 0;
      
      // It also can't be greater than the new dimensions when added
      // to the viewport dimensions (top/right overflow)
      if (newOffset.x + this.initialSize.width > newDim.width)
	newOffset.x = newDim.width - this.initialSize.width;
      if (newOffset.y + this.initialSize.height > newDim.height)
	newOffset.y = newDim.height - this.initialSize.height;
    }
    
    // Now we make any needed changes.
    this.elementBundle.setStyle({
      'left': newOffset.x + 'px',
      'top': newOffset.y + 'px',
      'width': newDim.width + 'px',
      'height': newDim.height + 'px'});
    this.element.setStyle({
      'width': newDim.width + 'px',
      'height': newDim.height + 'px'});
    this.canvas.setStyle({
      'width': newDim.width + 'px',
      'height': newDim.height + 'px'});
    this.canvas.width = newDim.width;
    this.canvas.height = newDim.height;
    this.refreshCanvas();
    
    // Note that we constrain the movement of the zoomed element such
    // that we can see the whole thing by scrolling.  This means each
    // dimension is twice the size of the zoomed element's
    // corresponding dimension, minus the corresponding dimension of
    // the viewport, and its margins are the negative difference of
    // the zoomed element's and the viewport's dimensions.
    this.elementConstraint.setStyle({
      'width': (2 * newDim.width - this.initialSize.width) + 'px',
      'height': (2 * newDim.height - this.initialSize.height) + 'px',
      'marginLeft': (-(newDim.width - this.initialSize.width)) + 'px',
      'marginTop': (-(newDim.height - this.initialSize.height)) + 'px'});
    
    // Set the zoom level.
    if (this.zoomLevel + Event.wheel(event) >= 0)
      this.zoomLevel += Event.wheel(event);
  },

  onMouseDown: function(event) {
    if (this.mode == 'drawable') {
      if (event.stop)
        event.stop();
      
      var canvasX = event.pointerX() - this.canvas.viewportOffset().left
      var canvasY = event.pointerY() - this.canvas.viewportOffset().top;
      
      // Convert the position to an image-relative position.
      canvasX = canvasX * (this.element.naturalWidth / this.canvas.offsetWidth);
      canvasY = canvasY * (this.element.naturalHeight / this.canvas.offsetHeight);
      
      var line = []
      line.push([canvasX, canvasY]);
      this.lines.push(line);
      this.mouseIsDown = true;
    }
  },
  
  onMouseMove: function(event) {
    if (this.mode == 'drawable' && this.mouseIsDown) {
      if (event.stop)
        event.stop();
      
      var canvasX = event.pointerX() - this.canvas.viewportOffset().left
      var canvasY = event.pointerY() - this.canvas.viewportOffset().top;
      
      // Convert the position to an image-relative position.
      canvasX = canvasX * (this.element.naturalWidth / this.canvas.offsetWidth);
      canvasY = canvasY * (this.element.naturalHeight / this.canvas.offsetHeight);
      
      var line = this.lines[this.lines.length - 1];
      line.push([canvasX, canvasY]);
      this.refreshCanvas();
      this.mouseIsDown = true;
    }
  },
  
  onMouseUp: function(event) {
    if (this.mode == 'drawable' && this.mouseIsDown) {
      if (event.stop)
        event.stop();
      
      var canvasX = event.pointerX() - this.canvas.viewportOffset().left
      var canvasY = event.pointerY() - this.canvas.viewportOffset().top;
      
      // Convert the position to an image-relative position.
      canvasX = canvasX * (this.element.naturalWidth / this.canvas.offsetWidth);
      canvasY = canvasY * (this.element.naturalHeight / this.canvas.offsetHeight);
      
      var line = this.lines[this.lines.length - 1];
      line.push([canvasX, canvasY]);
      this.refreshCanvas();
      this.mouseIsDown = false;
    }
  },

  refreshCanvas: function() {
    var i, j, line, x, y;
    var ctx = this.canvas.getContext("2d");

    ctx.clearRect(0, 0, this.canvas.width, this.canvas.height);
    
    for (i = 0; i < this.lines.length; i++) {
      line = this.lines[i];
      if (line.length == 1) {
        // Do nothing.
      } else {
        for (j = 0; j < line.length; j++) {
          if (j == 0) {
            ctx.beginPath();
            x = line[j][0] * (this.canvas.width / this.element.naturalWidth);
            y = line[j][1] * (this.canvas.height / this.element.naturalHeight);
            ctx.moveTo(x, y);
          } else {
            x = line[j][0] * (this.canvas.width / this.element.naturalWidth);
            y = line[j][1] * (this.canvas.height / this.element.naturalHeight);
            ctx.lineTo(x, y);
          }
        }
        ctx.strokeStyle = "#f00";
        // Width should be 5 at 100% zoom.
        ctx.lineWidth = 5 * (this.canvas.width / this.initialSize.width);
        ctx.stroke();
      }
    }
  },
  
  clearCanvas: function() {
    var ctx = this.canvas.getContext("2d");
    ctx.clearRect(0, 0, this.canvas.width, this.canvas.height);
    this.lines = [];
  }
});