require File.dirname(__FILE__) + '/../test_helper'
require 'anime_continuum_controller'

# Reraise errors caught by the controller.
class AnimeContinuumController; def rescue_action(e) raise e end; end

class AnimeContinuumControllerTest < ActionController::TestCase
  # Anyone can see the index.
  def test_index
    login_as :quentin
    
    get :index
    assert_response :success
    assert_template 'list'
  end
  
  # Anyone can see the list.
  def test_list
    login_as :quentin
    
    get :list
    assert_response :success
    assert_template 'list'
  end
  
  # Anyone can see show.
  def test_show
    login_as :quentin
    
    get :show, :id => anime_continuums(:genshiken).id
    assert_response :success
    assert_template 'show'
    
    # And it should have @anime_continuum set to the requested continuum.
    assert_equal anime_continuums(:genshiken), assigns(:anime_continuum)
  end
  
  # Anyone can see new.
  def test_new
    login_as :quentin
    
    get :new
    assert_response :success
    assert_template 'new'
    
    # And it should have @anime_continuum set to a new continuum.
    #assert_equal AnimeContinuum.new(), assigns(:anime_continuum)
  end
  
  # Anyone can create.
  def test_create_normal_user
    login_as :quentin
    
    # Set @anime_continuum before POSTing create.
    assert_difference 'AnimeContinuum.count' do
      post :create, :anime_continuum => create_continuum
      assert_redirected_to new_anime_continuum_path
    end
    
    # It should have gone successfully.
    assert_not_nil flash[:notice]
    assert_nil flash[:error]
    assert_nil flash[:anime_continuum]
    assert_nil flash[:submission_note]
  end
  
  def test_bad_create_normal_user
    login_as :quentin
    
    # Constrast when we have a bad POST
    assert_no_difference 'AnimeContinuum.count' do
      post :create, :anime_continuum => create_continuum(:name => anime_continuums(:genshiken).name)
      assert_redirected_to new_anime_continuum_path
    end
    
    # It should NOT have gone successfully.
    assert_nil flash[:notice]
    assert_not_nil flash[:error]
    assert flash[:error_is_html]
    assert_not_nil flash[:anime_continuum]
    assert_nil flash[:submission_note]
  end
  
  def test_complex_create_normal_user
    login_as :quentin
    
    # And now a complex success test with comments and alternate names.
    assert_difference 'AnimeContinuum.count' do
      assert_difference 'AlternateName.count', 3 do
        assert_difference 'Comment.count' do
          post :create, :anime_continuum => create_continuum(:name => '1234', :alternate_names => {1 => 'a', 2 => 'a', 3 => '', 4 => '', 5 => nil, 6 => 'b', 7 => 'c'}), :submission_note => 'A submission note.'
          assert_redirected_to new_anime_continuum_path
        end
      end
    end
    
    # It should have gone successfully.
    assert_not_nil flash[:notice]
    assert_nil flash[:error]
    assert_nil flash[:anime_continuum]
    assert_nil flash[:submission_note]
    
    # Should get one if we wrote the test right.
    anime_continuum = AnimeContinuum.find_by_name('1234')
    assert_not_nil anime_continuum
    
    # Main users should not auto-approve...
    assert anime_continuum.pending_approval?
    assert_equal users(:quentin), anime_continuum.creator
    
    # It should have eliminated nils, empties, and duplicates for the alternate_names.
    assert_equal 3, anime_continuum.alternate_names.count
    assert anime_continuum.alternate_names.to_ary.include?('a')
    assert anime_continuum.alternate_names.to_ary.include?('b')
    assert anime_continuum.alternate_names.to_ary.include?('c')
    
    # And we should find the comment attached.
    assert_equal 1, anime_continuum.comments.count
    assert_equal 'A submission note.', anime_continuum.comments[0].text
    assert_equal :submission_note, anime_continuum.comments[0].comment_type
  end
  
  def test_complex_bad_create_normal_user
    login_as :quentin
    
    # And lastly a complex failure test with comments and alternate names.
    assert_no_difference 'AnimeContinuum.count' do
      assert_no_difference 'AlternateName.count' do
        assert_no_difference 'Comment.count' do
          post :create, :anime_continuum => create_continuum(:name => nil, :alternate_names => {1 => 'a', 2 => 'a', 3 => '', 4 => '', 5 => nil, 6 => 'b', 7 => 'c'}), :submission_note => 'A submission note.'
          assert_redirected_to new_anime_continuum_path
        end
      end
    end
    
    # It should have gone successfully.
    assert_nil flash[:notice]
    assert_not_nil flash[:error]
    assert flash[:error_is_html]
    assert_not_nil flash[:anime_continuum]
    assert_not_nil flash[:submission_note]
  end
  
  def test_create_approver_user
    login_as :quentin
    
    # Oh right.  Approvers and above should have their changes auto-approve.
    user = users(:quentin)
    user.roles << roles(:anime_continuum_approver)
    user.save
    
    assert_difference 'AnimeContinuum.count' do
      post :create, :anime_continuum => create_continuum(:name => 'Evangelion')
      assert_redirected_to new_anime_continuum_path
    end
    
    # Should get one if we wrote the test right.
    anime_continuum = AnimeContinuum.find_by_name('Evangelion')
    assert_not_nil anime_continuum
    
    assert anime_continuum.approved?
    assert_equal users(:quentin), anime_continuum.creator
    assert_equal users(:quentin), anime_continuum.approver
  end
  
  # Approvers and above can see edit.
  def test_edit_normal_user
    login_as :quentin
    
    get :edit, :id => anime_continuums(:genshiken).id
    assert_response 401
  end
  
  def test_edit_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_continuum_approver)
    user.save
    
    get :edit, :id => anime_continuums(:genshiken).id
    assert_response :success
    assert_template 'edit'
    assert_equal anime_continuums(:genshiken), assigns(:anime_continuum)
  end
  
  def test_edit_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_continuum_moderator)
    user.save
    
    get :edit, :id => anime_continuums(:genshiken).id
    assert_response :success
    assert_template 'edit'
    assert_equal anime_continuums(:genshiken), assigns(:anime_continuum)
  end
  
  def test_edit_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_super_moderator)
    user.save
    
    get :edit, :id => anime_continuums(:genshiken).id
    assert_response :success
    assert_template 'edit'
    assert_equal anime_continuums(:genshiken), assigns(:anime_continuum)
  end
  
  # Approvers can approve.
  def test_approve_normal_user
    login_as :quentin
    
    put :approve, :id => anime_continuums(:nanoha).id
    assert_response 401
#    assert_redirected_to new_anime_continuum_path
  end
  
  def test_approve_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_continuum_approver)
    user.save
    
    put :approve, :id => anime_continuums(:nanoha).id
    assert_redirected_to edit_anime_continuum_path(anime_continuums(:nanoha).id)
    assert_not_nil flash[:notice]
    assert AnimeContinuum.find(anime_continuums(:nanoha).id).approved?
  end
  
  def test_approve_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_continuum_moderator)
    user.save
    
    put :approve, :id => anime_continuums(:nanoha).id
    assert_redirected_to edit_anime_continuum_path(anime_continuums(:nanoha).id)
    assert_not_nil flash[:notice]
    assert AnimeContinuum.find(anime_continuums(:nanoha).id).approved?
  end
  
  def test_approve_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_super_moderator)
    user.save
    
    put :approve, :id => anime_continuums(:nanoha).id
    assert_redirected_to edit_anime_continuum_path(anime_continuums(:nanoha).id)
    assert_not_nil flash[:notice]
    assert AnimeContinuum.find(anime_continuums(:nanoha).id).approved?
    
    # Okay, but Genshiken SHOULDN'T approve until it's clean.
    put :approve, :id => anime_continuums(:genshiken).id
    assert_redirected_to edit_anime_continuum_path(anime_continuums(:genshiken).id)
    assert_not_nil flash[:error]
    assert AnimeContinuum.find(anime_continuums(:genshiken).id).pending_changes?
    
    continuum = anime_continuums(:genshiken)
    continuum.delete_pending_change :name, :change, "Kujibiki Unbalance"
    continuum.save
    
    put :approve, :id => anime_continuums(:genshiken).id
    assert_redirected_to edit_anime_continuum_path(anime_continuums(:genshiken).id)
    assert_not_nil flash[:notice]
    assert AnimeContinuum.find(anime_continuums(:genshiken).id).approved?
  end
  
  # Approvers can approve.
  def test_approve_pending_change_normal_user
    login_as :quentin
    
    put :approve_pending_change, :id => anime_continuums(:genshiken).id, :change_action => 'change', :key => :name, :value => 'Kamichu!'
    assert_response 401
#    assert_redirected_to new_anime_continuum_path
  end
  
  def test_approve_pending_change_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_continuum_approver)
    user.save
    
    put :approve_pending_change, :id => anime_continuums(:genshiken).id, :change_action => 'change', :key => :name, :value => 'Genshiken/Kujibiki Unbalance'
    assert_redirected_to edit_anime_continuum_path(anime_continuums(:genshiken).id)
    assert_not_nil flash[:notice]
    assert_equal 'Genshiken', AnimeContinuum.find(anime_continuums(:genshiken).id).name
    assert AnimeContinuum.find(anime_continuums(:genshiken).id).pending_changes[:name]['Genshiken/Kujibiki Unbalance'].include?(user.id)
  end
  
  def test_approve_pending_change_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_continuum_moderator)
    user.save
    
    put :approve_pending_change, :id => anime_continuums(:genshiken).id, :change_action => 'change', :key => :name, :value => 'Genshiken/Kujibiki Unbalance'
    assert_redirected_to edit_anime_continuum_path(anime_continuums(:genshiken).id)
    assert_not_nil flash[:notice]
    assert_equal 'Genshiken/Kujibiki Unbalance', AnimeContinuum.find(anime_continuums(:genshiken).id).name
    assert !AnimeContinuum.find(anime_continuums(:genshiken).id).pending_changes[:name].include?('Genshiken/Kujibiki Unbalance')
  end
  
  def test_approve_pending_change_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_super_moderator)
    user.save
    
    put :approve_pending_change, :id => anime_continuums(:genshiken).id, :change_action => 'change', :key => :name, :value => 'Genshiken/Kujibiki Unbalance'
    assert_redirected_to edit_anime_continuum_path(anime_continuums(:genshiken).id)
    assert_not_nil flash[:notice]
    assert_equal 'Genshiken/Kujibiki Unbalance', AnimeContinuum.find(anime_continuums(:genshiken).id).name
    assert !AnimeContinuum.find(anime_continuums(:genshiken).id).pending_changes[:name].include?('Genshiken/Kujibiki Unbalance')
  end
  
  # Approvers can reject.
  def test_reject_pending_change_normal_user
    login_as :quentin
    
    put :reject_pending_change, :id => anime_continuums(:genshiken).id, :change_action => 'change', :key => :years, :value => '2006'
    assert_response 401
#    assert_redirected_to new_anime_continuum_path
  end
  
  def test_reject_pending_change_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_continuum_approver)
    user.save
    
    put :reject_pending_change, :id => anime_continuums(:genshiken).id, :change_action => 'change', :key => :name, :value => 'Kujibiki Unbalance'
    assert_redirected_to edit_anime_continuum_path(anime_continuums(:genshiken).id)
    assert_not_nil flash[:notice]
    assert_equal 'Genshiken', AnimeContinuum.find(anime_continuums(:genshiken).id).name
    assert !AnimeContinuum.find(anime_continuums(:genshiken).id).pending_changes[:name].include?('Kujibiki Unbalance')
    
    put :reject_pending_change, :id => anime_continuums(:genshiken).id, :change_action => 'change', :key => :name, :value => 'Kujibiki Unbalance'
    assert_redirected_to edit_anime_continuum_path(anime_continuums(:genshiken).id)
    assert_not_nil flash[:notice]
    assert_equal 'Genshiken', AnimeContinuum.find(anime_continuums(:genshiken).id).name
    assert !AnimeContinuum.find(anime_continuums(:genshiken).id).pending_changes[:name].include?('Kujibiki Unbalance')
  end
  
  def test_reject_pending_change_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_continuum_moderator)
    user.save
    
    put :reject_pending_change, :id => anime_continuums(:genshiken).id, :change_action => 'change', :key => :name, :value => 'Kujibiki Unbalance'
    assert_redirected_to edit_anime_continuum_path(anime_continuums(:genshiken).id)
    assert_not_nil flash[:notice]
    assert_equal 'Genshiken', AnimeContinuum.find(anime_continuums(:genshiken).id).name
    assert !AnimeContinuum.find(anime_continuums(:genshiken).id).pending_changes[:name].include?('Kujibiki Unbalance')
  end
  
  def test_reject_pending_change_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_super_moderator)
    user.save
    
    put :reject_pending_change, :id => anime_continuums(:genshiken).id, :change_action => 'change', :key => :name, :value => 'Kujibiki Unbalance'
    assert_redirected_to edit_anime_continuum_path(anime_continuums(:genshiken).id)
    assert_not_nil flash[:notice]
    assert_equal 'Genshiken', AnimeContinuum.find(anime_continuums(:genshiken).id).name
    assert !AnimeContinuum.find(anime_continuums(:genshiken).id).pending_changes[:name].include?('Kujibiki Unbalance')
  end
  
  # Only moderators can add an alternate name.
  def test_add_alternate_name_normal_user
    login_as :quentin
    
    # TODO: reimplement me
    assert_no_difference 'AlternateName.count' do
      post :add_alternate_name, :id => anime_continuums(:genshiken).id, :new_alternate_name => 'The Society for the Study of Modern Visual Culture'
      assert_response 401
    end
    
    continuum = AnimeContinuum.find(anime_continuums(:genshiken).id)
    assert !continuum.alternate_names.include?('The Society for the Study of Modern Visual Culture')
  end
  
  def test_add_alternate_name_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_continuum_approver)
    user.save
    
    continuum = AnimeContinuum.find(anime_continuums(:genshiken).id)
    assert continuum.pending_changes?
    
    assert_no_difference 'AlternateName.count' do
      put :add_alternate_name, :id => anime_continuums(:genshiken).id, :new_alternate_name => 'The Society for the Study of Modern Visual Culture'
      assert_redirected_to edit_anime_continuum_path(anime_continuums(:genshiken))
    end
    
    # However, approvers should have the name added to pending_changes.
    continuum = AnimeContinuum.find(anime_continuums(:genshiken).id)
    assert !continuum.alternate_names.include?('The Society for the Study of Modern Visual Culture')
    assert continuum.pending_changes?
    assert_not_nil continuum.pending_changes
    assert_not_nil continuum.pending_changes[:alternate_names]
    assert_not_nil continuum.pending_changes[:alternate_names][:add]
    assert_not_nil continuum.pending_changes[:alternate_names][:add]['The Society for the Study of Modern Visual Culture']
    assert continuum.pending_changes[:alternate_names][:add]['The Society for the Study of Modern Visual Culture'].include?(users(:quentin).id)
  end
  
  def test_add_alternate_name_moderator
    login_as :quentin
    
    # And as for moderators, it should stick immediately.
    user = users(:quentin)
    user.roles << roles(:anime_continuum_moderator)
    user.save
    
    assert_difference 'AlternateName.count' do
      put :add_alternate_name, :id => anime_continuums(:genshiken).id, :new_alternate_name => 'The Society for the Study of Modern Visual Culture'
      assert_redirected_to edit_anime_continuum_path(anime_continuums(:genshiken))
    end
    
    continuum = AnimeContinuum.find(anime_continuums(:genshiken).id)
    assert continuum.alternate_names.include?('The Society for the Study of Modern Visual Culture')
#    assert_nil continuum.pending_changes[:alternate_names][:add]['My Little Goddess']
  end
  
  # Only moderators can delete an alternate name.
  def test_remove_alternate_name_normal_user
    login_as :quentin
    
    # TODO: reimplement me
    assert_no_difference 'AlternateName.count' do
      put :remove_alternate_name, :id => anime_continuums(:nanoha).id, :name => 'Maho Shojo Lyrical Nanoha'
      assert_response 401
    end
    
    continuum = AnimeContinuum.find(anime_continuums(:nanoha).id)
    assert continuum.alternate_names.include?('Maho Shojo Lyrical Nanoha')
  end
    
  def test_remove_alternate_name_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_continuum_approver)
    user.save
    
    continuum = AnimeContinuum.find(anime_continuums(:nanoha).id)
    
    assert_no_difference 'AlternateName.count' do
      put :remove_alternate_name, :id => anime_continuums(:nanoha).id, :name => 'Maho Shojo Lyrical Nanoha'
      assert_redirected_to edit_anime_continuum_path(anime_continuums(:nanoha))
    end
    
    # However, approvers should have the name added to pending_changes.
    continuum = AnimeContinuum.find(anime_continuums(:nanoha).id)
    assert continuum.alternate_names.include?('Maho Shojo Lyrical Nanoha')
    assert continuum.pending_approval?
    assert_not_nil continuum.pending_changes
    assert_not_nil continuum.pending_changes[:alternate_names]
    assert_not_nil continuum.pending_changes[:alternate_names][:remove]
    assert_not_nil continuum.pending_changes[:alternate_names][:remove]['Maho Shojo Lyrical Nanoha']
    assert continuum.pending_changes[:alternate_names][:remove]['Maho Shojo Lyrical Nanoha'].include?(users(:quentin).id)
  end
  
  def test_remove_nonexistent_alternate_name_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_continuum_approver)
    user.save
    
    # Deleting a non-existent name should give an error.
    continuum = AnimeContinuum.find(anime_continuums(:nanoha).id)
    
    assert_no_difference 'AlternateName.count' do
      put :remove_alternate_name, :id => anime_continuums(:nanoha).id, :name => 'Mahurgh Shojo Lyrical Nanoha'
      assert_redirected_to edit_anime_continuum_path(anime_continuums(:nanoha))
    end
    
    continuum = AnimeContinuum.find(anime_continuums(:nanoha).id)
    assert !continuum.pending_changes?
    assert_nil continuum.pending_changes
  end
  
  def test_remove_alternate_name_moderator
    login_as :quentin
    
    # And as for moderators, it should stick immediately.
    user = users(:quentin)
    user.roles << roles(:anime_continuum_moderator)
    user.save
    
    assert_difference 'AlternateName.count', -1 do
      put :remove_alternate_name, :id => anime_continuums(:nanoha).id, :name => 'Maho Shojo Lyrical Nanoha'
      assert_redirected_to edit_anime_continuum_path(anime_continuums(:nanoha))
    end
    
    continuum = AnimeContinuum.find(anime_continuums(:nanoha).id)
    assert !continuum.alternate_names.include?('Maho Shojo Lyrical Nanoha')
    assert !continuum.pending_changes?
    assert_nil continuum.pending_changes
    
    # Deleting a non-existent name should give an error.
    continuum = AnimeContinuum.find(anime_continuums(:nanoha).id)
    
    assert_no_difference 'AlternateName.count' do
      put :remove_alternate_name, :id => anime_continuums(:nanoha).id, :name => 'Mahurgh Shojo Lyrical Nanoha'
      assert_redirected_to edit_anime_continuum_path(anime_continuums(:nanoha))
    end
    
    continuum = AnimeContinuum.find(anime_continuums(:nanoha).id)
    assert !continuum.pending_changes?
    assert_nil continuum.pending_changes
  end
  
  # Moderators can destroy.
  def test_destroy_normal_user
    login_as :quentin
    
    delete :destroy, :id => anime_continuums(:genshiken).id
    assert_response 401
    assert_nothing_raised { AnimeContinuum.find(anime_continuums(:genshiken).id) }
  end
  
  def test_destroy_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_continuum_approver)
    user.save
    
    delete :destroy, :id => anime_continuums(:genshiken).id
    assert_response 401
    assert_nothing_raised { AnimeContinuum.find(anime_continuums(:genshiken).id) }
  end
  
  def test_destroy_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_continuum_moderator)
    user.save
    
    delete :destroy, :id => anime_continuums(:genshiken).id
    assert_redirected_to manage_anime_continuum_path
    assert_raise ActiveRecord::RecordNotFound do; AnimeContinuum.find(anime_continuums(:genshiken).id); end
  end
  
  def test_destroy_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_super_moderator)
    user.save
    
    delete :destroy, :id => anime_continuums(:nanoha).id
    assert_redirected_to manage_anime_continuum_path
    assert_raise ActiveRecord::RecordNotFound do; AnimeContinuum.find(anime_continuums(:nanoha).id); end
  end
  
  # Approvers can manage.
  def test_manage_normal_user
    login_as :quentin
    
    get :manage
    assert_response 401
  end
  
  def test_manage_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_continuum_approver)
    user.save
    
    get :manage
    assert_response :success
    assert_template 'manage'
  end
  
  def test_manage_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_continuum_moderator)
    user.save
    
    get :manage
    assert_response :success
    assert_template 'manage'
  end
  
  def test_manage_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_super_moderator)
    user.save
    
    get :manage
    assert_response :success
    assert_template 'manage'
  end
  
  def test_select_merge_normal_user
    login_as :quentin
    
    get :select_merge, :id => anime_continuums(:kujibiki).id
    assert_response 401
  end
  
  def test_select_merge_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_continuum_approver)
    user.save
    
    get :select_merge, :id => anime_continuums(:kujibiki).id
    assert_response 401
  end
  
  def test_select_merge_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_continuum_moderator)
    user.save
    
    get :select_merge, :id => anime_continuums(:kujibiki).id
    assert_response :success
    assert_template 'select_merge'
    
    # And it should have @anime_continuum set to the requested continuum.
    assert_equal anime_continuums(:kujibiki), assigns(:anime_continuum)
  end
  
  def test_select_merge_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_super_moderator)
    user.save
    
    get :select_merge, :id => anime_continuums(:kujibiki).id
    assert_response :success
    assert_template 'select_merge'
    
    # And it should have @anime_continuum set to the requested continuum.
    assert_equal anime_continuums(:kujibiki), assigns(:anime_continuum)
  end
  
  # TODO: search_select_merge
  
  def test_merge_into_normal_user
    login_as :quentin
    
    put :merge_into, :id => anime_continuums(:kujibiki).id, :dest => anime_continuums(:genshiken).id
    assert_response 401
    assert_nothing_raised { AnimeContinuum.find(anime_continuums(:kujibiki).id) }
	assert !AnimeContinuum.find(anime_continuums(:genshiken).id).alternate_names.include?('Kujibiki Unbalance')
  end
  
  def test_merge_into_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_continuum_approver)
    user.save
    
    put :merge_into, :id => anime_continuums(:kujibiki).id, :dest => anime_continuums(:genshiken).id
    assert_response 401
    assert_nothing_raised { AnimeContinuum.find(anime_continuums(:kujibiki).id) }
	assert !AnimeContinuum.find(anime_continuums(:genshiken).id).alternate_names.include?('Kujibiki Unbalance')
  end
  
  def test_merge_into_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_continuum_moderator)
    user.save
    
    put :merge_into, :id => anime_continuums(:kujibiki).id, :dest => anime_continuums(:genshiken).id
    assert_redirected_to manage_anime_continuum_path
	
    assert_raise ActiveRecord::RecordNotFound do; AnimeContinuum.find(anime_continuums(:kujibiki).id); end
	assert AnimeContinuum.find(anime_continuums(:genshiken).id).alternate_names.include?('Kujibiki Unbalance')
  end
  
  def test_merge_into_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_super_moderator)
    user.save
    
    put :merge_into, :id => anime_continuums(:kujibiki).id, :dest => anime_continuums(:genshiken).id
    assert_redirected_to manage_anime_continuum_path
	
    assert_raise ActiveRecord::RecordNotFound do; AnimeContinuum.find(anime_continuums(:kujibiki).id); end
	assert AnimeContinuum.find(anime_continuums(:genshiken).id).alternate_names.include?('Kujibiki Unbalance')
  end

protected
  def create_continuum(options = {})
    { :name => 'Love Hina', :alternate_names => {} }.merge(options)
  end
end
