require File.dirname(__FILE__) + '/../test_helper'
require 'anime_scene_controller'

# Reraise errors caught by the controller.
class AnimeSceneController; def rescue_action(e) raise e end; end

class AnimeSceneControllerTest < ActionController::TestCase
  # Anyone can see the index.
  def test_index
    login_as :quentin
    
    get :index
    assert_response :success
    assert_template 'new'
  end
  
  # Anyone can see the list.
  def test_list
    login_as :quentin
    
    get :list
    assert_response :success
    assert_template 'list'
  end
  
  # Anyone can see show.
  def test_show
    login_as :quentin
    
    get :show, :id => anime_scenes(:yamato).id
    assert_response :success
    assert_template 'show'
    
    # And it should have @anime_scene set to the requested trivia question.
    assert_equal anime_scenes(:yamato), assigns(:anime_scene)
  end
  
  # Anyone can see new.
  def test_new
    login_as :quentin
    
    get :new
    assert_response :success
    assert_template 'new'
    
    # And it should have @anime_scene set to a new trivia question.
    #assert_equal AnimeScene.new(), assigns(:anime_scene)
  end
  
  # TODO: Anyone can create.
#  def test_create_normal_user
#    login_as :quentin
#    
#    # Set @anime_scene before POSTing create.
#    assert_difference 'AnimeScene.count' do
#      post :create, :anime_scene => create_scene, :anime_series => {:name => 'Kamichur?'}, :genre_checkbox => {'comedy' => '1'}
#      assert_redirected_to new_anime_scene_path
#    end
#    
#    # It should have gone successfully.
#    assert_not_nil flash[:notice]
#    assert_nil flash[:error]
#    assert_nil flash[:anime_scene]
#    assert_nil flash[:anime_series]
#  end
  
#  def test_bad_create_normal_user
#    login_as :quentin
#    
#    # Constrast when we have a bad POST
#    assert_no_difference 'AnimeScene.count' do
#      post :create, :anime_scene => create_scene(:image => nil), :anime_series => {:name => 'Kamichur?'}, :genre_checkbox => {'comedy' => '1'}
#      assert_redirected_to new_anime_scene_path
#    end
#    
#    # It should NOT have gone successfully.
#    assert_nil flash[:notice]
#    assert_not_nil flash[:error]
#    assert flash[:error_is_html]
#    assert_not_nil flash[:anime_scene]
#    assert_not_nil flash[:anime_series]
#  end
  
#  def test_create_approver_user
#    login_as :quentin
#    
#    # Oh right.  Approvers and above should have their changes auto-approve.
#    user = users(:quentin)
#    user.roles << roles(:anime_scene_approver)
#    user.save
#    
#    assert_difference 'AnimeScene.count' do
#      post :create, :anime_scene => create_scene, :anime_series => {:name => 'Kamichur?'}, :genre_checkbox => {'comedy' => '1'}
#      assert_redirected_to new_anime_scene_path
#    end
#    
#    # TODO: Should get one if we wrote the test right.
#    anime_scene = AnimeScene.find_by_image_file_name('test.png')
#    assert_not_nil anime_scene
    
#    assert anime_scene.approved?
#    assert_equal users(:quentin), anime_scene.creator
#    assert_equal users(:quentin), anime_scene.approver
#  end
  
  # TODO: test_create_with_continuum
  
  # Approvers and above can see edit.
  def test_edit_normal_user
    login_as :quentin
    
    get :edit, :id => anime_scenes(:yamato).id
    assert_response 401
  end
  
  def test_edit_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_scene_approver)
    user.save
    
    get :edit, :id => anime_scenes(:yamato).id
    assert_response :success
    assert_template 'edit'
    assert_equal anime_scenes(:yamato), assigns(:anime_scene)
  end
  
  def test_edit_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_scene_moderator)
    user.save
    
    get :edit, :id => anime_scenes(:yamato).id
    assert_response :success
    assert_template 'edit'
    assert_equal anime_scenes(:yamato), assigns(:anime_scene)
  end
  
  def test_edit_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_super_moderator)
    user.save
    
    get :edit, :id => anime_scenes(:yamato).id
    assert_response :success
    assert_template 'edit'
    assert_equal anime_scenes(:yamato), assigns(:anime_scene)
  end
  
  # Approvers can approve.
  def test_approve_normal_user
    login_as :quentin
    
    put :approve, :id => anime_scenes(:stellvia).id
    assert_response 401
#    assert_redirected_to new_anime_scenes_path
  end
  
  def test_approve_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_scene_approver)
    user.save
    
    put :approve, :id => anime_scenes(:stellvia).id
    assert_redirected_to edit_anime_scene_path(anime_scenes(:stellvia).id)
    assert_not_nil flash[:notice]
    assert AnimeScene.find(anime_scenes(:stellvia).id).approved?
  end
  
  def test_approve_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_scene_moderator)
    user.save
    
    put :approve, :id => anime_scenes(:stellvia).id
    assert_redirected_to edit_anime_scene_path(anime_scenes(:stellvia).id)
    assert_not_nil flash[:notice]
    assert AnimeScene.find(anime_scenes(:stellvia).id).approved?
  end
  
  def test_approve_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_super_moderator)
    user.save
    
    put :approve, :id => anime_scenes(:stellvia).id
    assert_redirected_to edit_anime_scene_path(anime_scenes(:stellvia).id)
    assert_not_nil flash[:notice]
    assert AnimeScene.find(anime_scenes(:stellvia).id).approved?
    
    # Okay, but Yamato SHOULDN'T approve until it's clean.
    put :approve, :id => anime_scenes(:yamato).id
    assert_redirected_to edit_anime_scene_path(anime_scenes(:yamato).id)
    assert_not_nil flash[:error]
    assert AnimeScene.find(anime_scenes(:yamato).id).pending_changes?
    
    scene = anime_scenes(:yamato)
    scene.delete_pending_change :episode, :change, "Space Battleship Yamato"
    scene.save
    
    put :approve, :id => anime_scenes(:yamato).id
    assert_redirected_to edit_anime_scene_path(anime_scenes(:yamato).id)
    assert_not_nil flash[:notice]
    assert AnimeScene.find(anime_scenes(:yamato).id).approved?
  end

  # Approvers can approve.
  def test_approve_pending_change_normal_user
    login_as :quentin
    
    put :approve_pending_change, :id => anime_scenes(:yamato).id, :change_action => 'change', :key => :episode, :value => 'Battleship Yamato'
    assert_response 401
#    assert_redirected_to new_anime_scene_path
  end
  
  def test_approve_pending_change_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_scene_approver)
    user.save
    
    put :approve_pending_change, :id => anime_scenes(:yamato).id, :change_action => 'change', :key => :episode, :value => 'Battleship Yamato'
    assert_redirected_to edit_anime_scene_path(anime_scenes(:yamato).id)
    assert_not_nil flash[:notice]
    assert_equal 'The Yamato episode.', AnimeScene.find(anime_scenes(:yamato).id).episode
    assert AnimeScene.find(anime_scenes(:yamato).id).pending_changes[:episode]['Battleship Yamato'].include?(user.id)
  end
  
  def test_approve_pending_change_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_scene_moderator)
    user.save
    
    put :approve_pending_change, :id => anime_scenes(:yamato).id, :change_action => 'change', :key => :episode, :value => 'Battleship Yamato'
    assert_redirected_to edit_anime_scene_path(anime_scenes(:yamato).id)
    assert_not_nil flash[:notice]
    assert_equal 'Battleship Yamato', AnimeScene.find(anime_scenes(:yamato).id).episode
    assert !AnimeScene.find(anime_scenes(:yamato).id).pending_changes[:episode].include?('Battleship Yamato')
  end
  
  def test_approve_pending_change_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_super_moderator)
    user.save
    
    put :approve_pending_change, :id => anime_scenes(:yamato).id, :change_action => 'change', :key => :episode, :value => 'Battleship Yamato'
    assert_redirected_to edit_anime_scene_path(anime_scenes(:yamato).id)
    assert_not_nil flash[:notice]
    assert_equal 'Battleship Yamato', AnimeScene.find(anime_scenes(:yamato).id).episode
    assert !AnimeScene.find(anime_scenes(:yamato).id).pending_changes[:episode].include?('Battleship Yamato')
  end
  
  # Approvers can reject.
  def test_reject_pending_change_normal_user
    login_as :quentin
    
    put :reject_pending_change, :id => anime_scenes(:yamato).id, :change_action => 'change', :key => :episode, :value => 'Space Battleship Yamato'
    assert_response 401
#    assert_redirected_to new_anime_scenes_path
  end
  
  def test_reject_pending_change_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_scene_approver)
    user.save
    
    put :reject_pending_change, :id => anime_scenes(:yamato).id, :change_action => 'change', :key => :episode, :value => 'Space Battleship Yamato'
    assert_redirected_to edit_anime_scene_path(anime_scenes(:yamato).id)
    assert_not_nil flash[:notice]
    assert_equal 'The Yamato episode.', AnimeScene.find(anime_scenes(:yamato).id).episode
    assert AnimeScene.find(anime_scenes(:yamato).id).pending_changes[:episode].include?('Space Battleship Yamato')

    put :reject_pending_change, :id => anime_scenes(:yamato).id, :change_action => 'change', :key => :episode, :value => 'Battleship Yamato'
    assert_redirected_to edit_anime_scene_path(anime_scenes(:yamato).id)
    assert_not_nil flash[:notice]
    assert_equal 'The Yamato episode.', AnimeScene.find(anime_scenes(:yamato).id).episode
    assert !AnimeScene.find(anime_scenes(:yamato).id).pending_changes[:episode]['Battleship Yamato'].include?(users(:quentin).id)
  end
  
  def test_reject_pending_change_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_scene_moderator)
    user.save
    
    put :reject_pending_change, :id => anime_scenes(:yamato).id, :change_action => 'change', :key => :episode, :value => 'Space Battleship Yamato'
    assert_redirected_to edit_anime_scene_path(anime_scenes(:yamato).id)
    assert_not_nil flash[:notice]
    assert_equal 'The Yamato episode.', AnimeScene.find(anime_scenes(:yamato).id).episode
    assert !AnimeScene.find(anime_scenes(:yamato).id).pending_changes[:episode].include?('Space Battleship Yamato')
  end
  
  def test_reject_pending_change_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_super_moderator)
    user.save
    
    put :reject_pending_change, :id => anime_scenes(:yamato).id, :change_action => 'change', :key => :episode, :value => 'Space Battleship Yamato'
    assert_redirected_to edit_anime_scene_path(anime_scenes(:yamato).id)
    assert_not_nil flash[:notice]
    assert_equal 'The Yamato episode.', AnimeScene.find(anime_scenes(:yamato).id).episode
    assert !AnimeScene.find(anime_scenes(:yamato).id).pending_changes[:episode].include?('Space Battleship Yamato')
  end
  
  # Moderators can destroy.
  def test_destroy_normal_user
    login_as :quentin
    
    delete :destroy, :id => anime_scenes(:yamato).id
    assert_response 401
    assert_nothing_raised { AnimeScene.find(anime_scenes(:yamato).id) }
  end
  
  def test_destroy_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_scene_approver)
    user.save
    
    delete :destroy, :id => anime_scenes(:yamato).id
    assert_response 401
    assert_nothing_raised { AnimeScene.find(anime_scenes(:yamato).id) }
  end
  
  def test_destroy_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_scene_moderator)
    user.save
    
    delete :destroy, :id => anime_scenes(:yamato).id
    assert_redirected_to manage_anime_scene_path
    assert_raise ActiveRecord::RecordNotFound do; AnimeScene.find(anime_scenes(:yamato).id); end
  end
  
  def test_destroy_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_super_moderator)
    user.save
    
    delete :destroy, :id => anime_scenes(:yamato).id
    assert_redirected_to manage_anime_scene_path
    assert_raise ActiveRecord::RecordNotFound do; AnimeScene.find(anime_scenes(:yamato).id); end
  end
  
  # Approvers can manage.
  def test_manage_normal_user
    login_as :quentin
    
    get :manage
    assert_response 401
  end
  
  def test_manage_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_scene_approver)
    user.save
    
    get :manage
    assert_response :success
    assert_template 'manage'
  end
  
  def test_manage_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_scene_moderator)
    user.save
    
    get :manage
    assert_response :success
    assert_template 'manage'
  end
  
  def test_manage_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_super_moderator)
    user.save
    
    get :manage
    assert_response :success
    assert_template 'manage'
  end

protected
  def create_scene(options = {})
    { :image => nil, :episode => 'Yes.' }.merge(options)
  end
end
