require File.dirname(__FILE__) + '/../test_helper'
require 'anime_series_controller'

# Reraise errors caught by the controller.
class AnimeSeriesController; def rescue_action(e) raise e end; end

class AnimeSeriesControllerTest < ActionController::TestCase
  # Anyone can see the index.
  def test_index
    login_as :quentin
    
    get :index
    assert_response :success
    assert_template 'list'
  end
  
  # Anyone can see the list.
  def test_list
    login_as :quentin
    
    get :list
    assert_response :success
    assert_template 'list'
  end
  
  # Anyone can see show.
  def test_show
    login_as :quentin
    
    get :show, :id => anime_series(:kamichu).id
    assert_response :success
    assert_template 'show'
    
    # And it should have @anime_series set to the requested series.
    assert_equal anime_series(:kamichu), assigns(:anime_series)
  end
  
  # Anyone can see new.
  def test_new
    login_as :quentin
    
    get :new
    assert_response :success
    assert_template 'new'
    
    # And it should have @anime_series set to a new series.
    #assert_equal AnimeSeries.new(), assigns(:anime_series)
  end
  
  # Anyone can create.
  def test_create_normal_user
    login_as :quentin
    
    # Set @anime_series before POSTing create.
    assert_difference 'AnimeSeries.count' do
      post :create, :anime_series => create_series, :anime_continuum => {:name => ''}, :genre_checkbox => {'comedy' => '1'}
      assert_redirected_to new_anime_series_path
    end
    
    # It should have gone successfully.
    assert_not_nil flash[:notice]
    assert_nil flash[:error]
    assert_nil flash[:anime_series]
    assert_nil flash[:submission_note]
  end
  
  def test_bad_create_normal_user
    login_as :quentin
    
    # Constrast when we have a bad POST
    assert_no_difference 'AnimeSeries.count' do
      post :create, :anime_series => create_series(:anidb_id => anime_series(:kamichu).anidb_id), :anime_continuum => {:name => ''}, :genre_checkbox => {'comedy' => '1'}
      assert_redirected_to new_anime_series_path
    end
    
    # It should NOT have gone successfully.
    assert_nil flash[:notice]
    assert_not_nil flash[:error]
    assert flash[:error_is_html]
    assert_not_nil flash[:anime_series]
    assert_nil flash[:submission_note]
  end
  
  def test_complex_create_normal_user
    login_as :quentin
    
    # And now a complex success test with comments and alternate names.
    assert_difference 'AnimeSeries.count' do
      assert_difference 'AlternateName.count', 3 do
        assert_difference 'Comment.count' do
          post :create, :anime_series => create_series(:name => '1234', :alternate_names => {1 => 'a', 2 => 'a', 3 => '', 4 => '', 5 => nil, 6 => 'b', 7 => 'c'}), :submission_note => 'A submission note.', :anime_continuum => {:name => ''}, :genre_checkbox => {'comedy' => '1'}
          assert_redirected_to new_anime_series_path
        end
      end
    end
    
    # It should have gone successfully.
    assert_not_nil flash[:notice]
    assert_nil flash[:error]
    assert_nil flash[:anime_series]
    assert_nil flash[:submission_note]
    
    # Should get one if we wrote the test right.
    anime_series = AnimeSeries.find_by_name('1234')
    assert_not_nil anime_series
    
    # Main users should not auto-approve...
    assert anime_series.pending_approval?
    assert_equal users(:quentin), anime_series.creator
    
    # It should have eliminated nils, empties, and duplicates for the alternate_names.
    assert_equal 3, anime_series.alternate_names.count
    assert anime_series.alternate_names.to_ary.include?('a')
    assert anime_series.alternate_names.to_ary.include?('b')
    assert anime_series.alternate_names.to_ary.include?('c')
    
    # And we should find the comment attached.
    assert_equal 1, anime_series.comments.count
    assert_equal 'A submission note.', anime_series.comments[0].text
    assert_equal :submission_note, anime_series.comments[0].comment_type
  end
  
  def test_complex_bad_create_normal_user
    login_as :quentin
    
    # And lastly a complex failure test with comments and alternate names.
    assert_no_difference 'AnimeSeries.count' do
      assert_no_difference 'AlternateName.count' do
        assert_no_difference 'Comment.count' do
          post :create, :anime_series => create_series(:name => nil, :alternate_names => {1 => 'a', 2 => 'a', 3 => '', 4 => '', 5 => nil, 6 => 'b', 7 => 'c'}), :submission_note => 'A submission note.', :anime_continuum => {:name => ''}, :genre_checkbox => {'comedy' => '1'}
          assert_redirected_to new_anime_series_path
        end
      end
    end
    
    # It should have gone successfully.
    assert_nil flash[:notice]
    assert_not_nil flash[:error]
    assert flash[:error_is_html]
    assert_not_nil flash[:anime_series]
    assert_not_nil flash[:submission_note]
  end
  
  def test_create_approver_user
    login_as :quentin
    
    # Oh right.  Approvers and above should have their changes auto-approve.
    user = users(:quentin)
    user.roles << roles(:anime_series_approver)
    user.save
    
    assert_difference 'AnimeSeries.count' do
      post :create, :anime_series => create_series(:name => 'Baccano!'), :anime_continuum => {:name => ''}, :genre_checkbox => {'comedy' => '1'}
      assert_redirected_to new_anime_series_path
    end
    
    # Should get one if we wrote the test right.
    anime_series = AnimeSeries.find_by_name('Baccano!')
    assert_not_nil anime_series
    
    assert anime_series.approved?
    assert_equal users(:quentin), anime_series.creator
    assert_equal users(:quentin), anime_series.approver
  end
  
  # TODO: test_create_with_continuum
  
  # Approvers and above can see edit.
  def test_edit_normal_user
    login_as :quentin
    
    get :edit, :id => anime_series(:kamichu).id
    assert_response 401
  end
  
  def test_edit_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_series_approver)
    user.save
    
    get :edit, :id => anime_series(:kamichu).id
    assert_response :success
    assert_template 'edit'
    assert_equal anime_series(:kamichu), assigns(:anime_series)
  end
  
  def test_edit_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_series_moderator)
    user.save
    
    get :edit, :id => anime_series(:kamichu).id
    assert_response :success
    assert_template 'edit'
    assert_equal anime_series(:kamichu), assigns(:anime_series)
  end
  
  def test_edit_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_super_moderator)
    user.save
    
    get :edit, :id => anime_series(:kamichu).id
    assert_response :success
    assert_template 'edit'
    assert_equal anime_series(:kamichu), assigns(:anime_series)
  end
  
  # Approvers can approve.
  def test_approve_normal_user
    login_as :quentin
    
    put :approve, :id => anime_series(:stellvia).id
    assert_response 401
#    assert_redirected_to new_anime_series_path
  end
  
  def test_approve_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_series_approver)
    user.save
    
    put :approve, :id => anime_series(:stellvia).id
    assert_redirected_to edit_anime_series_path(anime_series(:stellvia).id)
    assert_not_nil flash[:notice]
    assert AnimeSeries.find(anime_series(:stellvia).id).approved?
  end
  
  def test_approve_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_series_moderator)
    user.save
    
    put :approve, :id => anime_series(:stellvia).id
    assert_redirected_to edit_anime_series_path(anime_series(:stellvia).id)
    assert_not_nil flash[:notice]
    assert AnimeSeries.find(anime_series(:stellvia).id).approved?
  end
  
  def test_approve_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_super_moderator)
    user.save
    
    put :approve, :id => anime_series(:stellvia).id
    assert_redirected_to edit_anime_series_path(anime_series(:stellvia).id)
    assert_not_nil flash[:notice]
    assert AnimeSeries.find(anime_series(:stellvia).id).approved?
    
    # Okay, but Kamichu SHOULDN'T approve until it's clean.
    put :approve, :id => anime_series(:kamichu).id
    assert_redirected_to edit_anime_series_path(anime_series(:kamichu).id)
    assert_not_nil flash[:error]
    assert AnimeSeries.find(anime_series(:kamichu).id).pending_changes?
    
    series = anime_series(:kamichu)
    series.delete_pending_change :years, :change, "2006"
    series.save
    
    put :approve, :id => anime_series(:kamichu).id
    assert_redirected_to edit_anime_series_path(anime_series(:kamichu).id)
    assert_not_nil flash[:notice]
    assert AnimeSeries.find(anime_series(:kamichu).id).approved?
  end
  
  # Approvers can approve.
  def test_approve_pending_change_normal_user
    login_as :quentin
    
    put :approve_pending_change, :id => anime_series(:kamichu).id, :change_action => 'change', :key => :name, :value => 'Kamichu!'
    assert_response 401
#    assert_redirected_to new_anime_series_path
  end
  
  def test_approve_pending_change_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_series_approver)
    user.save
    
    put :approve_pending_change, :id => anime_series(:kamichu).id, :change_action => 'change', :key => :name, :value => 'Kamichu!'
    assert_redirected_to edit_anime_series_path(anime_series(:kamichu).id)
    assert_not_nil flash[:notice]
    assert_equal 'Kamichur?', AnimeSeries.find(anime_series(:kamichu).id).name
    assert AnimeSeries.find(anime_series(:kamichu).id).pending_changes[:name]['Kamichu!'].include?(user.id)
  end
  
  def test_approve_pending_change_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_series_moderator)
    user.save
    
    put :approve_pending_change, :id => anime_series(:kamichu).id, :change_action => 'change', :key => :name, :value => 'Kamichu!'
    assert_redirected_to edit_anime_series_path(anime_series(:kamichu).id)
    assert_not_nil flash[:notice]
    assert_equal 'Kamichu!', AnimeSeries.find(anime_series(:kamichu).id).name
    assert !AnimeSeries.find(anime_series(:kamichu).id).pending_changes.include?(:name)
  end
  
  def test_approve_pending_change_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_super_moderator)
    user.save
    
    put :approve_pending_change, :id => anime_series(:kamichu).id, :change_action => 'change', :key => :name, :value => 'Kamichu!'
    assert_redirected_to edit_anime_series_path(anime_series(:kamichu).id)
    assert_not_nil flash[:notice]
    assert_equal 'Kamichu!', AnimeSeries.find(anime_series(:kamichu).id).name
    assert !AnimeSeries.find(anime_series(:kamichu).id).pending_changes.include?(:name)
  end
  
  # Approvers can reject.
  def test_reject_pending_change_normal_user
    login_as :quentin
    
    put :reject_pending_change, :id => anime_series(:kamichu).id, :change_action => 'change', :key => :years, :value => '2006'
    assert_response 401
#    assert_redirected_to new_anime_series_path
  end
  
  def test_reject_pending_change_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_series_approver)
    user.save
    
    put :reject_pending_change, :id => anime_series(:kamichu).id, :change_action => 'change', :key => :years, :value => '2006'
    assert_redirected_to edit_anime_series_path(anime_series(:kamichu).id)
    assert_not_nil flash[:notice]
    assert_equal '2005-2006', AnimeSeries.find(anime_series(:kamichu).id).years
    assert !AnimeSeries.find(anime_series(:kamichu).id).pending_changes[:years].include?('2006')
    
    put :reject_pending_change, :id => anime_series(:kamichu).id, :change_action => 'change', :key => :years, :value => '2005'
    assert_redirected_to edit_anime_series_path(anime_series(:kamichu).id)
    assert_not_nil flash[:notice]
    assert_equal '2005-2006', AnimeSeries.find(anime_series(:kamichu).id).years
    assert AnimeSeries.find(anime_series(:kamichu).id).pending_changes[:years].include?('2005')
  end
  
  def test_reject_pending_change_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_series_moderator)
    user.save
    
    put :reject_pending_change, :id => anime_series(:kamichu).id, :change_action => 'change', :key => :years, :value => '2005'
    assert_redirected_to edit_anime_series_path(anime_series(:kamichu).id)
    assert_not_nil flash[:notice]
    assert_equal '2005-2006', AnimeSeries.find(anime_series(:kamichu).id).years
    assert !AnimeSeries.find(anime_series(:kamichu).id).pending_changes[:years].include?('2005')
  end
  
  def test_reject_pending_change_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_super_moderator)
    user.save
    
    put :reject_pending_change, :id => anime_series(:kamichu).id, :change_action => 'change', :key => :years, :value => '2005'
    assert_redirected_to edit_anime_series_path(anime_series(:kamichu).id)
    assert_not_nil flash[:notice]
    assert_equal '2005-2006', AnimeSeries.find(anime_series(:kamichu).id).years
    assert !AnimeSeries.find(anime_series(:kamichu).id).pending_changes[:years].include?('2005')
  end
  
  # Only moderators can add an alternate name.
  def test_add_alternate_name_normal_user
    login_as :quentin
    
    # TODO: reimplement me
    assert_no_difference 'AlternateName.count' do
      post :add_alternate_name, :id => anime_series(:kamichu).id, :new_alternate_name => 'My Little Goddess'
      assert_response 401
    end
    
    series = AnimeSeries.find(anime_series(:kamichu).id)
    assert !series.alternate_names.include?('My Little Goddess')
  end
  
  def test_add_alternate_name_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_series_approver)
    user.save
    
    series = AnimeSeries.find(anime_series(:kamichu).id)
    assert series.pending_changes?
    
    assert_no_difference 'AlternateName.count' do
      put :add_alternate_name, :id => anime_series(:kamichu).id, :new_alternate_name => 'My Little Goddess'
      assert_redirected_to edit_anime_series_path(anime_series(:kamichu))
    end
    
    # However, approvers should have the name added to pending_changes.
    series = AnimeSeries.find(anime_series(:kamichu).id)
    assert !series.alternate_names.include?('My Little Goddess')
    assert series.pending_changes?
    assert_not_nil series.pending_changes
    assert_not_nil series.pending_changes[:alternate_names]
    assert_not_nil series.pending_changes[:alternate_names][:add]
    assert_not_nil series.pending_changes[:alternate_names][:add]['My Little Goddess']
    assert series.pending_changes[:alternate_names][:add]['My Little Goddess'].include?(users(:quentin).id)
  end
  
  def test_add_alternate_name_moderator
    login_as :quentin
    
    # And as for moderators, it should stick immediately.
    user = users(:quentin)
    user.roles << roles(:anime_series_moderator)
    user.save
    
    assert_difference 'AlternateName.count' do
      put :add_alternate_name, :id => anime_series(:kamichu).id, :new_alternate_name => 'My Little Goddess'
      assert_redirected_to edit_anime_series_path(anime_series(:kamichu))
    end
    
    series = AnimeSeries.find(anime_series(:kamichu).id)
    assert series.alternate_names.include?('My Little Goddess')
#    assert_nil series.pending_changes[:alternate_names][:add]['My Little Goddess']
  end
  
  # Only moderators can delete an alternate name.
  def test_remove_alternate_name_normal_user
    login_as :quentin
    
    # TODO: reimplement me
    assert_no_difference 'AlternateName.count' do
      put :remove_alternate_name, :id => anime_series(:kinos_journey).id, :name => 'Kino no Tabi ~the beautiful world~'
      assert_response 401
    end
    
    series = AnimeSeries.find(anime_series(:kinos_journey).id)
    assert series.alternate_names.include?('Kino no Tabi ~the beautiful world~')
  end
    
  def test_remove_alternate_name_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_series_approver)
    user.save
    
    series = AnimeSeries.find(anime_series(:kinos_journey).id)
    
    assert_no_difference 'AlternateName.count' do
      put :remove_alternate_name, :id => anime_series(:kinos_journey).id, :name => 'Kino no Tabi ~the beautiful world~'
      assert_redirected_to edit_anime_series_path(anime_series(:kinos_journey))
    end
    
    # However, approvers should have the name added to pending_changes.
    series = AnimeSeries.find(anime_series(:kinos_journey).id)
    assert series.alternate_names.include?('Kino no Tabi ~the beautiful world~')
    assert series.pending_changes?
    assert_not_nil series.pending_changes
    assert_not_nil series.pending_changes[:alternate_names]
    assert_not_nil series.pending_changes[:alternate_names][:remove]
    assert_not_nil series.pending_changes[:alternate_names][:remove]['Kino no Tabi ~the beautiful world~']
    assert series.pending_changes[:alternate_names][:remove]['Kino no Tabi ~the beautiful world~'].include?(users(:quentin).id)
  end
  
  def test_remove_nonexistent_alternate_name_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_series_approver)
    user.save
    
    # Deleting a non-existent name should give an error.
    series = AnimeSeries.find(anime_series(:kinos_journey).id)
    
    assert_no_difference 'AlternateName.count' do
      put :remove_alternate_name, :id => anime_series(:kinos_journey).id, :name => 'Kino no Tabi ~da beautiful world~'
      assert_redirected_to edit_anime_series_path(anime_series(:kinos_journey))
    end
    
    series = AnimeSeries.find(anime_series(:kinos_journey).id)
    assert !series.pending_changes?
    assert_nil series.pending_changes
  end
  
  def test_remove_alternate_name_moderator
    login_as :quentin
    
    # And as for moderators, it should stick immediately.
    user = users(:quentin)
    user.roles << roles(:anime_series_moderator)
    user.save
    
    assert_difference 'AlternateName.count', -1 do
      put :remove_alternate_name, :id => anime_series(:kinos_journey).id, :name => 'Kino no Tabi ~the beautiful world~'
      assert_redirected_to edit_anime_series_path(anime_series(:kinos_journey))
    end
    
    series = AnimeSeries.find(anime_series(:kinos_journey).id)
    assert !series.alternate_names.include?('Kino no Tabi ~the beautiful world~')
    assert !series.pending_changes?
    assert_nil series.pending_changes
    
    # Deleting a non-existent name should give an error.
    series = AnimeSeries.find(anime_series(:kinos_journey).id)
    
    assert_no_difference 'AlternateName.count' do
      put :remove_alternate_name, :id => anime_series(:kinos_journey).id, :name => 'Kino no Tabi ~da beautiful world~'
      assert_redirected_to edit_anime_series_path(anime_series(:kinos_journey))
    end
    
    series = AnimeSeries.find(anime_series(:kinos_journey).id)
    assert !series.pending_changes?
    assert_nil series.pending_changes
  end
  
  # Only moderators can add an anime genre.
  def test_add_anime_genre_normal_user
    login_as :quentin
    
    # TODO: reimplement me
    assert_no_difference 'AnimeSeries.find(anime_series(:kamichu).id).anime_genres.count' do
      post :add_anime_genre, :id => anime_series(:kamichu).id, :new_anime_genre => 5
      assert_response 401
    end
    
    series = AnimeSeries.find(anime_series(:kamichu).id)
    assert !series.anime_genres.include?(AnimeGenre.find(5))
  end
  
  def test_add_anime_genre_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_series_approver)
    user.save
    
    series = AnimeSeries.find(anime_series(:kamichu).id)
    assert series.pending_changes?
    
    assert_no_difference 'AnimeSeries.find(anime_series(:kamichu).id).anime_genres.count' do
      post :add_anime_genre, :id => anime_series(:kamichu).id, :new_anime_genre => 5
      assert_redirected_to edit_anime_series_path(anime_series(:kamichu))
    end
    
    # However, approvers should have the name added to pending_changes.
    series = AnimeSeries.find(anime_series(:kamichu).id)
    assert !series.anime_genres.include?(AnimeGenre.find(5))
    assert series.pending_changes?
    assert_not_nil series.pending_changes
    assert_not_nil series.pending_changes[:anime_genres]
    assert_not_nil series.pending_changes[:anime_genres][:add]
    assert_not_nil series.pending_changes[:anime_genres][:add]['5']
    assert series.pending_changes[:anime_genres][:add]['5'].include?(users(:quentin).id)
  end
  
  def test_add_anime_genre_moderator
    login_as :quentin
    
    # And as for moderators, it should stick immediately.
    user = users(:quentin)
    user.roles << roles(:anime_series_moderator)
    user.save
    
    assert_difference 'AnimeSeries.find(anime_series(:kamichu).id).anime_genres.count' do
      post :add_anime_genre, :id => anime_series(:kamichu).id, :new_anime_genre => 5
      assert_redirected_to edit_anime_series_path(anime_series(:kamichu))
    end
    
    series = AnimeSeries.find(anime_series(:kamichu).id)
    assert series.anime_genres.include?(AnimeGenre.find(5))
#    assert_nil series.pending_changes[:anime_genres][:add]['My Little Goddess']
  end
  
  # Only moderators can delete an anime genre.
  def test_remove_anime_genre_normal_user
    login_as :quentin
    
    # TODO: reimplement me
    assert_no_difference 'AnimeSeries.find(anime_series(:kinos_journey).id).anime_genres.count' do
      put :remove_anime_genre, :id => anime_series(:kinos_journey).id, :genre_id => 3
      assert_response 401
    end
    
    series = AnimeSeries.find(anime_series(:kinos_journey).id)
    assert series.anime_genres.include?(AnimeGenre.find(3))
  end
    
  def test_remove_anime_genre_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_series_approver)
    user.save
    
    series = AnimeSeries.find(anime_series(:kinos_journey).id)
    
    assert_no_difference 'AnimeSeries.find(anime_series(:kinos_journey).id).anime_genres.count' do
      put :remove_anime_genre, :id => anime_series(:kinos_journey).id, :genre_id => 3
      assert_redirected_to edit_anime_series_path(anime_series(:kinos_journey))
    end
    
    # However, approvers should have the name added to pending_changes.
    series = AnimeSeries.find(anime_series(:kinos_journey).id)
    assert series.anime_genres.include?(AnimeGenre.find(3))
    assert series.pending_changes?
    assert_not_nil series.pending_changes
    assert_not_nil series.pending_changes[:anime_genres]
    assert_not_nil series.pending_changes[:anime_genres][:remove]
    assert_not_nil series.pending_changes[:anime_genres][:remove]['3']
    assert series.pending_changes[:anime_genres][:remove]['3'].include?(users(:quentin).id)
  end
  
  def test_remove_nonexistent_anime_genre_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_series_approver)
    user.save
    
    # Deleting a non-existent name should give an error.
    series = AnimeSeries.find(anime_series(:kinos_journey).id)
    
    assert_no_difference 'AnimeSeries.find(anime_series(:kinos_journey).id).anime_genres.count' do
      put :remove_anime_genre, :id => anime_series(:kinos_journey).id, :genre_id => 5
      assert_redirected_to edit_anime_series_path(anime_series(:kinos_journey))
    end
    
    series = AnimeSeries.find(anime_series(:kinos_journey).id)
    assert !series.pending_changes?
    assert_nil series.pending_changes
  end
  
  def test_remove_anime_genre_moderator
    login_as :quentin
    
    # And as for moderators, it should stick immediately.
    user = users(:quentin)
    user.roles << roles(:anime_series_moderator)
    user.save
    
    assert_difference 'AnimeSeries.find(anime_series(:kinos_journey).id).anime_genres.count', -1 do
      put :remove_anime_genre, :id => anime_series(:kinos_journey).id, :genre_id => 3
      assert_redirected_to edit_anime_series_path(anime_series(:kinos_journey))
    end
    
    series = AnimeSeries.find(anime_series(:kinos_journey).id)
    assert !series.anime_genres.include?(AnimeGenre.find(3))
    assert !series.pending_changes?
    assert_nil series.pending_changes
    
    # Deleting a non-existent name should give an error.
    series = AnimeSeries.find(anime_series(:kinos_journey).id)
    
    assert_no_difference 'AnimeSeries.find(anime_series(:kinos_journey).id).anime_genres.count' do
      put :remove_anime_genre, :id => anime_series(:kinos_journey).id, :genre_id => 5
      assert_redirected_to edit_anime_series_path(anime_series(:kinos_journey))
    end
    
    series = AnimeSeries.find(anime_series(:kinos_journey).id)
    assert !series.pending_changes?
    assert_nil series.pending_changes
  end
  
  # Moderators can destroy.
  def test_destroy_normal_user
    login_as :quentin
    
    delete :destroy, :id => anime_series(:kamichu).id
    assert_response 401
    assert_nothing_raised { AnimeSeries.find(anime_series(:kamichu).id) }
  end
  
  def test_destroy_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_series_approver)
    user.save
    
    delete :destroy, :id => anime_series(:kamichu).id
    assert_response 401
    assert_nothing_raised { AnimeSeries.find(anime_series(:kamichu).id) }
  end
  
  def test_destroy_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_series_moderator)
    user.save
    
    delete :destroy, :id => anime_series(:kamichu).id
    assert_redirected_to manage_anime_series_path
    assert_raise ActiveRecord::RecordNotFound do; AnimeSeries.find(anime_series(:kamichu).id); end
  end
  
  def test_destroy_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_super_moderator)
    user.save
    
    delete :destroy, :id => anime_series(:kinos_journey).id
    assert_redirected_to manage_anime_series_path
    assert_raise ActiveRecord::RecordNotFound do; AnimeSeries.find(anime_series(:kinos_journey).id); end
  end
  
  # Approvers can manage.
  def test_manage_normal_user
    login_as :quentin
    
    get :manage
    assert_response 401
  end
  
  def test_manage_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_series_approver)
    user.save
    
    get :manage
    assert_response :success
    assert_template 'manage'
  end
  
  def test_manage_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_series_moderator)
    user.save
    
    get :manage
    assert_response :success
    assert_template 'manage'
  end
  
  def test_manage_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_super_moderator)
    user.save
    
    get :manage
    assert_response :success
    assert_template 'manage'
  end

  def test_select_merge_normal_user
    login_as :quentin
    
    get :select_merge, :id => anime_series(:little_goddess).id
    assert_response 401
  end
  
  def test_select_merge_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_series_approver)
    user.save
    
    get :select_merge, :id => anime_series(:little_goddess).id
    assert_response 401
  end
  
  def test_select_merge_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_series_moderator)
    user.save
    
    get :select_merge, :id => anime_series(:little_goddess).id
    assert_response :success
    assert_template 'select_merge'
    
    # And it should have @anime_series set to the requested series.
    assert_equal anime_series(:little_goddess), assigns(:anime_series)
  end
  
  def test_select_merge_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_super_moderator)
    user.save
    
    get :select_merge, :id => anime_series(:little_goddess).id
    assert_response :success
    assert_template 'select_merge'
    
    # And it should have @anime_series set to the requested series.
    assert_equal anime_series(:little_goddess), assigns(:anime_series)
  end
  
  # TODO: search_select_merge
  
  def test_merge_into_normal_user
    login_as :quentin
    
    put :merge_into, :id => anime_series(:little_goddess).id, :dest => anime_series(:kamichu).id
    assert_response 401
    assert_nothing_raised { AnimeSeries.find(anime_series(:little_goddess).id) }
    assert !AnimeSeries.find(anime_series(:kamichu).id).alternate_names.include?('My Little Goddess')
  end
  
  def test_merge_into_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_series_approver)
    user.save
    
    put :merge_into, :id => anime_series(:little_goddess).id, :dest => anime_series(:kamichu).id
    assert_response 401
    assert_nothing_raised { AnimeSeries.find(anime_series(:little_goddess).id) }
    assert !AnimeSeries.find(anime_series(:kamichu).id).alternate_names.include?('My Little Goddess')
  end
  
  def test_merge_into_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_series_moderator)
    user.save
    
    put :merge_into, :id => anime_series(:little_goddess).id, :dest => anime_series(:kamichu).id
    assert_redirected_to manage_anime_series_path
	
    assert_raise ActiveRecord::RecordNotFound do; AnimeSeries.find(anime_series(:little_goddess).id); end
    assert AnimeSeries.find(anime_series(:kamichu).id).alternate_names.include?('My Little Goddess')
  end
  
  def test_merge_into_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_super_moderator)
    user.save
    
    put :merge_into, :id => anime_series(:little_goddess).id, :dest => anime_series(:kamichu).id
    assert_redirected_to manage_anime_series_path
	
    assert_raise ActiveRecord::RecordNotFound do; AnimeSeries.find(anime_series(:little_goddess).id); end
    assert AnimeSeries.find(anime_series(:kamichu).id).alternate_names.include?('My Little Goddess')
  end

protected
  def create_series(options = {})
    { :name => 'The Melancholy of Haruhi Suzumiya', :years => '2006', :alternate_names => {} }.merge(options)
  end
end
