require File.dirname(__FILE__) + '/../test_helper'
require 'anime_trivia_question_controller'

# Reraise errors caught by the controller.
class AnimeTriviaQuestionController; def rescue_action(e) raise e end; end

class AnimeTriviaQuestionControllerTest < ActionController::TestCase
  # Anyone can see the index.
  def test_index
    login_as :quentin
    
    get :index
    assert_response :success
    assert_template 'new'
  end
  
  # Anyone can see the list.
  def test_list
    login_as :quentin
    
    get :list
    assert_response :success
    assert_template 'list'
  end
  
  # Anyone can see show.
  def test_show
    login_as :quentin
    
    get :show, :id => anime_trivia_questions(:yamato).id
    assert_response :success
    assert_template 'show'
    
    # And it should have @anime_trivia_question set to the requested trivia question.
    assert_equal anime_trivia_questions(:yamato), assigns(:anime_trivia_question)
  end
  
  # Anyone can see new.
  def test_new
    login_as :quentin
    
    get :new
    assert_response :success
    assert_template 'new'
    
    # And it should have @anime_trivia_question set to a new trivia question.
    #assert_equal AnimeTriviaQuestion.new(), assigns(:anime_trivia_question)
  end
  
  # Anyone can create.
  def test_create_normal_user
    login_as :quentin
    
    # Set @anime_trivia_question before POSTing create.
    assert_difference 'AnimeTriviaQuestion.count' do
      post :create, :anime_trivia_question => create_trivia_question, :anime_series => {:name => 'Kamichur?'}, :genre_checkbox => {'comedy' => '1'}
      assert_redirected_to new_anime_trivia_question_path
    end
    
    # It should have gone successfully.
    assert_not_nil flash[:notice]
    assert_nil flash[:error]
    assert_nil flash[:anime_trivia_question]
    assert_nil flash[:anime_series]
  end
  
  def test_bad_create_normal_user
    login_as :quentin
    
    # Constrast when we have a bad POST
    assert_no_difference 'AnimeTriviaQuestion.count' do
      post :create, :anime_trivia_question => create_trivia_question(:answer => ''), :anime_series => {:name => 'Kamichur?'}, :genre_checkbox => {'comedy' => '1'}
      assert_redirected_to new_anime_trivia_question_path
    end
    
    # It should NOT have gone successfully.
    assert_nil flash[:notice]
    assert_not_nil flash[:error]
    assert flash[:error_is_html]
    assert_not_nil flash[:anime_trivia_question]
    assert_not_nil flash[:anime_series]
  end
  
  def test_create_approver_user
    login_as :quentin
    
    # Oh right.  Approvers and above should have their changes auto-approve.
    user = users(:quentin)
    user.roles << roles(:anime_trivia_approver)
    user.save
    
    assert_difference 'AnimeTriviaQuestion.count' do
      post :create, :anime_trivia_question => create_trivia_question, :anime_series => {:name => 'Kamichur?'}, :genre_checkbox => {'comedy' => '1'}
      assert_redirected_to new_anime_trivia_question_path
    end
    
    # Should get one if we wrote the test right.
    anime_trivia_question = AnimeTriviaQuestion.find_by_question('Is this a test?')
    assert_not_nil anime_trivia_question
    
    assert anime_trivia_question.approved?
    assert_equal users(:quentin), anime_trivia_question.creator
    assert_equal users(:quentin), anime_trivia_question.approver
  end
  
  # TODO: test_create_with_continuum
  
  # Approvers and above can see edit.
  def test_edit_normal_user
    login_as :quentin
    
    get :edit, :id => anime_trivia_questions(:yamato).id
    assert_response 401
  end
  
  def test_edit_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_trivia_approver)
    user.save
    
    get :edit, :id => anime_trivia_questions(:yamato).id
    assert_response :success
    assert_template 'edit'
    assert_equal anime_trivia_questions(:yamato), assigns(:anime_trivia_question)
  end
  
  def test_edit_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_trivia_moderator)
    user.save
    
    get :edit, :id => anime_trivia_questions(:yamato).id
    assert_response :success
    assert_template 'edit'
    assert_equal anime_trivia_questions(:yamato), assigns(:anime_trivia_question)
  end
  
  def test_edit_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_super_moderator)
    user.save
    
    get :edit, :id => anime_trivia_questions(:yamato).id
    assert_response :success
    assert_template 'edit'
    assert_equal anime_trivia_questions(:yamato), assigns(:anime_trivia_question)
  end
  
  # Approvers can approve.
  def test_approve_normal_user
    login_as :quentin
    
    put :approve, :id => anime_trivia_questions(:foundations).id
    assert_response 401
#    assert_redirected_to new_anime_trivia_questions_path
  end
  
  def test_approve_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_trivia_approver)
    user.save
    
    put :approve, :id => anime_trivia_questions(:foundations).id
    assert_redirected_to edit_anime_trivia_question_path(anime_trivia_questions(:foundations).id)
    assert_not_nil flash[:notice]
    assert AnimeTriviaQuestion.find(anime_trivia_questions(:foundations).id).approved?
  end
  
  def test_approve_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_trivia_moderator)
    user.save
    
    put :approve, :id => anime_trivia_questions(:foundations).id
    assert_redirected_to edit_anime_trivia_question_path(anime_trivia_questions(:foundations).id)
    assert_not_nil flash[:notice]
    assert AnimeTriviaQuestion.find(anime_trivia_questions(:foundations).id).approved?
  end
  
  def test_approve_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_super_moderator)
    user.save
    
    put :approve, :id => anime_trivia_questions(:foundations).id
    assert_redirected_to edit_anime_trivia_question_path(anime_trivia_questions(:foundations).id)
    assert_not_nil flash[:notice]
    assert AnimeTriviaQuestion.find(anime_trivia_questions(:foundations).id).approved?
    
    # Okay, but Yamato SHOULDN'T approve until it's clean.
    put :approve, :id => anime_trivia_questions(:yamato).id
    assert_redirected_to edit_anime_trivia_question_path(anime_trivia_questions(:yamato).id)
    assert_not_nil flash[:error]
    assert AnimeTriviaQuestion.find(anime_trivia_questions(:yamato).id).pending_changes?
    
    trivia_question = anime_trivia_questions(:yamato)
    trivia_question.delete_pending_change :answer, :change, "Space Battleship Yamato"
    trivia_question.save
    
    put :approve, :id => anime_trivia_questions(:yamato).id
    assert_redirected_to edit_anime_trivia_question_path(anime_trivia_questions(:yamato).id)
    assert_not_nil flash[:notice]
    assert AnimeTriviaQuestion.find(anime_trivia_questions(:yamato).id).approved?
  end

  # Approvers can approve.
  def test_approve_pending_change_normal_user
    login_as :quentin
    
    put :approve_pending_change, :id => anime_trivia_questions(:yamato).id, :change_action => 'change', :key => :answer, :value => 'Battleship Yamato'
    assert_response 401
#    assert_redirected_to new_anime_trivia_question_path
  end
  
  def test_approve_pending_change_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_trivia_approver)
    user.save
    
    put :approve_pending_change, :id => anime_trivia_questions(:yamato).id, :change_action => 'change', :key => :answer, :value => 'Battleship Yamato'
    assert_redirected_to edit_anime_trivia_question_path(anime_trivia_questions(:yamato).id)
    assert_not_nil flash[:notice]
    assert_equal 'Yamato', AnimeTriviaQuestion.find(anime_trivia_questions(:yamato).id).answer
    assert AnimeTriviaQuestion.find(anime_trivia_questions(:yamato).id).pending_changes[:answer]['Battleship Yamato'].include?(user.id)
  end
  
  def test_approve_pending_change_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_trivia_moderator)
    user.save
    
    put :approve_pending_change, :id => anime_trivia_questions(:yamato).id, :change_action => 'change', :key => :answer, :value => 'Battleship Yamato'
    assert_redirected_to edit_anime_trivia_question_path(anime_trivia_questions(:yamato).id)
    assert_not_nil flash[:notice]
    assert_equal 'Battleship Yamato', AnimeTriviaQuestion.find(anime_trivia_questions(:yamato).id).answer
    assert !AnimeTriviaQuestion.find(anime_trivia_questions(:yamato).id).pending_changes[:answer].include?('Battleship Yamato')
  end
  
  def test_approve_pending_change_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_super_moderator)
    user.save
    
    put :approve_pending_change, :id => anime_trivia_questions(:yamato).id, :change_action => 'change', :key => :answer, :value => 'Battleship Yamato'
    assert_redirected_to edit_anime_trivia_question_path(anime_trivia_questions(:yamato).id)
    assert_not_nil flash[:notice]
    assert_equal 'Battleship Yamato', AnimeTriviaQuestion.find(anime_trivia_questions(:yamato).id).answer
    assert !AnimeTriviaQuestion.find(anime_trivia_questions(:yamato).id).pending_changes[:answer].include?('Battleship Yamato')
  end
  
  # Approvers can reject.
  def test_reject_pending_change_normal_user
    login_as :quentin
    
    put :reject_pending_change, :id => anime_trivia_questions(:yamato).id, :change_action => 'change', :key => :answer, :value => 'Space Battleship Yamato'
    assert_response 401
#    assert_redirected_to new_anime_trivia_questions_path
  end
  
  def test_reject_pending_change_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_trivia_approver)
    user.save
    
    put :reject_pending_change, :id => anime_trivia_questions(:yamato).id, :change_action => 'change', :key => :answer, :value => 'Space Battleship Yamato'
    assert_redirected_to edit_anime_trivia_question_path(anime_trivia_questions(:yamato).id)
    assert_not_nil flash[:notice]
    assert_equal 'Yamato', AnimeTriviaQuestion.find(anime_trivia_questions(:yamato).id).answer
    assert AnimeTriviaQuestion.find(anime_trivia_questions(:yamato).id).pending_changes[:answer].include?('Space Battleship Yamato')

    put :reject_pending_change, :id => anime_trivia_questions(:yamato).id, :change_action => 'change', :key => :answer, :value => 'Battleship Yamato'
    assert_redirected_to edit_anime_trivia_question_path(anime_trivia_questions(:yamato).id)
    assert_not_nil flash[:notice]
    assert_equal 'Yamato', AnimeTriviaQuestion.find(anime_trivia_questions(:yamato).id).answer
    assert !AnimeTriviaQuestion.find(anime_trivia_questions(:yamato).id).pending_changes[:answer]['Battleship Yamato'].include?(users(:quentin).id)
  end
  
  def test_reject_pending_change_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_trivia_moderator)
    user.save
    
    put :reject_pending_change, :id => anime_trivia_questions(:yamato).id, :change_action => 'change', :key => :answer, :value => 'Space Battleship Yamato'
    assert_redirected_to edit_anime_trivia_question_path(anime_trivia_questions(:yamato).id)
    assert_not_nil flash[:notice]
    assert_equal 'Yamato', AnimeTriviaQuestion.find(anime_trivia_questions(:yamato).id).answer
    assert !AnimeTriviaQuestion.find(anime_trivia_questions(:yamato).id).pending_changes[:answer].include?('Space Battleship Yamato')
  end
  
  def test_reject_pending_change_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_super_moderator)
    user.save
    
    put :reject_pending_change, :id => anime_trivia_questions(:yamato).id, :change_action => 'change', :key => :answer, :value => 'Space Battleship Yamato'
    assert_redirected_to edit_anime_trivia_question_path(anime_trivia_questions(:yamato).id)
    assert_not_nil flash[:notice]
    assert_equal 'Yamato', AnimeTriviaQuestion.find(anime_trivia_questions(:yamato).id).answer
    assert !AnimeTriviaQuestion.find(anime_trivia_questions(:yamato).id).pending_changes[:answer].include?('Space Battleship Yamato')
  end
  
  # Moderators can destroy.
  def test_destroy_normal_user
    login_as :quentin
    
    delete :destroy, :id => anime_trivia_questions(:yamato).id
    assert_response 401
    assert_nothing_raised { AnimeTriviaQuestion.find(anime_trivia_questions(:yamato).id) }
  end
  
  def test_destroy_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_trivia_approver)
    user.save
    
    delete :destroy, :id => anime_trivia_questions(:yamato).id
    assert_response 401
    assert_nothing_raised { AnimeTriviaQuestion.find(anime_trivia_questions(:yamato).id) }
  end
  
  def test_destroy_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_trivia_moderator)
    user.save
    
    delete :destroy, :id => anime_trivia_questions(:yamato).id
    assert_redirected_to manage_anime_trivia_question_path
    assert_raise ActiveRecord::RecordNotFound do; AnimeTriviaQuestion.find(anime_trivia_questions(:yamato).id); end
  end
  
  def test_destroy_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_super_moderator)
    user.save
    
    delete :destroy, :id => anime_trivia_questions(:yamato).id
    assert_redirected_to manage_anime_trivia_question_path
    assert_raise ActiveRecord::RecordNotFound do; AnimeTriviaQuestion.find(anime_trivia_questions(:yamato).id); end
  end
  
  # Approvers can manage.
  def test_manage_normal_user
    login_as :quentin
    
    get :manage
    assert_response 401
  end
  
  def test_manage_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_trivia_approver)
    user.save
    
    get :manage
    assert_response :success
    assert_template 'manage'
  end
  
  def test_manage_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_trivia_moderator)
    user.save
    
    get :manage
    assert_response :success
    assert_template 'manage'
  end
  
  def test_manage_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:anime_super_moderator)
    user.save
    
    get :manage
    assert_response :success
    assert_template 'manage'
  end

protected
  def create_trivia_question(options = {})
    { :question => 'Is this a test?', :answer => 'Yes.', :points => 1 }.merge(options)
  end
end
