require File.dirname(__FILE__) + '/../test_helper'
require 'roles_controller'

# Reraise errors caught by the controller.
class RolesController; def rescue_action(e) raise e end; end

class RolesControllerTest < ActionController::TestCase
  fixtures :roles
  fixtures :users
  fixtures :delegators_roles
  fixtures :roles_users
  fixtures :roles_encompassed_roles
  
  def test_index
    login_as :quentin
    
    get :index
    assert_response :success
    assert_template 'list'
  end
  
  def test_list
    login_as :quentin
    
    get :list
    
    assert_response :success
    assert_template 'list'
    
    assert_not_nil assigns(:roles)
  end
  
  def test_show
    login_as :quentin
    
    get :show, :id => 1
    
    assert_response :success
    assert_template 'show'
    
    assert_not_nil assigns(:role)
    assert assigns(:role).valid?
  end
  
  def test_non_delegator_cannot_add_user
    login_as :quentin
    
    post :add_user, :user => {:login => users(:quentin).login}, :role => roles(:admin_role).id
    
    assert_response 401
    
    assert !users(:quentin).roles.include?(roles(:admin_role))
  end
  
  def test_delegator_add_user
    login_as :quentin
    
    post :add_user, :user => {:login => users(:admin).login}, :role => roles(:user_role).id
    
    assert_redirected_to show_role_path(roles(:user_role).id)
    
    assert_not_nil flash[:notice]
    assert users(:admin).roles.include?(roles(:user_role))
  end
  
  def test_non_delegator_cannot_remove_user
    login_as :quentin
    
    post :remove_user, :user => users(:admin).id, :role => roles(:admin_role).id
    
    assert_response 401
    
    assert users(:admin).roles.include?(roles(:admin_role))
  end
  
  def test_delegator_remove_user
    login_as :quentin
    
    users(:admin).roles << roles(:user_role)
    users(:admin).save
    post :remove_user, :user => users(:admin).id, :role => roles(:user_role).id
    
    assert_redirected_to show_role_path(roles(:user_role).id)
    
    assert_not_nil flash[:notice]
    assert !users(:admin).roles.include?(roles(:user_role))
  end
  
  def test_delegator_remove_user_removes_delegation
    login_as :quentin
    
    users(:admin).delegator_roles << roles(:user_role)
    users(:admin).save
    post :remove_user, :user => users(:admin).id, :role => roles(:user_role).id
    
    assert_redirected_to show_role_path(roles(:user_role).id)
    
    assert_not_nil flash[:notice]
    assert !users(:admin).delegator_roles.include?(roles(:user_role))
  end
  
  def test_delegator_remove_user_cannot_remove_admin_from_self
    login_as :admin
    
    post :remove_user, :user => users(:admin).id, :role => roles(:admin_role).id
    
    assert_redirected_to show_role_path(roles(:admin_role).id)
    
    assert_not_nil flash[:error]
    assert users(:admin).roles.include?(roles(:admin_role))
  end
  
  def test_delegator_cannot_add_non_user_role
    login_as :quentin
    
    post :add_user, :user => {:login => 'aoeuidhtns'}, :role => roles(:user_role).id
    
    assert_redirected_to show_role_path(roles(:user_role).id)
    
    assert_not_nil flash[:error]
  end
  
  def test_non_delegator_cannot_add_delegator
    login_as :quentin
    
    post :add_delegator, :user => {:login => users(:quentin).login}, :role => roles(:admin_role).id
    
    assert_response 401
    
    assert !users(:quentin).delegator_roles.include?(roles(:admin_role))
  end
  
  def test_delegator_add_delegator_adds_role
    login_as :quentin
    
    post :add_delegator, :user => {:login => users(:admin).login}, :role => roles(:user_role).id
    
    assert_redirected_to show_role_path(roles(:user_role).id)
    
    assert_not_nil flash[:notice]
    assert users(:admin).roles.include?(roles(:user_role))
    assert users(:admin).delegator_roles.include?(roles(:user_role))
  end
  
  def test_non_delegator_cannot_remove_delegator
    login_as :quentin
    
    post :remove_delegator, :user => users(:admin), :role => roles(:admin_role).id
    
    assert_response 401
    
    assert users(:admin).delegator_roles.include?(roles(:admin_role))
  end
  
  def test_delegator_remove_delegator
    login_as :quentin
    
    users(:admin).delegator_roles << roles(:user_role)
    users(:admin).save
    post :remove_delegator, :user => users(:admin), :role => roles(:user_role).id
    
    assert_redirected_to show_role_path(roles(:user_role).id)
    
    assert_not_nil flash[:notice]
    assert !users(:admin).delegator_roles.include?(roles(:user_role))
  end
  
  def test_delegator_cannot_add_non_user_as_delegator
    login_as :quentin
    
    post :add_delegator, :user => {:login => 'aoeuidhtns'}, :role => roles(:user_role).id
    
    assert_redirected_to show_role_path(roles(:user_role).id)
    
    assert_not_nil flash[:error]
  end
end
