require File.dirname(__FILE__) + '/../test_helper'
require 'video_game_continuum_controller'

# Reraise errors caught by the controller.
class VideoGameContinuumController; def rescue_action(e) raise e end; end

class VideoGameContinuumControllerTest < ActionController::TestCase
  # Anyone can see the index.
  def test_index
    login_as :quentin
    
    get :index
    assert_response :success
    assert_template 'list'
  end
  
  # Anyone can see the list.
  def test_list
    login_as :quentin
    
    get :list
    assert_response :success
    assert_template 'list'
  end
  
  # Anyone can see show.
  def test_show
    login_as :quentin
    
    get :show, :id => video_game_continuums(:dragon_quest).id
    assert_response :success
    assert_template 'show'
    
    # And it should have @video_game_continuum set to the requested continuum.
    assert_equal video_game_continuums(:dragon_quest), assigns(:video_game_continuum)
  end
  
  # Anyone can see new.
  def test_new
    login_as :quentin
    
    get :new
    assert_response :success
    assert_template 'new'
    
    # And it should have @video_game_continuum set to a new continuum.
    #assert_equal VideoGameContinuum.new(), assigns(:video_game_continuum)
  end
  
  # Anyone can create.
  def test_create_normal_user
    login_as :quentin
    
    # Set @video_game_continuum before POSTing create.
    assert_difference 'VideoGameContinuum.count' do
      post :create, :video_game_continuum => create_continuum
      assert_redirected_to new_video_game_continuum_path
    end
    
    # It should have gone successfully.
    assert_not_nil flash[:notice]
    assert_nil flash[:error]
    assert_nil flash[:video_game_continuum]
    assert_nil flash[:submission_note]
  end
  
  def test_bad_create_normal_user
    login_as :quentin
    
    # Constrast when we have a bad POST
    assert_no_difference 'VideoGameContinuum.count' do
      post :create, :video_game_continuum => create_continuum(:name => video_game_continuums(:dragon_quest).name)
      assert_redirected_to new_video_game_continuum_path
    end
    
    # It should NOT have gone successfully.
    assert_nil flash[:notice]
    assert_not_nil flash[:error]
    assert flash[:error_is_html]
    assert_not_nil flash[:video_game_continuum]
    assert_nil flash[:submission_note]
  end
  
  def test_complex_create_normal_user
    login_as :quentin
    
    # And now a complex success test with comments and alternate names.
    assert_difference 'VideoGameContinuum.count' do
      assert_difference 'AlternateName.count', 3 do
        assert_difference 'Comment.count' do
          post :create, :video_game_continuum => create_continuum(:name => '1234', :alternate_names => {1 => 'a', 2 => 'a', 3 => '', 4 => '', 5 => nil, 6 => 'b', 7 => 'c'}), :submission_note => 'A submission note.'
          assert_redirected_to new_video_game_continuum_path
        end
      end
    end
    
    # It should have gone successfully.
    assert_not_nil flash[:notice]
    assert_nil flash[:error]
    assert_nil flash[:video_game_continuum]
    assert_nil flash[:submission_note]
    
    # Should get one if we wrote the test right.
    video_game_continuum = VideoGameContinuum.find_by_name('1234')
    assert_not_nil video_game_continuum
    
    # Main users should not auto-approve...
    assert video_game_continuum.pending_approval?
    assert_equal users(:quentin), video_game_continuum.creator
    
    # It should have eliminated nils, empties, and duplicates for the alternate_names.
    assert_equal 3, video_game_continuum.alternate_names.count
    assert video_game_continuum.alternate_names.to_ary.include?('a')
    assert video_game_continuum.alternate_names.to_ary.include?('b')
    assert video_game_continuum.alternate_names.to_ary.include?('c')
    
    # And we should find the comment attached.
    assert_equal 1, video_game_continuum.comments.count
    assert_equal 'A submission note.', video_game_continuum.comments[0].text
    assert_equal :submission_note, video_game_continuum.comments[0].comment_type
  end
  
  def test_complex_bad_create_normal_user
    login_as :quentin
    
    # And lastly a complex failure test with comments and alternate names.
    assert_no_difference 'VideoGameContinuum.count' do
      assert_no_difference 'AlternateName.count' do
        assert_no_difference 'Comment.count' do
          post :create, :video_game_continuum => create_continuum(:name => nil, :alternate_names => {1 => 'a', 2 => 'a', 3 => '', 4 => '', 5 => nil, 6 => 'b', 7 => 'c'}), :submission_note => 'A submission note.'
          assert_redirected_to new_video_game_continuum_path
        end
      end
    end
    
    # It should have gone successfully.
    assert_nil flash[:notice]
    assert_not_nil flash[:error]
    assert flash[:error_is_html]
    assert_not_nil flash[:video_game_continuum]
    assert_not_nil flash[:submission_note]
  end
  
  def test_create_approver_user
    login_as :quentin
    
    # Oh right.  Approvers and above should have their changes auto-approve.
    user = users(:quentin)
    user.roles << roles(:video_game_continuum_approver)
    user.save
    
    assert_difference 'VideoGameContinuum.count' do
      post :create, :video_game_continuum => create_continuum(:name => 'Touhou')
      assert_redirected_to new_video_game_continuum_path
    end
    
    # Should get one if we wrote the test right.
    video_game_continuum = VideoGameContinuum.find_by_name('Touhou')
    assert_not_nil video_game_continuum
    
    assert video_game_continuum.approved?
    assert_equal users(:quentin), video_game_continuum.creator
    assert_equal users(:quentin), video_game_continuum.approver
  end
  
  # Approvers and above can see edit.
  def test_edit_normal_user
    login_as :quentin
    
    get :edit, :id => video_game_continuums(:dragon_quest).id
    assert_response 401
  end
  
  def test_edit_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_continuum_approver)
    user.save
    
    get :edit, :id => video_game_continuums(:dragon_quest).id
    assert_response :success
    assert_template 'edit'
    assert_equal video_game_continuums(:dragon_quest), assigns(:video_game_continuum)
  end
  
  def test_edit_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_continuum_moderator)
    user.save
    
    get :edit, :id => video_game_continuums(:dragon_quest).id
    assert_response :success
    assert_template 'edit'
    assert_equal video_game_continuums(:dragon_quest), assigns(:video_game_continuum)
  end
  
  def test_edit_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_super_moderator)
    user.save
    
    get :edit, :id => video_game_continuums(:dragon_quest).id
    assert_response :success
    assert_template 'edit'
    assert_equal video_game_continuums(:dragon_quest), assigns(:video_game_continuum)
  end
  
  # Approvers can approve.
  def test_approve_normal_user
    login_as :quentin
    
    put :approve, :id => video_game_continuums(:sonic).id
    assert_response 401
#    assert_redirected_to new_video_game_continuum_path
  end
  
  def test_approve_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_continuum_approver)
    user.save
    
    put :approve, :id => video_game_continuums(:sonic).id
    assert_redirected_to edit_video_game_continuum_path(video_game_continuums(:sonic).id)
    assert_not_nil flash[:notice]
    assert VideoGameContinuum.find(video_game_continuums(:sonic).id).approved?
  end
  
  def test_approve_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_continuum_moderator)
    user.save
    
    put :approve, :id => video_game_continuums(:sonic).id
    assert_redirected_to edit_video_game_continuum_path(video_game_continuums(:sonic).id)
    assert_not_nil flash[:notice]
    assert VideoGameContinuum.find(video_game_continuums(:sonic).id).approved?
  end
  
  def test_approve_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_super_moderator)
    user.save
    
    put :approve, :id => video_game_continuums(:sonic).id
    assert_redirected_to edit_video_game_continuum_path(video_game_continuums(:sonic).id)
    assert_not_nil flash[:notice]
    assert VideoGameContinuum.find(video_game_continuums(:sonic).id).approved?
    
    # Okay, but Dragon Quest SHOULDN'T approve until it's clean.
    put :approve, :id => video_game_continuums(:dragon_quest).id
    assert_redirected_to edit_video_game_continuum_path(video_game_continuums(:dragon_quest).id)
    assert_not_nil flash[:error]
    assert VideoGameContinuum.find(video_game_continuums(:dragon_quest).id).pending_changes?
    
    continuum = video_game_continuums(:dragon_quest)
    continuum.delete_pending_change :name, :change, "Dragon Warrior"
    continuum.save
    
    put :approve, :id => video_game_continuums(:dragon_quest).id
    assert_redirected_to edit_video_game_continuum_path(video_game_continuums(:dragon_quest).id)
    assert_not_nil flash[:notice]
    assert VideoGameContinuum.find(video_game_continuums(:dragon_quest).id).approved?
  end
  
  # Approvers can approve.
  def test_approve_pending_change_normal_user
    login_as :quentin
    
    put :approve_pending_change, :id => video_game_continuums(:dragon_quest).id, :change_action => 'change', :key => :name, :value => 'Dragon Quest/Dragon Warrior'
    assert_response 401
#    assert_redirected_to new_video_game_continuum_path
  end
  
  def test_approve_pending_change_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_continuum_approver)
    user.save
    
    put :approve_pending_change, :id => video_game_continuums(:dragon_quest).id, :change_action => 'change', :key => :name, :value => 'Dragon Quest/Dragon Warrior'
    assert_redirected_to edit_video_game_continuum_path(video_game_continuums(:dragon_quest).id)
    assert_not_nil flash[:notice]
    assert_equal 'Dragon Quest', VideoGameContinuum.find(video_game_continuums(:dragon_quest).id).name
    assert VideoGameContinuum.find(video_game_continuums(:dragon_quest).id).pending_changes[:name]['Dragon Quest/Dragon Warrior'].include?(user.id)
  end
  
  def test_approve_pending_change_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_continuum_moderator)
    user.save
    
    put :approve_pending_change, :id => video_game_continuums(:dragon_quest).id, :change_action => 'change', :key => :name, :value => 'Dragon Quest/Dragon Warrior'
    assert_redirected_to edit_video_game_continuum_path(video_game_continuums(:dragon_quest).id)
    assert_not_nil flash[:notice]
    assert_equal 'Dragon Quest/Dragon Warrior', VideoGameContinuum.find(video_game_continuums(:dragon_quest).id).name
    assert !VideoGameContinuum.find(video_game_continuums(:dragon_quest).id).pending_changes[:name].include?('Dragon Quest/Dragon Warrior')
  end
  
  def test_approve_pending_change_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_super_moderator)
    user.save
    
    put :approve_pending_change, :id => video_game_continuums(:dragon_quest).id, :change_action => 'change', :key => :name, :value => 'Dragon Quest/Dragon Warrior'
    assert_redirected_to edit_video_game_continuum_path(video_game_continuums(:dragon_quest).id)
    assert_not_nil flash[:notice]
    assert_equal 'Dragon Quest/Dragon Warrior', VideoGameContinuum.find(video_game_continuums(:dragon_quest).id).name
    assert !VideoGameContinuum.find(video_game_continuums(:dragon_quest).id).pending_changes[:name].include?('Dragon Quest/Dragon Warrior')
  end
  
  # Approvers can reject.
  def test_reject_pending_change_normal_user
    login_as :quentin
    
    put :reject_pending_change, :id => video_game_continuums(:dragon_quest).id, :change_action => 'change', :key => :name, :value => 'Dragon Quest/Dragon Warrior'
    assert_response 401
#    assert_redirected_to new_video_game_continuum_path
  end
  
  def test_reject_pending_change_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_continuum_approver)
    user.save
    
    put :reject_pending_change, :id => video_game_continuums(:dragon_quest).id, :change_action => 'change', :key => :name, :value => 'Dragon Warrior'
    assert_redirected_to edit_video_game_continuum_path(video_game_continuums(:dragon_quest).id)
    assert_not_nil flash[:notice]
    assert_equal 'Dragon Quest', VideoGameContinuum.find(video_game_continuums(:dragon_quest).id).name
    assert !VideoGameContinuum.find(video_game_continuums(:dragon_quest).id).pending_changes[:name].include?('Dragon Warrior')
    
    put :reject_pending_change, :id => video_game_continuums(:dragon_quest).id, :change_action => 'change', :key => :name, :value => 'Dragon Warrior'
    assert_redirected_to edit_video_game_continuum_path(video_game_continuums(:dragon_quest).id)
    assert_not_nil flash[:notice]
    assert_equal 'Dragon Quest', VideoGameContinuum.find(video_game_continuums(:dragon_quest).id).name
    assert !VideoGameContinuum.find(video_game_continuums(:dragon_quest).id).pending_changes[:name].include?('Dragon Warrior')
  end
  
  def test_reject_pending_change_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_continuum_moderator)
    user.save
    
    put :reject_pending_change, :id => video_game_continuums(:dragon_quest).id, :change_action => 'change', :key => :name, :value => 'Dragon Warrior'
    assert_redirected_to edit_video_game_continuum_path(video_game_continuums(:dragon_quest).id)
    assert_not_nil flash[:notice]
    assert_equal 'Dragon Quest', VideoGameContinuum.find(video_game_continuums(:dragon_quest).id).name
    assert !VideoGameContinuum.find(video_game_continuums(:dragon_quest).id).pending_changes[:name].include?('Dragon Warrior')
  end
  
  def test_reject_pending_change_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_super_moderator)
    user.save
    
    put :reject_pending_change, :id => video_game_continuums(:dragon_quest).id, :change_action => 'change', :key => :name, :value => 'Dragon Warrior'
    assert_redirected_to edit_video_game_continuum_path(video_game_continuums(:dragon_quest).id)
    assert_not_nil flash[:notice]
    assert_equal 'Dragon Quest', VideoGameContinuum.find(video_game_continuums(:dragon_quest).id).name
    assert !VideoGameContinuum.find(video_game_continuums(:dragon_quest).id).pending_changes[:name].include?('Dragon Warrior')
  end
  
  # Only moderators can add an alternate name.
  def test_add_alternate_name_normal_user
    login_as :quentin
    
    # TODO: reimplement me
    assert_no_difference 'AlternateName.count' do
      post :add_alternate_name, :id => video_game_continuums(:dragon_quest).id, :new_alternate_name => 'Quest for Dragon'
      assert_response 401
    end
    
    continuum = VideoGameContinuum.find(video_game_continuums(:dragon_quest).id)
    assert !continuum.alternate_names.include?('Quest for Dragon')
  end
  
  def test_add_alternate_name_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_continuum_approver)
    user.save
    
    continuum = VideoGameContinuum.find(video_game_continuums(:dragon_quest).id)
    assert continuum.pending_changes?
    
    assert_no_difference 'AlternateName.count' do
      put :add_alternate_name, :id => video_game_continuums(:dragon_quest).id, :new_alternate_name => 'Quest for Dragon'
      assert_redirected_to edit_video_game_continuum_path(video_game_continuums(:dragon_quest))
    end
    
    # However, approvers should have the name added to pending_changes.
    continuum = VideoGameContinuum.find(video_game_continuums(:dragon_quest).id)
    assert !continuum.alternate_names.include?('Quest for Dragon')
    assert continuum.pending_changes?
    assert_not_nil continuum.pending_changes
    assert_not_nil continuum.pending_changes[:alternate_names]
    assert_not_nil continuum.pending_changes[:alternate_names][:add]
    assert_not_nil continuum.pending_changes[:alternate_names][:add]['Quest for Dragon']
    assert continuum.pending_changes[:alternate_names][:add]['Quest for Dragon'].include?(users(:quentin).id)
  end
  
  def test_add_alternate_name_moderator
    login_as :quentin
    
    # And as for moderators, it should stick immediately.
    user = users(:quentin)
    user.roles << roles(:video_game_continuum_moderator)
    user.save
    
    assert_difference 'AlternateName.count' do
      put :add_alternate_name, :id => video_game_continuums(:dragon_quest).id, :new_alternate_name => 'Quest for Dragon'
      assert_redirected_to edit_video_game_continuum_path(video_game_continuums(:dragon_quest))
    end
    
    continuum = VideoGameContinuum.find(video_game_continuums(:dragon_quest).id)
    assert continuum.alternate_names.include?('Quest for Dragon')
#    assert_nil continuum.pending_changes[:alternate_names][:add]['My Little Goddess']
  end
  
  # Only moderators can delete an alternate name.
  def test_remove_alternate_name_normal_user
    login_as :quentin
    
    # TODO: reimplement me
    assert_no_difference 'AlternateName.count' do
      put :remove_alternate_name, :id => video_game_continuums(:sonic).id, :name => 'Sonic'
      assert_response 401
    end
    
    continuum = VideoGameContinuum.find(video_game_continuums(:sonic).id)
    assert continuum.alternate_names.include?('Sonic')
  end
    
  def test_remove_alternate_name_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_continuum_approver)
    user.save
    
    continuum = VideoGameContinuum.find(video_game_continuums(:sonic).id)
    
    assert_no_difference 'AlternateName.count' do
      put :remove_alternate_name, :id => video_game_continuums(:sonic).id, :name => 'Sonic'
      assert_redirected_to edit_video_game_continuum_path(video_game_continuums(:sonic))
    end
    
    # However, approvers should have the name added to pending_changes.
    continuum = VideoGameContinuum.find(video_game_continuums(:sonic).id)
    assert continuum.alternate_names.include?('Sonic')
    assert continuum.pending_approval?
    assert_not_nil continuum.pending_changes
    assert_not_nil continuum.pending_changes[:alternate_names]
    assert_not_nil continuum.pending_changes[:alternate_names][:remove]
    assert_not_nil continuum.pending_changes[:alternate_names][:remove]['Sonic']
    assert continuum.pending_changes[:alternate_names][:remove]['Sonic'].include?(users(:quentin).id)
  end
  
  def test_remove_nonexistent_alternate_name_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_continuum_approver)
    user.save
    
    # Deleting a non-existent name should give an error.
    continuum = VideoGameContinuum.find(video_game_continuums(:sonic).id)
    
    assert_no_difference 'AlternateName.count' do
      put :remove_alternate_name, :id => video_game_continuums(:sonic).id, :name => 'Sornic'
      assert_redirected_to edit_video_game_continuum_path(video_game_continuums(:sonic))
    end
    
    continuum = VideoGameContinuum.find(video_game_continuums(:sonic).id)
    assert !continuum.pending_changes?
    assert_nil continuum.pending_changes
  end
  
  def test_remove_alternate_name_moderator
    login_as :quentin
    
    # And as for moderators, it should stick immediately.
    user = users(:quentin)
    user.roles << roles(:video_game_continuum_moderator)
    user.save
    
    assert_difference 'AlternateName.count', -1 do
      put :remove_alternate_name, :id => video_game_continuums(:sonic).id, :name => 'Sonic'
      assert_redirected_to edit_video_game_continuum_path(video_game_continuums(:sonic))
    end
    
    continuum = VideoGameContinuum.find(video_game_continuums(:sonic).id)
    assert !continuum.alternate_names.include?('Sonic')
    assert !continuum.pending_changes?
    assert_nil continuum.pending_changes
    
    # Deleting a non-existent name should give an error.
    continuum = VideoGameContinuum.find(video_game_continuums(:sonic).id)
    
    assert_no_difference 'AlternateName.count' do
      put :remove_alternate_name, :id => video_game_continuums(:sonic).id, :name => 'Sornic'
      assert_redirected_to edit_video_game_continuum_path(video_game_continuums(:sonic))
    end
    
    continuum = VideoGameContinuum.find(video_game_continuums(:sonic).id)
    assert !continuum.pending_changes?
    assert_nil continuum.pending_changes
  end
  
  # Moderators can destroy.
  def test_destroy_normal_user
    login_as :quentin
    
    delete :destroy, :id => video_game_continuums(:dragon_quest).id
    assert_response 401
    assert_nothing_raised { VideoGameContinuum.find(video_game_continuums(:dragon_quest).id) }
  end
  
  def test_destroy_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_continuum_approver)
    user.save
    
    delete :destroy, :id => video_game_continuums(:dragon_quest).id
    assert_response 401
    assert_nothing_raised { VideoGameContinuum.find(video_game_continuums(:dragon_quest).id) }
  end
  
  def test_destroy_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_continuum_moderator)
    user.save
    
    delete :destroy, :id => video_game_continuums(:dragon_quest).id
    assert_redirected_to manage_video_game_continuum_path
    assert_raise ActiveRecord::RecordNotFound do; VideoGameContinuum.find(video_game_continuums(:dragon_quest).id); end
  end
  
  def test_destroy_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_super_moderator)
    user.save
    
    delete :destroy, :id => video_game_continuums(:sonic).id
    assert_redirected_to manage_video_game_continuum_path
    assert_raise ActiveRecord::RecordNotFound do; VideoGameContinuum.find(video_game_continuums(:sonic).id); end
  end
  
  # Approvers can manage.
  def test_manage_normal_user
    login_as :quentin
    
    get :manage
    assert_response 401
  end
  
  def test_manage_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_continuum_approver)
    user.save
    
    get :manage
    assert_response :success
    assert_template 'manage'
  end
  
  def test_manage_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_continuum_moderator)
    user.save
    
    get :manage
    assert_response :success
    assert_template 'manage'
  end
  
  def test_manage_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_super_moderator)
    user.save
    
    get :manage
    assert_response :success
    assert_template 'manage'
  end
  
  def test_select_merge_normal_user
    login_as :quentin
    
    get :select_merge, :id => video_game_continuums(:dragon_warrior).id
    assert_response 401
  end
  
  def test_select_merge_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_continuum_approver)
    user.save
    
    get :select_merge, :id => video_game_continuums(:dragon_warrior).id
    assert_response 401
  end
  
  def test_select_merge_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_continuum_moderator)
    user.save
    
    get :select_merge, :id => video_game_continuums(:dragon_warrior).id
    assert_response :success
    assert_template 'select_merge'
    
    # And it should have @video_game_continuum set to the requested continuum.
    assert_equal video_game_continuums(:dragon_warrior), assigns(:video_game_continuum)
  end
  
  def test_select_merge_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_super_moderator)
    user.save
    
    get :select_merge, :id => video_game_continuums(:dragon_warrior).id
    assert_response :success
    assert_template 'select_merge'
    
    # And it should have @video_game_continuum set to the requested continuum.
    assert_equal video_game_continuums(:dragon_warrior), assigns(:video_game_continuum)
  end
  
  # TODO: search_select_merge
  
  def test_merge_into_normal_user
    login_as :quentin
    
    put :merge_into, :id => video_game_continuums(:dragon_warrior).id, :dest => video_game_continuums(:dragon_quest).id
    assert_response 401
    assert_nothing_raised { VideoGameContinuum.find(video_game_continuums(:dragon_warrior).id) }
    assert !VideoGameContinuum.find(video_game_continuums(:dragon_quest).id).alternate_names.include?('Dragon Warrior')
  end
  
  def test_merge_into_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_continuum_approver)
    user.save
    
    put :merge_into, :id => video_game_continuums(:dragon_warrior).id, :dest => video_game_continuums(:dragon_quest).id
    assert_response 401
    assert_nothing_raised { VideoGameContinuum.find(video_game_continuums(:dragon_warrior).id) }
    assert !VideoGameContinuum.find(video_game_continuums(:dragon_quest).id).alternate_names.include?('Dragon Warrior')
  end
  
  def test_merge_into_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_continuum_moderator)
    user.save
    
    put :merge_into, :id => video_game_continuums(:dragon_warrior).id, :dest => video_game_continuums(:dragon_quest).id
    assert_redirected_to manage_video_game_continuum_path
	
    assert_raise ActiveRecord::RecordNotFound do; VideoGameContinuum.find(video_game_continuums(:dragon_warrior).id); end
    assert VideoGameContinuum.find(video_game_continuums(:dragon_quest).id).alternate_names.include?('Dragon Warrior')
  end
  
  def test_merge_into_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_super_moderator)
    user.save
    
    put :merge_into, :id => video_game_continuums(:dragon_warrior).id, :dest => video_game_continuums(:dragon_quest).id
    assert_redirected_to manage_video_game_continuum_path
	
    assert_raise ActiveRecord::RecordNotFound do; VideoGameContinuum.find(video_game_continuums(:dragon_warrior).id); end
    assert VideoGameContinuum.find(video_game_continuums(:dragon_quest).id).alternate_names.include?('Dragon Warrior')
  end

protected
  def create_continuum(options = {})
    { :name => 'Megaman', :alternate_names => {} }.merge(options)
  end
end
