require File.dirname(__FILE__) + '/../test_helper'
require 'video_game_genre_controller'

# Reraise errors caught by the controller.
class VideoGameGenreController; def rescue_action(e) raise e end; end

class VideoGameGenreControllerTest < ActionController::TestCase
  # Anyone can see the index.
  def test_index
    login_as :quentin
    
    get :index
    assert_response :success
    assert_template 'list'
  end
  
  # Anyone can see the list.
  def test_list
    login_as :quentin
    
    get :list
    assert_response :success
    assert_template 'list'
  end
  
  # Only moderators can see new.
  def test_new_normal_user
    login_as :quentin
    
    get :new
    assert_response 401
    
    # And it should have @video_game_genre set to a new genre.
    #assert_equal VideoGameGenre.new(), assigns(:video_game_genre)
  end
  
  def test_new_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_genre_moderator)
    user.save
    
    get :new
    assert_response :success
    assert_template 'new'
    
    # And it should have @video_game_genre set to a new genre.
    #assert_equal VideoGameGenre.new(), assigns(:video_game_genre)
  end
  
  def test_new_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_super_moderator)
    user.save
    
    get :new
    assert_response :success
    assert_template 'new'
    
    # And it should have @video_game_genre set to a new genre.
    #assert_equal VideoGameGenre.new(), assigns(:video_game_genre)
  end
  
  # Only mods can create.
  def test_create_normal_user
    login_as :quentin
    
    # Set @video_game_genre before POSTing create.
    assert_no_difference 'VideoGameGenre.count' do
      post :create, :video_game_genre => create_genre
      assert_response 401
    end
    
    # It should have not gone.
    assert_nil flash[:notice]
    assert_nil flash[:error]
    assert_nil flash[:video_game_genre]
  end
  
  def test_create_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_genre_moderator)
    user.save
    
    # Set @video_game_genre before POSTing create.
    assert_difference 'VideoGameGenre.count' do
      post :create, :video_game_genre => create_genre
      assert_redirected_to new_video_game_genre_path
    end
    
    # It should have gone.
    assert_not_nil flash[:notice]
    assert_nil flash[:error]
    assert_nil flash[:video_game_genre]
  end
  
  def test_bad_create_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_genre_moderator)
    user.save
    
    # Constrast when we have a bad POST
    assert_no_difference 'VideoGameGenre.count' do
      post :create, :video_game_genre => create_genre(:name => video_game_genres(:rpg).name)
      assert_redirected_to new_video_game_genre_path
    end
    
    # It should NOT have gone successfully.
    assert_nil flash[:notice]
    assert_not_nil flash[:error]
    assert flash[:error_is_html]
    assert_not_nil flash[:video_game_genre]
  end
  
  # Only moderators can see edit.
  def test_edit_normal_user
    login_as :quentin
    
    get :edit, :id => video_game_genres(:rpg).id
    assert_response 401
    
    #assert_equal VideoGameGenre.new(), assigns(:video_game_genre)
  end
  
  def test_edit_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_genre_moderator)
    user.save
    
    get :edit, :id => video_game_genres(:rpg).id
    assert_response :success
    assert_template 'edit'
    
    assert_equal video_game_genres(:rpg), assigns(:video_game_genre)
  end
  
  def test_edit_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_super_moderator)
    user.save
    
    get :edit, :id => video_game_genres(:rpg).id
    assert_response :success
    assert_template 'edit'
    
    assert_equal video_game_genres(:rpg), assigns(:video_game_genre)
  end
  
  # Moderators can destroy.
  def test_destroy_normal_user
    login_as :quentin
    
    delete :destroy, :id => video_game_genres(:rpg).id
    assert_response 401
    assert_nothing_raised { VideoGameGenre.find(video_game_genres(:rpg).id) }
  end
  
  def test_destroy_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_genre_moderator)
    user.save
    
    delete :destroy, :id => video_game_genres(:rpg).id
    assert_redirected_to manage_video_game_genre_path
    assert_raise ActiveRecord::RecordNotFound do; VideoGameGenre.find(video_game_genres(:rpg).id); end
  end
  
  def test_destroy_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_super_moderator)
    user.save
    
    delete :destroy, :id => video_game_genres(:rpg).id
    assert_redirected_to manage_video_game_genre_path
    assert_raise ActiveRecord::RecordNotFound do; VideoGameGenre.find(video_game_genres(:rpg).id); end
  end
  
  # Moderators can manage.
  def test_manage_normal_user
    login_as :quentin
    
    get :manage
    assert_response 401
  end
  
  def test_manage_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_genre_moderator)
    user.save
    
    get :manage
    assert_response :success
    assert_template 'manage'
  end
  
  def test_manage_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_super_moderator)
    user.save
    
    get :manage
    assert_response :success
    assert_template 'manage'
  end
  
  def test_select_merge_normal_user
    login_as :quentin
    
    get :select_merge, :id => video_game_genres(:role_playing).id
    assert_response 401
  end
  
  def test_select_merge_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_genre_moderator)
    user.save
    
    get :select_merge, :id => video_game_genres(:role_playing).id
    assert_response :success
    assert_template 'select_merge'
    
    # And it should have @video_game_genre set to the requested genre.
    assert_equal video_game_genres(:role_playing), assigns(:video_game_genre)
  end
  
  def test_select_merge_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_super_moderator)
    user.save
    
    get :select_merge, :id => video_game_genres(:role_playing).id
    assert_response :success
    assert_template 'select_merge'
    
    # And it should have @video_game_genre set to the requested genre.
    assert_equal video_game_genres(:role_playing), assigns(:video_game_genre)
  end
  
  # TODO: search_select_merge
  
  def test_merge_into_normal_user
    login_as :quentin
    
    put :merge_into, :id => video_game_genres(:role_playing).id, :dest => video_game_genres(:rpg).id
    assert_response 401
    assert_nothing_raised { VideoGameGenre.find(video_game_genres(:role_playing).id) }
  end
  
  def test_merge_into_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_genre_moderator)
    user.save
    
    put :merge_into, :id => video_game_genres(:role_playing).id, :dest => video_game_genres(:rpg).id
    assert_redirected_to manage_video_game_genre_path
	
    assert_raise ActiveRecord::RecordNotFound do; VideoGameGenre.find(video_game_genres(:role_playing).id); end
  end
  
  def test_merge_into_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_super_moderator)
    user.save
    
    put :merge_into, :id => video_game_genres(:role_playing).id, :dest => video_game_genres(:rpg).id
    assert_redirected_to manage_video_game_genre_path
	
    assert_raise ActiveRecord::RecordNotFound do; VideoGameGenre.find(video_game_genres(:role_playing).id); end
  end

protected
  def create_genre(options = {})
    { :name => 'Dramatis', :description => 'Something dramatic' }.merge(options)
  end
end
