require File.dirname(__FILE__) + '/../test_helper'
require 'video_game_scene_controller'

# Reraise errors caught by the controller.
class VideoGameSceneController; def rescue_action(e) raise e end; end

class VideoGameSceneControllerTest < ActionController::TestCase
  # Anyone can see the index.
  def test_index
    login_as :quentin
    
    get :index
    assert_response :success
    assert_template 'new'
  end
  
  # Anyone can see the list.
  def test_list
    login_as :quentin
    
    get :list
    assert_response :success
    assert_template 'list'
  end
  
  # Anyone can see show.
  def test_show
    login_as :quentin
    
    get :show, :id => video_game_scenes(:green_hill).id
    assert_response :success
    assert_template 'show'
    
    # And it should have @video_game_scene set to the requested trivia question.
    assert_equal video_game_scenes(:green_hill), assigns(:video_game_scene)
  end
  
  # Anyone can see new.
  def test_new
    login_as :quentin
    
    get :new
    assert_response :success
    assert_template 'new'
    
    # And it should have @video_game_scene set to a new trivia question.
    #assert_equal VideoGameScene.new(), assigns(:video_game_scene)
  end
  
  # TODO: Anyone can create.
#  def test_create_normal_user
#    login_as :quentin
#    
#    # Set @video_game_scene before POSTing create.
#    assert_difference 'VideoGameScene.count' do
#      post :create, :video_game_scene => create_scene, :video_game => {:name => 'Kamichur?'}, :genre_checkbox => {'comedy' => '1'}
#      assert_redirected_to new_video_game_scene_path
#    end
#    
#    # It should have gone successfully.
#    assert_not_nil flash[:notice]
#    assert_nil flash[:error]
#    assert_nil flash[:video_game_scene]
#    assert_nil flash[:video_game]
#  end
  
#  def test_bad_create_normal_user
#    login_as :quentin
#    
#    # Constrast when we have a bad POST
#    assert_no_difference 'VideoGameScene.count' do
#      post :create, :video_game_scene => create_scene(:image => nil), :video_game => {:name => 'Kamichur?'}, :genre_checkbox => {'comedy' => '1'}
#      assert_redirected_to new_video_game_scene_path
#    end
#    
#    # It should NOT have gone successfully.
#    assert_nil flash[:notice]
#    assert_not_nil flash[:error]
#    assert flash[:error_is_html]
#    assert_not_nil flash[:video_game_scene]
#    assert_not_nil flash[:video_game]
#  end
  
#  def test_create_approver_user
#    login_as :quentin
#    
#    # Oh right.  Approvers and above should have their changes auto-approve.
#    user = users(:quentin)
#    user.roles << roles(:video_game_scene_approver)
#    user.save
#    
#    assert_difference 'VideoGameScene.count' do
#      post :create, :video_game_scene => create_scene, :video_game => {:name => 'Kamichur?'}, :genre_checkbox => {'comedy' => '1'}
#      assert_redirected_to new_video_game_scene_path
#    end
#    
#    # TODO: Should get one if we wrote the test right.
#    video_game_scene = VideoGameScene.find_by_image_file_name('test.png')
#    assert_not_nil video_game_scene
    
#    assert video_game_scene.approved?
#    assert_equal users(:quentin), video_game_scene.creator
#    assert_equal users(:quentin), video_game_scene.approver
#  end
  
  # TODO: test_create_with_continuum
  
  # Approvers and above can see edit.
  def test_edit_normal_user
    login_as :quentin
    
    get :edit, :id => video_game_scenes(:green_hill).id
    assert_response 401
  end
  
  def test_edit_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_scene_approver)
    user.save
    
    get :edit, :id => video_game_scenes(:green_hill).id
    assert_response :success
    assert_template 'edit'
    assert_equal video_game_scenes(:green_hill), assigns(:video_game_scene)
  end
  
  def test_edit_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_scene_moderator)
    user.save
    
    get :edit, :id => video_game_scenes(:green_hill).id
    assert_response :success
    assert_template 'edit'
    assert_equal video_game_scenes(:green_hill), assigns(:video_game_scene)
  end
  
  def test_edit_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_super_moderator)
    user.save
    
    get :edit, :id => video_game_scenes(:green_hill).id
    assert_response :success
    assert_template 'edit'
    assert_equal video_game_scenes(:green_hill), assigns(:video_game_scene)
  end
  
  # Approvers can approve.
  def test_approve_normal_user
    login_as :quentin
    
    put :approve, :id => video_game_scenes(:okami).id
    assert_response 401
#    assert_redirected_to new_video_game_scenes_path
  end
  
  def test_approve_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_scene_approver)
    user.save
    
    put :approve, :id => video_game_scenes(:okami).id
    assert_redirected_to edit_video_game_scene_path(video_game_scenes(:okami).id)
    assert_not_nil flash[:notice]
    assert VideoGameScene.find(video_game_scenes(:okami).id).approved?
  end
  
  def test_approve_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_scene_moderator)
    user.save
    
    put :approve, :id => video_game_scenes(:okami).id
    assert_redirected_to edit_video_game_scene_path(video_game_scenes(:okami).id)
    assert_not_nil flash[:notice]
    assert VideoGameScene.find(video_game_scenes(:okami).id).approved?
  end
  
  def test_approve_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_super_moderator)
    user.save
    
    put :approve, :id => video_game_scenes(:okami).id
    assert_redirected_to edit_video_game_scene_path(video_game_scenes(:okami).id)
    assert_not_nil flash[:notice]
    assert VideoGameScene.find(video_game_scenes(:okami).id).approved?
    
    # Okay, but Yamato SHOULDN'T approve until it's clean.
    put :approve, :id => video_game_scenes(:green_hill).id
    assert_redirected_to edit_video_game_scene_path(video_game_scenes(:green_hill).id)
    assert_not_nil flash[:error]
    assert VideoGameScene.find(video_game_scenes(:green_hill).id).pending_changes?
    
    scene = video_game_scenes(:green_hill)
    scene.delete_pending_change :location, :change, "Green Hill"
    scene.save
    
    put :approve, :id => video_game_scenes(:green_hill).id
    assert_redirected_to edit_video_game_scene_path(video_game_scenes(:green_hill).id)
    assert_not_nil flash[:notice]
    assert VideoGameScene.find(video_game_scenes(:green_hill).id).approved?
  end

  # Approvers can approve.
  def test_approve_pending_change_normal_user
    login_as :quentin
    
    put :approve_pending_change, :id => video_game_scenes(:green_hill).id, :change_action => 'change', :key => :location, :value => 'Green Hill Zone'
    assert_response 401
#    assert_redirected_to new_video_game_scene_path
  end
  
  def test_approve_pending_change_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_scene_approver)
    user.save
    
    put :approve_pending_change, :id => video_game_scenes(:green_hill).id, :change_action => 'change', :key => :location, :value => 'Green Hill Zone'
    assert_redirected_to edit_video_game_scene_path(video_game_scenes(:green_hill).id)
    assert_not_nil flash[:notice]
    assert_equal 'Green Hill Zone, Act 2', VideoGameScene.find(video_game_scenes(:green_hill).id).location
    assert VideoGameScene.find(video_game_scenes(:green_hill).id).pending_changes[:location]['Green Hill Zone'].include?(user.id)
  end
  
  def test_approve_pending_change_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_scene_moderator)
    user.save
    
    put :approve_pending_change, :id => video_game_scenes(:green_hill).id, :change_action => 'change', :key => :location, :value => 'Green Hill Zone'
    assert_redirected_to edit_video_game_scene_path(video_game_scenes(:green_hill).id)
    assert_not_nil flash[:notice]
    assert_equal 'Green Hill Zone', VideoGameScene.find(video_game_scenes(:green_hill).id).location
    assert !VideoGameScene.find(video_game_scenes(:green_hill).id).pending_changes[:location].include?('Green Hill Zone')
  end
  
  def test_approve_pending_change_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_super_moderator)
    user.save
    
    put :approve_pending_change, :id => video_game_scenes(:green_hill).id, :change_action => 'change', :key => :location, :value => 'Green Hill Zone'
    assert_redirected_to edit_video_game_scene_path(video_game_scenes(:green_hill).id)
    assert_not_nil flash[:notice]
    assert_equal 'Green Hill Zone', VideoGameScene.find(video_game_scenes(:green_hill).id).location
    assert !VideoGameScene.find(video_game_scenes(:green_hill).id).pending_changes[:location].include?('Green Hill Zone')
  end
  
  # Approvers can reject.
  def test_reject_pending_change_normal_user
    login_as :quentin
    
    put :reject_pending_change, :id => video_game_scenes(:green_hill).id, :change_action => 'change', :key => :location, :value => 'Green Hill'
    assert_response 401
#    assert_redirected_to new_video_game_scenes_path
  end
  
  def test_reject_pending_change_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_scene_approver)
    user.save
    
    put :reject_pending_change, :id => video_game_scenes(:green_hill).id, :change_action => 'change', :key => :location, :value => 'Green Hill'
    assert_redirected_to edit_video_game_scene_path(video_game_scenes(:green_hill).id)
    assert_not_nil flash[:notice]
    assert_equal 'Green Hill Zone, Act 2', VideoGameScene.find(video_game_scenes(:green_hill).id).location
    assert VideoGameScene.find(video_game_scenes(:green_hill).id).pending_changes[:location].include?('Green Hill')

    put :reject_pending_change, :id => video_game_scenes(:green_hill).id, :change_action => 'change', :key => :location, :value => 'Green Hill Zone'
    assert_redirected_to edit_video_game_scene_path(video_game_scenes(:green_hill).id)
    assert_not_nil flash[:notice]
    assert_equal 'Green Hill Zone, Act 2', VideoGameScene.find(video_game_scenes(:green_hill).id).location
    assert !VideoGameScene.find(video_game_scenes(:green_hill).id).pending_changes[:location]['Green Hill'].include?(users(:quentin).id)
  end
  
  def test_reject_pending_change_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_scene_moderator)
    user.save
    
    put :reject_pending_change, :id => video_game_scenes(:green_hill).id, :change_action => 'change', :key => :location, :value => 'Green Hill'
    assert_redirected_to edit_video_game_scene_path(video_game_scenes(:green_hill).id)
    assert_not_nil flash[:notice]
    assert_equal 'Green Hill Zone, Act 2', VideoGameScene.find(video_game_scenes(:green_hill).id).location
    assert !VideoGameScene.find(video_game_scenes(:green_hill).id).pending_changes[:location].include?('Green Hill')
  end
  
  def test_reject_pending_change_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_super_moderator)
    user.save
    
    put :reject_pending_change, :id => video_game_scenes(:green_hill).id, :change_action => 'change', :key => :location, :value => 'Green Hill'
    assert_redirected_to edit_video_game_scene_path(video_game_scenes(:green_hill).id)
    assert_not_nil flash[:notice]
    assert_equal 'Green Hill Zone, Act 2', VideoGameScene.find(video_game_scenes(:green_hill).id).location
    assert !VideoGameScene.find(video_game_scenes(:green_hill).id).pending_changes[:location].include?('Green Hill')
  end
  
  # Moderators can destroy.
  def test_destroy_normal_user
    login_as :quentin
    
    delete :destroy, :id => video_game_scenes(:green_hill).id
    assert_response 401
    assert_nothing_raised { VideoGameScene.find(video_game_scenes(:green_hill).id) }
  end
  
  def test_destroy_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_scene_approver)
    user.save
    
    delete :destroy, :id => video_game_scenes(:green_hill).id
    assert_response 401
    assert_nothing_raised { VideoGameScene.find(video_game_scenes(:green_hill).id) }
  end
  
  def test_destroy_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_scene_moderator)
    user.save
    
    delete :destroy, :id => video_game_scenes(:green_hill).id
    assert_redirected_to manage_video_game_scene_path
    assert_raise ActiveRecord::RecordNotFound do; VideoGameScene.find(video_game_scenes(:green_hill).id); end
  end
  
  def test_destroy_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_super_moderator)
    user.save
    
    delete :destroy, :id => video_game_scenes(:green_hill).id
    assert_redirected_to manage_video_game_scene_path
    assert_raise ActiveRecord::RecordNotFound do; VideoGameScene.find(video_game_scenes(:green_hill).id); end
  end
  
  # Approvers can manage.
  def test_manage_normal_user
    login_as :quentin
    
    get :manage
    assert_response 401
  end
  
  def test_manage_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_scene_approver)
    user.save
    
    get :manage
    assert_response :success
    assert_template 'manage'
  end
  
  def test_manage_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_scene_moderator)
    user.save
    
    get :manage
    assert_response :success
    assert_template 'manage'
  end
  
  def test_manage_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_super_moderator)
    user.save
    
    get :manage
    assert_response :success
    assert_template 'manage'
  end

protected
  def create_scene(options = {})
    { :image => nil, :location => 'Yes.' }.merge(options)
  end
end
