require File.dirname(__FILE__) + '/../test_helper'
require 'video_game_trivia_question_controller'

# Reraise errors caught by the controller.
class VideoGameTriviaQuestionController; def rescue_action(e) raise e end; end

class VideoGameTriviaQuestionControllerTest < ActionController::TestCase
  # Anyone can see the index.
  def test_index
    login_as :quentin
    
    get :index
    assert_response :success
    assert_template 'new'
  end
  
  # Anyone can see the list.
  def test_list
    login_as :quentin
    
    get :list
    assert_response :success
    assert_template 'list'
  end
  
  # Anyone can see show.
  def test_show
    login_as :quentin
    
    get :show, :id => video_game_trivia_questions(:kaguya).id
    assert_response :success
    assert_template 'show'
    
    # And it should have @video_game_trivia_question set to the requested trivia question.
    assert_equal video_game_trivia_questions(:kaguya), assigns(:video_game_trivia_question)
  end
  
  # Anyone can see new.
  def test_new
    login_as :quentin
    
    get :new
    assert_response :success
    assert_template 'new'
    
    # And it should have @video_game_trivia_question set to a new trivia question.
    #assert_equal VideoGameTriviaQuestion.new(), assigns(:video_game_trivia_question)
  end
  
  # Anyone can create.
  def test_create_normal_user
    login_as :quentin
    
    # Set @video_game_trivia_question before POSTing create.
    assert_difference 'VideoGameTriviaQuestion.count' do
      post :create, :video_game_trivia_question => create_trivia_question, :video_game => {:name => 'Sornic the hedghog?'}, :genre_checkbox => {'rpg' => '1'}, :video_game_system => {:name => 'Genesis'}
      assert_redirected_to new_video_game_trivia_question_path
    end
    
    # It should have gone successfully.
    assert_not_nil flash[:notice]
    assert_nil flash[:error]
    assert_nil flash[:video_game_trivia_question]
    assert_nil flash[:video_game]
  end
  
  def test_bad_create_normal_user
    login_as :quentin
    
    # Constrast when we have a bad POST
    assert_no_difference 'VideoGameTriviaQuestion.count' do
      post :create, :video_game_trivia_question => create_trivia_question(:answer => ''), :video_game => {:name => 'Sornic the hedgehog?'}, :genre_checkbox => {'rpg' => '1'}, :video_game_system => {:name => 'Genesis'}
      assert_redirected_to new_video_game_trivia_question_path
    end
    
    # It should NOT have gone successfully.
    assert_nil flash[:notice]
    assert_not_nil flash[:error]
    assert flash[:error_is_html]
    assert_not_nil flash[:video_game_trivia_question]
    assert_not_nil flash[:video_game]
  end
  
  def test_create_approver_user
    login_as :quentin
    
    # Oh right.  Approvers and above should have their changes auto-approve.
    user = users(:quentin)
    user.roles << roles(:video_game_trivia_approver)
    user.save
    
    assert_difference 'VideoGameTriviaQuestion.count' do
      post :create, :video_game_trivia_question => create_trivia_question, :video_game => {:name => 'Sornic the hedgehog?'}, :genre_checkbox => {'rpg' => '1'}, :video_game_system => {:name => 'Genesis'}
      assert_redirected_to new_video_game_trivia_question_path
    end
    
    # Should get one if we wrote the test right.
    video_game_trivia_question = VideoGameTriviaQuestion.find_by_question('Is this a test?')
    assert_not_nil video_game_trivia_question
    
    assert video_game_trivia_question.approved?
    assert_equal users(:quentin), video_game_trivia_question.creator
    assert_equal users(:quentin), video_game_trivia_question.approver
  end
  
  # TODO: test_create_with_continuum
  
  # Approvers and above can see edit.
  def test_edit_normal_user
    login_as :quentin
    
    get :edit, :id => video_game_trivia_questions(:kaguya).id
    assert_response 401
  end
  
  def test_edit_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_trivia_approver)
    user.save
    
    get :edit, :id => video_game_trivia_questions(:kaguya).id
    assert_response :success
    assert_template 'edit'
    assert_equal video_game_trivia_questions(:kaguya), assigns(:video_game_trivia_question)
  end
  
  def test_edit_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_trivia_moderator)
    user.save
    
    get :edit, :id => video_game_trivia_questions(:kaguya).id
    assert_response :success
    assert_template 'edit'
    assert_equal video_game_trivia_questions(:kaguya), assigns(:video_game_trivia_question)
  end
  
  def test_edit_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_super_moderator)
    user.save
    
    get :edit, :id => video_game_trivia_questions(:kaguya).id
    assert_response :success
    assert_template 'edit'
    assert_equal video_game_trivia_questions(:kaguya), assigns(:video_game_trivia_question)
  end
  
  # Approvers can approve.
  def test_approve_normal_user
    login_as :quentin
    
    put :approve, :id => video_game_trivia_questions(:worlds).id
    assert_response 401
#    assert_redirected_to new_video_game_trivia_questions_path
  end
  
  def test_approve_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_trivia_approver)
    user.save
    
    put :approve, :id => video_game_trivia_questions(:worlds).id
    assert_redirected_to edit_video_game_trivia_question_path(video_game_trivia_questions(:worlds).id)
    assert_not_nil flash[:notice]
    assert VideoGameTriviaQuestion.find(video_game_trivia_questions(:worlds).id).approved?
  end
  
  def test_approve_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_trivia_moderator)
    user.save
    
    put :approve, :id => video_game_trivia_questions(:worlds).id
    assert_redirected_to edit_video_game_trivia_question_path(video_game_trivia_questions(:worlds).id)
    assert_not_nil flash[:notice]
    assert VideoGameTriviaQuestion.find(video_game_trivia_questions(:worlds).id).approved?
  end
  
  def test_approve_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_super_moderator)
    user.save
    
    put :approve, :id => video_game_trivia_questions(:worlds).id
    assert_redirected_to edit_video_game_trivia_question_path(video_game_trivia_questions(:worlds).id)
    assert_not_nil flash[:notice]
    assert VideoGameTriviaQuestion.find(video_game_trivia_questions(:worlds).id).approved?
    
    # Okay, but Kaguya SHOULDN'T approve until it's clean.
    put :approve, :id => video_game_trivia_questions(:kaguya).id
    assert_redirected_to edit_video_game_trivia_question_path(video_game_trivia_questions(:kaguya).id)
    assert_not_nil flash[:error]
    assert VideoGameTriviaQuestion.find(video_game_trivia_questions(:kaguya).id).pending_changes?
    
    trivia_question = video_game_trivia_questions(:kaguya)
    trivia_question.delete_pending_change :answer, :change, "Moon Princess Kaguya"
    trivia_question.save
    
    put :approve, :id => video_game_trivia_questions(:kaguya).id
    assert_redirected_to edit_video_game_trivia_question_path(video_game_trivia_questions(:kaguya).id)
    assert_not_nil flash[:notice]
    assert VideoGameTriviaQuestion.find(video_game_trivia_questions(:kaguya).id).approved?
  end

  # Approvers can approve.
  def test_approve_pending_change_normal_user
    login_as :quentin
    
    put :approve_pending_change, :id => video_game_trivia_questions(:kaguya).id, :change_action => 'change', :key => :answer, :value => 'Princess Kaguya'
    assert_response 401
#    assert_redirected_to new_video_game_trivia_question_path
  end
  
  def test_approve_pending_change_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_trivia_approver)
    user.save
    
    put :approve_pending_change, :id => video_game_trivia_questions(:kaguya).id, :change_action => 'change', :key => :answer, :value => 'Princess Kaguya'
    assert_redirected_to edit_video_game_trivia_question_path(video_game_trivia_questions(:kaguya).id)
    assert_not_nil flash[:notice]
    assert_equal 'Kaguya', VideoGameTriviaQuestion.find(video_game_trivia_questions(:kaguya).id).answer
    assert VideoGameTriviaQuestion.find(video_game_trivia_questions(:kaguya).id).pending_changes[:answer]['Princess Kaguya'].include?(user.id)
  end
  
  def test_approve_pending_change_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_trivia_moderator)
    user.save
    
    put :approve_pending_change, :id => video_game_trivia_questions(:kaguya).id, :change_action => 'change', :key => :answer, :value => 'Princess Kaguya'
    assert_redirected_to edit_video_game_trivia_question_path(video_game_trivia_questions(:kaguya).id)
    assert_not_nil flash[:notice]
    assert_equal 'Princess Kaguya', VideoGameTriviaQuestion.find(video_game_trivia_questions(:kaguya).id).answer
    assert !VideoGameTriviaQuestion.find(video_game_trivia_questions(:kaguya).id).pending_changes[:answer].include?('Princess Kaguya')
  end
  
  def test_approve_pending_change_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_super_moderator)
    user.save
    
    put :approve_pending_change, :id => video_game_trivia_questions(:kaguya).id, :change_action => 'change', :key => :answer, :value => 'Princess Kaguya'
    assert_redirected_to edit_video_game_trivia_question_path(video_game_trivia_questions(:kaguya).id)
    assert_not_nil flash[:notice]
    assert_equal 'Princess Kaguya', VideoGameTriviaQuestion.find(video_game_trivia_questions(:kaguya).id).answer
    assert !VideoGameTriviaQuestion.find(video_game_trivia_questions(:kaguya).id).pending_changes[:answer].include?('Princess Kaguya')
  end
  
  # Approvers can reject.
  def test_reject_pending_change_normal_user
    login_as :quentin
    
    put :reject_pending_change, :id => video_game_trivia_questions(:kaguya).id, :change_action => 'change', :key => :answer, :value => 'Moon Princess Kaguya'
    assert_response 401
#    assert_redirected_to new_video_game_trivia_questions_path
  end
  
  def test_reject_pending_change_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_trivia_approver)
    user.save
    
    put :reject_pending_change, :id => video_game_trivia_questions(:kaguya).id, :change_action => 'change', :key => :answer, :value => 'Moon Princess Kaguya'
    assert_redirected_to edit_video_game_trivia_question_path(video_game_trivia_questions(:kaguya).id)
    assert_not_nil flash[:notice]
    assert_equal 'Kaguya', VideoGameTriviaQuestion.find(video_game_trivia_questions(:kaguya).id).answer
    assert VideoGameTriviaQuestion.find(video_game_trivia_questions(:kaguya).id).pending_changes[:answer].include?('Moon Princess Kaguya')

    put :reject_pending_change, :id => video_game_trivia_questions(:kaguya).id, :change_action => 'change', :key => :answer, :value => 'Princess Kaguya'
    assert_redirected_to edit_video_game_trivia_question_path(video_game_trivia_questions(:kaguya).id)
    assert_not_nil flash[:notice]
    assert_equal 'Kaguya', VideoGameTriviaQuestion.find(video_game_trivia_questions(:kaguya).id).answer
    assert !VideoGameTriviaQuestion.find(video_game_trivia_questions(:kaguya).id).pending_changes[:answer]['Princess Kaguya'].include?(users(:quentin).id)
  end
  
  def test_reject_pending_change_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_trivia_moderator)
    user.save
    
    put :reject_pending_change, :id => video_game_trivia_questions(:kaguya).id, :change_action => 'change', :key => :answer, :value => 'Moon Princess Kaguya'
    assert_redirected_to edit_video_game_trivia_question_path(video_game_trivia_questions(:kaguya).id)
    assert_not_nil flash[:notice]
    assert_equal 'Kaguya', VideoGameTriviaQuestion.find(video_game_trivia_questions(:kaguya).id).answer
    assert !VideoGameTriviaQuestion.find(video_game_trivia_questions(:kaguya).id).pending_changes[:answer].include?('Moon Princess Kaguya')
  end
  
  def test_reject_pending_change_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_super_moderator)
    user.save
    
    put :reject_pending_change, :id => video_game_trivia_questions(:kaguya).id, :change_action => 'change', :key => :answer, :value => 'Moon Princess Kaguya'
    assert_redirected_to edit_video_game_trivia_question_path(video_game_trivia_questions(:kaguya).id)
    assert_not_nil flash[:notice]
    assert_equal 'Kaguya', VideoGameTriviaQuestion.find(video_game_trivia_questions(:kaguya).id).answer
    assert !VideoGameTriviaQuestion.find(video_game_trivia_questions(:kaguya).id).pending_changes[:answer].include?('Moon Princess Kaguya')
  end
  
  # Moderators can destroy.
  def test_destroy_normal_user
    login_as :quentin
    
    delete :destroy, :id => video_game_trivia_questions(:kaguya).id
    assert_response 401
    assert_nothing_raised { VideoGameTriviaQuestion.find(video_game_trivia_questions(:kaguya).id) }
  end
  
  def test_destroy_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_trivia_approver)
    user.save
    
    delete :destroy, :id => video_game_trivia_questions(:kaguya).id
    assert_response 401
    assert_nothing_raised { VideoGameTriviaQuestion.find(video_game_trivia_questions(:kaguya).id) }
  end
  
  def test_destroy_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_trivia_moderator)
    user.save
    
    delete :destroy, :id => video_game_trivia_questions(:kaguya).id
    assert_redirected_to manage_video_game_trivia_question_path
    assert_raise ActiveRecord::RecordNotFound do; VideoGameTriviaQuestion.find(video_game_trivia_questions(:kaguya).id); end
  end
  
  def test_destroy_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_super_moderator)
    user.save
    
    delete :destroy, :id => video_game_trivia_questions(:kaguya).id
    assert_redirected_to manage_video_game_trivia_question_path
    assert_raise ActiveRecord::RecordNotFound do; VideoGameTriviaQuestion.find(video_game_trivia_questions(:kaguya).id); end
  end
  
  # Approvers can manage.
  def test_manage_normal_user
    login_as :quentin
    
    get :manage
    assert_response 401
  end
  
  def test_manage_approver
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_trivia_approver)
    user.save
    
    get :manage
    assert_response :success
    assert_template 'manage'
  end
  
  def test_manage_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_trivia_moderator)
    user.save
    
    get :manage
    assert_response :success
    assert_template 'manage'
  end
  
  def test_manage_super_moderator
    login_as :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_super_moderator)
    user.save
    
    get :manage
    assert_response :success
    assert_template 'manage'
  end

protected
  def create_trivia_question(options = {})
    { :question => 'Is this a test?', :answer => 'Yes.', :points => 1 }.merge(options)
  end
end
