require File.dirname(__FILE__) + '/../test_helper'

class AnimeGenreTest < ActiveSupport::TestCase
  Error = ''
  fixtures :anime_series
  fixtures :anime_genres
  fixtures :users
  
  def test_validations
    assert !create_genre(:name => nil).valid?
    assert !create_genre(:name => '').valid?
    
    assert_valid create_genre()
    assert_nil create_genre().description
    assert_valid create_genre(:description => 'An anime featuring gothic elements.')
    assert_equal 'An anime featuring gothic elements.', create_genre(:description => 'An anime featuring gothic elements.').description
  end
  
  def test_merge_into
    # Merging into should delete the anime continuum, merge its names into the
	# other, and reassign all other anime series tied to it.
	genre = create_genre(:name => 'Humor')
	genre.save
	
	series = AnimeSeries.new({:name => 'Minami-ke'})
	series.save
	series.anime_genres << genre
	series.save
	
	assert !AnimeGenre.find_by_name('Humor').nil?
	assert AnimeGenre.find_by_name('Humor').anime_series.include?(series)
	assert !AnimeGenre.find_by_name('Comedy').anime_series.include?(series)
	
	# Let's try merging.
	genre.merge_into!(AnimeGenre.find_by_name('Comedy'))
	
	assert_nil AnimeGenre.find_by_name('Humor')
	
	genre = AnimeGenre.find_by_name('Comedy')
	assert genre.anime_series.include?(series)
	assert AnimeSeries.find_by_name('Minami-ke').anime_genres.include?(genre)
  end

protected
  def create_genre(options = {})
    AnimeGenre.new({ :name => 'Gothic' }.merge(options))
  end
end
