require File.dirname(__FILE__) + '/../test_helper'

class VideoGameGenreTest < ActiveSupport::TestCase
  Error = ''
#  fixtures :video_game_series
  fixtures :video_game_genres
  fixtures :users
  
  def test_validations
    assert !create_genre(:name => nil).valid?
    assert !create_genre(:name => '').valid?
    
    assert_valid create_genre()
    assert_nil create_genre().description
    assert_valid create_genre(:description => 'An video game featuring gothic elements.')
    assert_equal 'An video game featuring gothic elements.', create_genre(:description => 'An video game featuring gothic elements.').description
  end
  
  def test_parenthood
    genre_parent = create_genre(:name => 'a')
    genre_parent.save
    
    genre_child = create_genre(:name => 'b')
    genre_child.save
    genre_child.parent = genre_parent
    genre_child.save
    
    genre_parent = VideoGameGenre.find_by_name('a')
    genre_child = VideoGameGenre.find_by_name('b')
    assert_equal genre_parent, genre_child.parent
    assert_equal 1, genre_parent.children.length
    assert genre_parent.children.include?(genre_child)
    
    genre_parent = VideoGameGenre.find_by_name('RPG')
    genre_child = VideoGameGenre.find_by_name('Strategy RPG')
    assert_equal genre_parent, genre_child.parent
    assert_equal 2, genre_parent.children.length
    assert genre_parent.children.include?(genre_child)
  end
  
  def test_merge_into
    # Merging into should delete the video_game continuum, merge its names into the
    # other, and reassign all other video_game series tied to it.
    genre = create_genre(:name => 'Role Playing')
    genre.save
    
#    series = VideoGameSeries.new({:name => 'Minami-ke'})
#    series.save
#    series.video_game_genres << genre
#    series.save
    
    assert !VideoGameGenre.find_by_name('Role Playing').nil?
#    assert VideoGameGenre.find_by_name('Role Playing').video_game_series.include?(series)
#    assert !VideoGameGenre.find_by_name('RPG').video_game_series.include?(series)
    
    # Let's try merging.
    genre.merge_into!(VideoGameGenre.find_by_name('RPG'))
    
    assert_nil VideoGameGenre.find_by_name('Role Playing')
    
    genre = VideoGameGenre.find_by_name('RPG')
#    assert genre.video_game_series.include?(series)
#    assert VideoGameSeries.find_by_name('Minami-ke').video_game_genres.include?(genre)
  end

protected
  def create_genre(options = {})
    VideoGameGenre.new({ :name => 'Gothic' }.merge(options))
  end
end
