require File.dirname(__FILE__) + '/../test_helper'

class VideoGameSceneTest < ActiveSupport::TestCase
  Error = ''
  fixtures :video_game_scenes
  fixtures :video_games
  fixtures :users
  
#  def test_validations
#    assert !create_scene(:video_game_scene => nil).valid?
#    
#    assert_valid create_scene()
#  end
  
  def test_comments
    # Test adding a comment.
    comment = Comment.new(:text => 'Hey there.  I like this series.')
    child = Comment.new(:text => 'So do I.')
    scene = video_game_scenes(:minus_world)
    old_comments = scene.comments.clone
    
    assert scene.comments.empty?
    assert_difference 'scene.comments.size' do
      scene.comments << comment
    end
    assert_valid scene
    scene.save
    assert scene.comments.include?(comment)
    assert_equal comment, scene.comments[-1]
    assert_equal old_comments + [comment], scene.comments
    assert_nil comment.parent
    assert_equal scene, comment.item
    assert_nothing_raised { Comment.find(comment.id) }
    
    # Test adding a child comment.
    # NOTE: Can't save series and have ALL children saved.
    assert_no_difference 'scene.comments.size' do
      child = comment.children.create(:text => 'So do I.')
    end
    child.save
    assert_equal old_comments + [comment], scene.comments
    # Uh, no way to make sure we can find it...
    #assert_nothing_raised { Comment.find(child.id) }
    #assert_equal child, Comment.find(child.id)
    
    # Test deleting a comment
    assert_difference 'scene.comments.size', -1 do
      scene.comments.delete(comment)
    end
    assert_equal old_comments, scene.comments
    assert_raise ActiveRecord::RecordNotFound do; Comment.find(comment.id); end
    assert_raise ActiveRecord::RecordNotFound do; Comment.find(child.id); end
    
    # Test deleting all comments
    assert_nothing_raised { scene.comments.delete_all }
    assert scene.comments.empty?
    assert_equal [], scene.comments
  end
  
  # TODO: Test VideoGame
  
  def test_state_transitions
    # Here we test a legitimate lifecycle of several trivia questions.
    set_current_user :admin
    
    # Can't modify something that is pending_changes, but should -> approve -> changed
    scene = video_game_scenes(:green_hill)
    old_scene = scene.clone
    
    assert scene.pending_changes?
    scene.changed!
    assert scene.pending_changes?
    scene.approve!
    assert scene.pending_changes?
    
    # TODO: assert change in genres.
    assert_equal '/uploads/video_game_scenes/2/original/green_hill.png', scene.video_game_scene.url
    assert_equal 'Green Hill Zone, Act 2', scene.location
    assert_not_nil scene.pending_changes
    
    # Should throw an error because scene.location has a conflict.
    assert scene.errors.empty?
    assert !scene.approvable_changes?
    assert !scene.errors.empty?
    assert_equal "has more than one pending change which must be resolved prior to approving all changes.", scene.errors[:location]
    scene.errors.clear
    assert scene.errors.empty?
    scene.approve_changes!
    assert !scene.errors.empty?
    assert_equal "has more than one pending change which must be resolved prior to approving all changes.", scene.errors[:location]
    assert_equal '/uploads/video_game_scenes/2/original/green_hill.png', scene.video_game_scene.url
    assert_equal 'Green Hill Zone, Act 2', scene.location
    assert_not_nil scene.pending_changes
    
    # Removing the extra year should resolve the conflict.
    scene.delete_pending_change(:location, :change, "Green Hill")
    assert scene.approvable_changes?
    scene.approve_changes!
    assert scene.approved?
    assert_equal '/uploads/video_game_scenes/2/original/green_hill.png', scene.video_game_scene.url
    assert_equal 'Green Hill Zone', scene.location
    assert_nil scene.pending_changes
    
    # This test is now irrelevant. We 'change' without a transition.
    #series.changed!
    #assert series.pending_changes?
    
    assert_equal old_scene.creator, scene.creator
    assert_equal old_scene.created_at, scene.created_at
    assert_equal old_scene.approver, scene.approver
    assert_equal old_scene.approved_at, scene.approved_at
    assert_not_equal old_scene.updater, scene.updater
    assert_not_equal old_scene.updated_at, scene.updated_at
    
    # Can't approve something that is approved, but should -> changed -> approve
    scene = video_game_scenes(:minus_world)
    old_scene = scene.clone
    assert scene.approved?
    scene.approve!
    assert scene.approved?
    scene.approve_changes!
    assert scene.approved?
    # This test now irrelevant.  We 'change' without a transition.
    #scene.changed!
    #assert scene.pending_changes?
    #scene.approve_changes!
    #assert scene.approved?
    
    assert_equal old_scene.creator, scene.creator
    assert_equal old_scene.created_at, scene.created_at
    assert_equal old_scene.approver, scene.approver
    assert_equal old_scene.approved_at, scene.approved_at
    # These two are irrelevant, as we now have never changed state.
    #assert_not_equal old_scene.updater, scene.updater
    #assert_not_equal old_scene.updated_at, scene.updated_at
    
    # Can't change something that is pending_approval, but should -> approve -> changed
    scene = video_game_scenes(:okami)
    old_scene = scene.clone
    assert scene.pending_approval?
    scene.changed!
    assert scene.pending_approval?
    scene.approve_changes!
    assert scene.pending_approval?
    scene.approve!
    assert scene.approved?
    # This test now irrelevant.  We 'change' without a transition.
    #scene.changed!
    #assert scene.pending_changes?
    #scene.approve_changes!
    #assert scene.approved?
    
    assert_equal old_scene.creator, scene.creator
    assert_equal old_scene.created_at, scene.created_at
    assert_not_equal old_scene.approver, scene.approver
    assert_not_equal old_scene.approved_at, scene.approved_at
    assert_not_equal old_scene.updater, scene.updater
    assert_not_equal old_scene.updated_at, scene.updated_at
    
    # And new trivia questions should be pending_approval.
#    scene = create_scene()
#    scene.save
#    old_scene = scene.clone
#    assert_equal users(:admin), scene.creator
    
#    set_current_user :quentin
    
#    assert scene.pending_approval?
#    scene.changed!
#    assert scene.pending_approval?
#    scene.approve_changes!
#    assert scene.pending_approval?
#    scene.approve!
#    assert scene.approved?
    # This test now irrelevant.  We 'change' without a transition.
    #scene.changed!
    #assert scene.pending_changes?
    #scene.approve_changes!
    #assert scene.approved?
    
#    assert_equal old_scene.creator, scene.creator
#    assert_equal old_scene.created_at, scene.created_at
#    assert_not_equal old_scene.approver, scene.approver
#    assert_not_equal old_scene.approved_at, scene.approved_at
#    assert_not_equal old_scene.updater, scene.updater
#    assert_not_equal old_scene.updated_at, scene.updated_at
    
#    assert_equal users(:quentin), scene.approver
    
    # TODO: Test that approver_id, approved_at are set
  end
  
  def test_pending_changes_non_moderator
    set_current_user :quentin
    
    scene = video_game_scenes(:minus_world)
    
    assert_equal "The Minus World", scene.location
    assert_nil scene.pending_changes
    assert scene.approved?
    
    scene.add_pending_change_support :location, :change, 'Sommat else'
    
    assert scene.pending_changes?
    assert_equal "The Minus World", scene.location
    assert_not_nil scene.pending_changes
    assert_not_nil scene.pending_changes[:location]
    assert_not_nil scene.pending_changes[:location]['Sommat else']
    assert scene.pending_changes[:location]['Sommat else'].include?(users(:quentin).id)
    
    scene.pending_changes = nil
    
    assert_nil scene.pending_changes
    assert scene.approved?
    
    # Also, test that adding pending changes to something pending approval does
    # nothing to the state.
    
    scene = video_game_scenes(:okami)
    
    assert_nil scene.pending_changes
    assert scene.pending_approval?
    assert_equal "midboss", scene.location
    
    scene.add_pending_change_support :location, :change, "Midboss"
    
    assert scene.pending_approval?
    assert_equal "midboss", scene.location
    assert_not_nil scene.pending_changes
    assert_not_nil scene.pending_changes[:location]
    assert_not_nil scene.pending_changes[:location]["Midboss"]
    assert scene.pending_changes[:location]["Midboss"].include?(users(:quentin).id)
    
    # TODO: Test remove pending changes.
  end
  
  def test_pending_changes_moderator
    set_current_user :quentin
    
    user = users(:quentin)
    user.roles << roles(:video_game_scene_moderator)
    user.save
    
    scene = video_game_scenes(:minus_world)
    
    assert_equal "The Minus World", scene.location
    assert_nil scene.pending_changes
    assert scene.approved?
    
    scene.location = 'Sommat else'
    
    assert scene.approved?
    assert_equal 'Sommat else', scene.location
    assert_nil scene.pending_changes
    
    # Also, test that adding pending changes to something pending approval does
    # nothing to the state.
    
    scene = video_game_scenes(:okami)
    
    assert_nil scene.pending_changes
    assert scene.pending_approval?
    assert_equal "midboss", scene.location
    
    scene.location = "Midboss"
    
    assert scene.pending_approval?
    assert_equal "Midboss", scene.location
    assert_nil scene.pending_changes
    
    # TODO: Alternate names, genres
    # TODO: Test remove pending changes.
  end

protected
  def create_scene(options = {})
    VideoGameScene.new({ :video_game => VideoGame.find_by_name('Super Mario Brothers'), :video_game_scene => nil, :location => 'End of game' }.merge(options))
  end
  
  def set_current_user(user_key)
    User.stamper = users(user_key)
  end
end
